@@ -1,6 +1,5 @@
 /* Commands for the curses interface to Xconq.
-   Copyright (C) 1986, 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
-   Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -8,8 +7,7 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
-extern void update_debugging PARAMS ((void));
-extern void toggle_debugging PARAMS ((int *flagp));
+#include "kpublic.h"
 #include "cconq.h"
 extern int autofinish_start;
 extern int autofinish_count;
@@ -34,8 +32,6 @@ static void execute_named_command PARAMS ((char *cmdstr));
 static int execute_command_from_table PARAMS ((CmdTab *cmdtab));
 static int execute_named_command_from_table PARAMS ((char *cmdstr,
 						    CmdTab *cmdtab));
-static void describe_command_table PARAMS ((int arg, char *key, char *buf,
-					   CmdTab *cmdtab));
 static void cmd_error PARAMS ((char *fmt, ...));
 
 /* Declarations of all the command functions. */
@@ -121,9 +117,10 @@ static void
 execute_named_command(cmdstr)
 char *cmdstr;
 {
+    int prefix;
     char *cmdname;
 
-    parse_long_name_command(cmdstr, &cmdname, &cmdargstr, copy_string(cmdstr)); 
+    parse_long_name_command(cmdstr, &prefix, &cmdname, &cmdargstr, copy_string(cmdstr)); 
     if (empty_string(cmdname)) {
 	notify(dside, "No command.");
     }
@@ -164,33 +161,40 @@ CmdTab *cmdtab;
 void
 describe_commands(arg, key, buf)
 int arg;
-char *key, *buf;
-{
-    strcat(buf, "To move a unit, use [hlyubn]\n");
-    strcat(buf, "[HLYUBN] moves unit repeatedly in that direction\n");
-    strcat(buf, "To look at another unit, use survey mode ('z')\n");
-    strcat(buf, "and use [hlyubnHLYUBN] to move the cursor\n");
-    strcat(buf, "\n");
-    strcat(buf, "Generic commands:\n");
-    describe_command_table(arg, key, buf, commands);
-    strcat(buf, "Cconq-specific commands:\n");
-    describe_command_table(arg, key, buf, ccommands);
-}
-
-static void
-describe_command_table(arg, key, buf, cmdtab)
-int arg;
-char *key, *buf;
-CmdTab *cmdtab;
+char *key;
+TextBuffer *buf;
 {
     CmdTab *cmd;
 
-    strcat(buf, "Single-key commands:\n\n");
-    for (cmd = cmdtab; cmd->name != NULL; ++cmd) {
-	describe_command (cmd->fchar, cmd->name, cmd->help, TRUE, buf);
+    tbcat(buf, "To move a unit, use [hjklyubn]\n");
+    tbcat(buf, "[HJKLYUBN] moves unit repeatedly in that direction\n");
+    tbcat(buf, "To look at another unit, use survey mode ('z')\n");
+    tbcat(buf, "and use [hjklyubnHJKLYUBN] to move the cursor\n");
+    tbcat(buf, "\n");
+    tbcat(buf, "Single-key commands:\n\n");
+    for (cmd = commands; cmd->name != NULL; ++cmd) {
+	describe_command(cmd->fchar, cmd->name, cmd->help, TRUE, buf);
     }
-    strcat(buf, "\nLong name commands:\n\n");
-    for (cmd = cmdtab; cmd->name != NULL; ++cmd) {
+    tbcat(buf, "\n\nSingle-key commands (curses-only):\n\n");
+    for (cmd = ccommands; cmd->name != NULL; ++cmd) {
+	describe_command(cmd->fchar, cmd->name, cmd->help, TRUE, buf);
+    }
+}
+
+void
+describe_long_commands(arg, key, buf)
+int arg;
+char *key;
+TextBuffer *buf;
+{
+    CmdTab *cmd;
+
+    tbcat(buf, "\nText commands:\n\n");
+    for (cmd = commands; cmd->name != NULL; ++cmd) {
+	describe_command (cmd->fchar, cmd->name, cmd->help, FALSE, buf);
+    }
+    tbcat(buf, "\nText commands (curses-only):\n\n");
+    for (cmd = ccommands; cmd->name != NULL; ++cmd) {
 	describe_command (cmd->fchar, cmd->name, cmd->help, FALSE, buf);
     }
 }
@@ -222,7 +226,7 @@ find_next_and_look()
 void
 do_add_player()
 {
-    request_additional_side(cmdargstr);
+    net_request_additional_side(cmdargstr);
 }
 
 void
@@ -238,7 +242,7 @@ do_add_terrain()
 		&& curunit->act
 		&& curunit->act->acp >= ut_acp_to_add_terrain(u, t)) {
 		if (0 <= ut_alter_range(curunit->type, t)) {
-		    if (prep_add_terrain_action(curunit, curunit,
+		    if (net_prep_add_terrain_action(curunit, curunit,
 						curunit->x, curunit->y,
 						dir, t))
 		      ;
@@ -256,9 +260,9 @@ void
 do_ai_side()
 {
     if (side_has_ai(dside)) {
-	set_side_ai(dside, NULL);
+	net_set_side_ai(dside, NULL);
     } else {
-	set_side_ai(dside, "mplayer");
+	net_set_side_ai(dside, "mplayer");
     }
 }
 
@@ -273,7 +277,7 @@ do_attack()
 	for_all_stack(x, y, other) {
 	    if (!unit_trusts_unit(curunit, other)) {
 		if (valid(check_attack_action(curunit, curunit, other, 100))) {
-		    prep_attack_action(curunit, curunit, other, 100);
+		    net_prep_attack_action(curunit, curunit, other, 100);
 		    return;
 		}
 		/* (should try other types of actions?) */
@@ -291,7 +295,7 @@ do_auto()
 {
     REQUIRE_UNIT();
     if (curunit->plan) {
-	set_unit_ai_control(dside, curunit, !curunit->plan->aicontrol, FALSE);
+	net_set_unit_ai_control(dside, curunit, !curunit->plan->aicontrol, FALSE);
 	/* a hack */
 	curunit->plan->waitingfortasks = !curunit->plan->aicontrol;
     }
@@ -356,7 +360,7 @@ int u2, n;
       n = 99;
     notify(side, "%s will build %d %s",
 	   unit_handle(side, unit), n, u_type_name(u2));
-    push_build_task(unit, u2, n);
+    net_push_build_task(unit, u2, n);
 }
 
 void
@@ -364,7 +368,7 @@ do_clear_plan()
 {
     REQUIRE_UNIT();
     if (curunit->plan) {
-	set_unit_plan_type(dside, curunit, PLAN_NONE);
+	net_set_unit_plan_type(dside, curunit, PLAN_NONE);
     }
 }
 
@@ -400,7 +404,7 @@ do_detach()
 	return;
     }
     if (valid(check_transfer_part_action(unit, unit, unit->hp / 2, NULL))) {
-	prep_transfer_part_action(unit, unit, unit->hp / 2, NULL);
+	net_prep_transfer_part_action(unit, unit, unit->hp / 2, NULL);
     } else {
 	cmd_error("cannot detach right now");
     }
@@ -412,7 +416,7 @@ do_detonate()
     Unit *unit = curunit;
 
     REQUIRE_UNIT();
-    prep_detonate_action(unit, unit, unit->x, unit->y, unit->z);
+    net_prep_detonate_action(unit, unit, unit->x, unit->y, unit->z);
 }
 
 /* Supposedly you could only get to these by typing the full command names. */
@@ -420,7 +424,15 @@ do_detonate()
 void
 do_dir()
 {
-    cmd_error("use the single-character commands instead");
+    int ndirs, dir1, dir2, modif;
+
+    ndirs = char_to_dir(tmpkey, &dir1, &dir2, &modif);
+    if (ndirs >= 1) {
+	do_dir_2(dir1, prefixarg);
+    } else {
+	xbeep();
+	return;
+    }
 }
 
 void
@@ -434,27 +446,12 @@ do_dir_multiple()
 void
 do_disband()
 {
-    int u;
+    int rslt;
 
     REQUIRE_UNIT();
-    u = curunit->type;
-#ifdef DESIGNERS
-    if (dside->designer) {
-	kill_unit(curunit, -1);
-	return;
-    }
-#endif /* DESIGNERS */
-    if (!completed(curunit)) {
-	kill_unit(curunit, H_UNIT_DISBANDED);
-    } else if (u_hp_per_disband(u) > 0) {
-	if (prep_disband_action(curunit, curunit)) {
-	    /* (text should come from game design) */
-	    notify(dside, "%s will go home.", unit_handle(dside, curunit));
-	} else {
-	    /* failed, should say something */
-	}
-    } else {
-	cmd_error("You can't just get rid of %s!",
+    rslt = net_disband_unit(dside, curunit);
+    if (!rslt) {
+	cmd_error("You can't get rid of %s!",
 		  unit_handle(dside, curunit));
     }
 }
@@ -477,14 +474,14 @@ do_disembark()
     /* Try moving into the transport's transport, if there is one. */
     if (transport->transport != NULL
         && can_occupy(curunit, transport->transport)) {
-	prep_enter_action(curunit, curunit, transport->transport);
+	net_prep_enter_action(curunit, curunit, transport->transport);
 	/* (should be able to set up task if can't do action immedly) */
 	return;
     }
     /* Try moving into the open in the cell. */
     if (can_occupy_cell(curunit, curunit->x, curunit->y)
 	|| can_occupy_conn(curunit, curunit->x, curunit->y, curunit->z)) {
-	prep_move_action(curunit, curunit, curunit->x, curunit->y, curunit->z);
+	net_prep_move_action(curunit, curunit, curunit->x, curunit->y, curunit->z);
 	/* (should be able to set up task if can't do action immedly) */
 	return;
     }
@@ -511,19 +508,39 @@ do_distrust()
     if (cmdargstr) {
 	side2 = parse_side_spec(cmdargstr);
 	if (side2 != NULL && side2 != dside) {
-	    set_trust(dside, side2, 0);
+	    net_set_trust(dside, side2, 0);
 	    return;
 	} 
     }
     cmd_error("no side to distrust");
 }
 
+void
+do_doctrine()
+{
+  /* (should make generic) */
+  if (cmdargstr) {
+    if (strcmp(cmdargstr, "set") == 0) {
+      notify(dside, "Set what?");
+    } else if (strncmp(cmdargstr, "set ", 4) == 0) {
+      net_set_doctrine(dside, cmdargstr + 4);
+    }
+    return;
+  }
+  notify_doctrine(dside, (cmdargstr ? cmdargstr : ""));
+}
+
+void
+do_down()
+{
+}
+
 void
 do_draw_willingness()
 {
     if (prefixarg < 0)
       prefixarg = 1;
-    set_willing_to_draw(dside, (prefixarg ? 1 : 0));
+    net_set_willing_to_draw(dside, (prefixarg ? 1 : 0));
 }
 
 /* What about trying to embark a unit on itself or on its previous transp? */
@@ -544,7 +561,7 @@ do_embark()
 	    cmd_error("Can't occupy chosen transport");
 	    return;
 	}
-	prep_enter_action(curunit, curunit, transport);
+	net_prep_enter_action(curunit, curunit, transport);
 	/* (should be able to set up task if can't do action immedly) */
     } else {
 	xbeep();
@@ -556,7 +573,7 @@ do_embark()
 void
 do_end_turn()
 {
-    finish_turn(dside);
+    net_finish_turn(dside);
 }
 
 /* Command to fire at a specified unit or location. */
@@ -574,14 +591,14 @@ do_fire()
 	if (units_visible(dside, x, y)) {
 	    for_all_stack(x, y, unit2) {
 		if (unit2->side != curunit->side) {
-		    prep_fire_at_action(curunit, curunit, unit2, -1);
+		    net_prep_fire_at_action(curunit, curunit, unit2, -1);
 		    return;
 		}
 	    }
 	}
 	/* (should say that nothing is visible and verify firing) */
 	if (1) {
-	    prep_fire_into_action(curunit, curunit, x, y, 0, -1);
+	    net_prep_fire_into_action(curunit, curunit, x, y, 0, -1);
 	} else {
 	    xbeep();
 	}
@@ -598,7 +615,7 @@ do_fire_into()
     sprintf(spbuf, "Fire %s at where?", unit_handle(dside, curunit));
     /* (should have some sort of range feedback) */
     if (ask_position(spbuf, &x, &y)) {
-	prep_fire_into_action(curunit, curunit, x, y, 0, -1);
+	net_prep_fire_into_action(curunit, curunit, x, y, 0, -1);
     }
 }
 
@@ -662,22 +679,17 @@ do_give()
 void
 do_give_unit()
 {
-    int u;
-
     REQUIRE_UNIT();
-    u = curunit->type;
 #ifdef DESIGNERS
     if (dside->designer) {
-	change_unit_side(curunit, side_n(prefixarg), -1, NULL);
-	update_unit_display(dside, curunit, TRUE);
-	all_see_cell(curunit->x, curunit->y);
+	net_designer_change_side(curunit, side_n(prefixarg));
 	return;
     }
 #endif /* DESIGNERS */
     if (1) { /* (should test both temporary and permanent invalidity) */
-	prep_change_side_action(curunit, curunit, side_n(prefixarg));
+	net_prep_change_side_action(curunit, curunit, side_n(prefixarg));
     } else {
-	cmd_error("You can't just give away the %s!", u_type_name(u));
+	cmd_error("You can't just give away the %s!", unit_handle(dside, curunit));
     }
 }
 
@@ -693,6 +705,17 @@ do_help()
     refresh();
 }
 
+/* Tell the unit to sit around for a given number of turns. */
+
+void
+do_idle()
+{
+    REQUIRE_UNIT();
+    if (prefixarg < 0)
+      prefixarg = 9999;
+    net_set_sentry_task(curunit, prefixarg);
+}
+
 /* Send a short message to another side. */
 
 void
@@ -718,7 +741,7 @@ do_message()
 	    sidemask = add_side_to_set(side2, NOSIDES);
 	}
 	if (!empty_string(msg) && sidemask != NOSIDES)
-	  send_message(dside, sidemask, msg);
+	  net_send_message(dside, sidemask, msg);
     }
 }
 
@@ -734,12 +757,12 @@ do_move_to()
     if (ask_position(spbuf, &x, &y)) {
 #ifdef DESIGNERS
 	if (dside->designer) {
-	    designer_teleport(curunit, x, y, NULL);
+	    net_designer_teleport(curunit, x, y, NULL);
 	    make_current(curunit);
 	    return;
 	}
 #endif /* DESIGNERS */
-	set_move_to_task(curunit, x, y);
+	net_set_move_to_task(curunit, x, y);
     }
 }
 
@@ -754,7 +777,7 @@ do_name()
     if (ask_string("New name for unit:", curunit->name, &newname)) {
 	if (empty_string(newname))
 	  newname = NULL;
-	set_unit_name(dside, curunit, newname);
+	net_set_unit_name(dside, curunit, newname);
     }
 }
 
@@ -811,7 +834,7 @@ do_produce()
     /* Find the first produceable type and set up to produce it. */
     for_all_material_types(m) {
 	if (um_acp_to_produce(unit->type, m) > 0) {
-	    push_produce_task(unit, m, n);
+	    net_push_produce_task(unit, m, n);
 	    return;
 	}
     }
@@ -834,14 +857,14 @@ do_quit()
 		exit_cconq();
 		return;
 	    } else {
-		set_willing_to_save(dside, TRUE);
+		net_set_willing_to_save(dside, TRUE);
 		notify(dside, "Other sides not willing to save.");
 	    }
 	}
     }
     if (all_others_willing_to_quit(dside)) {
 	if (ask_bool("Do you really want to declare a draw?", FALSE)) {
-	    set_willing_to_draw(dside, TRUE);
+	    net_set_willing_to_draw(dside, TRUE);
 	} else {
 	    notify(dside, "Not willing to draw.");
 	}
@@ -876,6 +899,12 @@ do_refresh()
     redraw();
 }
 
+void
+do_release()
+{
+    cmd_error("can't release anybody yet");
+}
+
 void
 do_remove_terrain()
 {
@@ -888,7 +917,7 @@ do_remove_terrain()
 	    && curunit->act
 	    && curunit->act->acp >= ut_acp_to_remove_terrain(curunit->type, t)) {
 	    if (0 <= ut_alter_range(curunit->type, t)) {
-		if (prep_remove_terrain_action(curunit, curunit, curunit->x, curunit->y, dir, t))
+		if (net_prep_remove_terrain_action(curunit, curunit, curunit->x, curunit->y, dir, t))
 		  ;
 		else
 		  xbeep();
@@ -898,11 +927,17 @@ do_remove_terrain()
     }
 }
 
+void
+do_repair()
+{
+  cmd_error("repair command not implemented");
+}
+
 void
 do_reserve()
 {
     REQUIRE_UNIT();
-    set_unit_asleep(dside, curunit, TRUE, TRUE);
+    net_set_unit_asleep(dside, curunit, TRUE, TRUE);
 }
 
 void
@@ -923,7 +958,7 @@ do_resign()
 		side2 = NULL;
 	    }
 	}
-	resign_game(dside, side2);
+	net_resign_game(dside, side2);
     }
 }
 
@@ -933,7 +968,7 @@ void
 do_return()
 {
     REQUIRE_UNIT();
-    set_resupply_task(curunit, NONMTYPE);
+    net_set_resupply_task(curunit, NONMTYPE);
 }
 
 /* Stuff game state into a file.  By default, it goes into the current
@@ -1004,7 +1039,7 @@ do_set_formation()
 	} else if (leader->side != dside /* or "trusted side"? */) {
 	    cmd_error("Can't follow somebody else's unit!");
 	} else {
-	    set_formation(curunit, leader, curunit->x - leader->x, curunit->y - leader->y, 1, 1);
+	    net_set_formation(curunit, leader, curunit->x - leader->x, curunit->y - leader->y, 1, 1);
 	}
     }
 }
@@ -1026,7 +1061,7 @@ void
 do_sleep()
 {
     REQUIRE_UNIT();
-    set_unit_asleep(dside, curunit, TRUE, TRUE);
+    net_set_unit_asleep(dside, curunit, TRUE, TRUE);
 }
 
 void
@@ -1043,9 +1078,18 @@ do_standing_orders()
 }
 
 void
-do_surrender_to()
+do_submit()
 {
-    cmd_error("can't surrender to anybody yet");
+    Side *side2;
+
+    if (cmdargstr) {
+	side2 = parse_side_spec(cmdargstr);
+	if (side2 != NULL && side2 != dside) {
+	    net_set_controlled_by(dside, side2, TRUE);
+	    return;
+	} 
+    }
+    xbeep();
 }
 
 /* Command to toggle between interaction modes. */
@@ -1092,12 +1136,6 @@ do_take()
     }
 }
 
-void
-do_take_unit()
-{
-    cmd_error("can't take units yet");
-}
-
 void
 do_trust()
 {
@@ -1106,13 +1144,27 @@ do_trust()
     if (cmdargstr) {
 	side2 = parse_side_spec(cmdargstr);
 	if (side2 != NULL && side2 != dside) {
-	    set_trust(dside, side2, 1);
+	    net_set_trust(dside, side2, 1);
 	    return;
 	} 
     }
     cmd_error("no side to trust");
 }
 
+void
+do_up()
+{
+}
+
+/* Display the program version. */
+
+void
+do_version()
+{
+    notify(dside, "Curses Xconq version %s", version_string());
+    notify(dside, "(c) %s", copyright_string());
+}
+
 /* Wake *everything* (that's ours) within the given radius.  Two commands
    actually; "top-level" units (not in a transport) vs all units. */
 
@@ -1121,7 +1173,7 @@ do_trust()
 void
 do_wake()
 {
-    wake_area(dside, curx, cury, prefixarg, FALSE);
+    net_wake_area(dside, curx, cury, prefixarg, FALSE);
 }
 
 /* Wake all units found. */
@@ -1129,16 +1181,7 @@ do_wake()
 void
 do_wake_all()
 {
-    wake_area(dside, curx, cury, prefixarg, TRUE);
-}
-
-/* Display the program version. */
-
-void
-do_version()
-{
-    notify(dside, "Curses Xconq version %s", version_string());
-    notify(dside, "(c) %s", copyright_string());
+    net_wake_area(dside, curx, cury, prefixarg, TRUE);
 }
 
 void
@@ -1227,7 +1270,7 @@ do_c_set_info_lines()
     mh = LINES - 2 - infoh - 1;
     closeupwin->h = infoh;
     mapwin->y = 2 + infoh;
-    mapwin->h = mh;
+    mapwin->h = mh + 1;
     /* Update the screen to reflect the changes. */
     set_scroll();
     redraw();
@@ -1242,7 +1285,7 @@ do_c_run()
     notify(dside, "Running free for %d turn%s.",
 	   turns, (turns == 1 ? "" : "s"));
     if (turns > 0) {
-	set_autofinish(dside, TRUE);
+	net_set_autofinish(dside, TRUE);
 	autofinish_start = g_turn();
 	autofinish_count = turns;
     }
@@ -1333,9 +1376,9 @@ void
 do_design()
 {
     if (!dside->designer) {
-	become_designer(dside);
+	net_become_designer(dside);
     } else {
-	become_nondesigner(dside);
+	net_become_nondesigner(dside);
     }
 }
 
@@ -1343,7 +1386,7 @@ void
 do_gdl()
 {
     if (cmdargstr)
-      interp_form(NULL, read_form_from_string(cmdargstr, NULL, NULL));
+      interp_form(NULL, read_form_from_string(cmdargstr, NULL, NULL, NULL));
 }
 
 void
@@ -1370,7 +1413,7 @@ do_c_add_unit()
 
     if (!check_designer_status("create units"))
       return;
-    unit = designer_create_unit(dside, curutype, cursidenumber, curx, cury);
+    unit = net_designer_create_unit(dside, curutype, cursidenumber, curx, cury);
     if (unit != NULL) {
 	make_current(unit);
     } else {
@@ -1408,7 +1451,7 @@ do_c_paint_terrain()
     /* If command's arg is nonegative, interpret as temporary
        change of terrain type. */
     t = (prefixarg >= 0 ? prefixarg : curttype);
-    paint_cell(dside, curx, cury, curradius, t);
+    net_paint_cell(dside, curx, cury, curradius, t);
 }
 
 /* (should add painting for all other layers here) */
