@@ -1,5 +1,5 @@
 /* Main include file for Xconq.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1991-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,25 +23,10 @@ any later version.  See the file COPYING.  */
 #include "history.h"
 #include "score.h"
 #include "help.h"
+#include "tp.h"
 #include "ai.h"
 #include "system.h"
 
-/* (should be in a movies.h?) */
-
-enum movie_type {
-  movie_null,
-  movie_miss,
-  movie_hit,
-  movie_death,
-  movie_nuke,
-  movie_extra_0
-};
-
-typedef struct a_movie_type {
-  enum movie_type type;
-  char *name;
-} MovieType;
-
 /* Declarations of globally visible globals. */
 
 extern int typesdefined;
@@ -53,7 +38,7 @@ extern int endofgame;
 extern int gamestatesafe;
 extern int interfacestatesafe;
 extern int any_post_action_scores;
-extern int any_people_side_changes;
+extern short any_people_side_changes;
 extern int max_zoc_range;
 extern int warnings_suppressed;
 extern time_t game_start_in_real_time;
@@ -61,6 +46,28 @@ extern time_t turn_play_start_in_real_time;
 
 extern short *u_possible;
 
+extern int checkpoint_interval;
+extern int statistics_wanted;
+extern int numsoundplays;
+
+extern int allbedesigners;
+
+extern int tmphevtdata1;
+extern int max_detonate_on_approach_range;
+extern int max_u_detonate_effect_range;
+extern int max_t_detonate_effect_range;
+
+extern int xmalloc_warnings;
+extern int memory_exhausted;
+extern Feature *featurelist;
+extern Doctrine *doctrine_list;
+
+extern char *curdatestr;
+
+extern int sunx, suny;
+
+extern short daynight;
+
 /* Declarations of functions that must be supplied by an interface. */
 
 extern void announce_read_progress PARAMS ((void));
@@ -90,11 +97,9 @@ extern void update_all_progress_displays PARAMS ((char *str, int s));
 extern void update_everything PARAMS ((void));
 extern void flush_display_buffers PARAMS ((Side *side));
 
-extern int schedule_movie PARAMS ((Side *side, enum movie_type movie, ...));
+extern int schedule_movie PARAMS ((Side *side, char *movie, ...));
 extern void play_movies PARAMS ((SideMask sidemask));
 
-extern void ui_save_state PARAMS ((Side *side));
-
 extern void action_point PARAMS ((Side *side, int x, int y));
 
 extern void notify_all PARAMS ((char *str, ...));
@@ -120,6 +125,8 @@ extern void close_displays PARAMS ((void));
 
 /* Declarations of functions not elsewhere declared. */
 
+/* init.c */
+
 extern void init_library_path PARAMS ((char *path));
 extern void init_data_structures PARAMS ((void));
 extern void check_game_validity PARAMS ((void));
@@ -137,7 +144,6 @@ extern int make_weather PARAMS ((int calls, int runs));
 extern void final_init PARAMS ((void));
 extern void configure_sides PARAMS ((void));
 extern void init_all_views PARAMS ((void));
-extern void init_area_views PARAMS ((void));
 extern int make_initial_materials PARAMS ((int calls, int runs));
 extern void init_supply PARAMS ((Unit *unit));
 extern int saved_game PARAMS ((void));
@@ -147,6 +153,9 @@ extern void assign_players_to_sides PARAMS ((void));
 extern char *version_string PARAMS ((void));
 extern char *copyright_string PARAMS ((void));
 extern char *license_string PARAMS ((void));
+extern int already_seen_chance PARAMS ((Side *side, Unit *unit));
+
+/* read.c */
 
 extern void syntax_error PARAMS ((Obj *x, char *msg));
 extern void type_error PARAMS ((Obj *x, char *msg));
@@ -158,17 +167,16 @@ extern void read_warning PARAMS ((char *str, ...));
 extern int utype_from_name PARAMS ((char *str));
 extern int utype_from_symbol PARAMS ((Obj *sym));
 
+/* actions.c */
+
 extern void init_actions PARAMS ((void));
 extern int can_move_via_conn PARAMS ((Unit *unit, int nx, int ny));
 extern int can_occupy_conn PARAMS ((Unit *unit, int nx, int ny, int nz));
-extern int move_unit PARAMS ((Unit *unit, int nx, int ny));
 extern int can_move_at_all PARAMS ((Unit *unit));
 extern int in_blocking_zoc PARAMS ((Unit *unit, int x, int y, int z));
 extern int unit_blockable_by PARAMS ((Unit *unit, Unit *unit2));
 extern int total_move_cost PARAMS ((int u, int u2, int x1, int y1, int z1, int x2, int y2, int z2));
 extern int zoc_move_cost PARAMS ((Unit *unit, int ox, int oy, int oz));
-extern int maybe_react_to_move PARAMS ((Unit *unit, int ox, int oy));
-extern void consume_move_supplies PARAMS ((Unit *unit));
 extern int sides_allow_entry PARAMS ((Unit *unit, Unit *transport));
 extern int total_entry_cost PARAMS ((int u1, int u3, int x1, int y1, int z1, int u2, int x2, int y2, int z2));
 extern int transfer_supply PARAMS ((Unit *from, Unit *to, int m, int amount));
@@ -219,44 +227,43 @@ extern int can_detonate PARAMS ((Unit *unit));
 
 extern int found_blocking_elevation PARAMS ((int u, int ux, int uy, int uz, int u2, int u2x, int u2y, int u2z));
 extern int enough_ammo PARAMS ((Unit *unit, Unit *other));
-extern void reckon_damage PARAMS ((void));
-extern void reckon_damage_around PARAMS ((int x, int y, int r));
-extern void damage_unit PARAMS ((Unit *unit, enum damage_reasons reason));
 extern int capture_chance PARAMS ((int u, int u2, Side *side2));
-extern void capture_unit PARAMS ((Unit *unit, Unit *pris));
-extern int detonate_unit PARAMS ((Unit *unit, int x, int y, int z));
-
 extern int multiply_dice PARAMS ((int dice, int mult));
-extern void damage_terrain PARAMS ((int u, int x, int y));
 
+/* run.c */
+
+extern void init_run PARAMS ((void));
 extern int run_game PARAMS ((int maxactions));
+extern int run_local_ai PARAMS ((int when, int maxplanning));
 extern void test_for_game_start PARAMS ((void));
 extern void test_for_game_end PARAMS ((void));
 extern int damaged_acp PARAMS ((Unit *unit, Obj *effect));
 extern int all_sides_finished PARAMS ((void));
-extern void finish_turn PARAMS ((Side *side));
 extern void check_realtime PARAMS ((void));
 extern int exceeded_rt_for_game PARAMS ((void));
 extern int exceeded_rt_per_turn PARAMS ((void));
 extern void compute_acp PARAMS ((Unit *unit));
 extern int units_still_acting PARAMS ((Side *side));
-extern void resign_game PARAMS ((Side *side, Side *side2));
 extern int realtime_game PARAMS ((void));
 extern int all_others_willing_to_save PARAMS ((Side *side));
 extern int all_others_willing_to_quit PARAMS ((Side *side));
 extern void end_the_game PARAMS ((void));
-extern void change_people_side_around PARAMS ((int x, int y, int u, Side *side));
-extern void init_random_events PARAMS ((void));
-extern void maybe_starve PARAMS ((Unit *unit, int partial));
+
 extern int people_surrender_chance PARAMS ((int u, int x, int y));
 extern int unit_priority PARAMS ((Unit *unit));
 extern int num_people_at PARAMS ((int x, int y));
 
-extern int request_additional_side PARAMS ((char *playerspec));
 extern void parse_player_spec PARAMS ((Player *player, char *spec));
 
 extern int get_synth_method_uses PARAMS ((int methkey, int *calls, int *runs));
 
+/* run2.c */
+
+extern void update_cell_display_all_sides PARAMS ((int x, int y, int rightnow));
+extern void update_side_display_all_sides PARAMS ((Side *side, int rightnow));
+
+/* mkterr.c */
+
 extern int make_fractal_terrain PARAMS ((int calls, int runs));
 extern int make_random_terrain PARAMS ((int calls, int runs));
 extern int make_earthlike_terrain PARAMS ((int calls, int runs));
@@ -264,15 +271,22 @@ extern int make_maze_terrain PARAMS ((int calls, int runs));
 extern int name_geographical_features PARAMS ((int calls, int runs));
 extern void add_edge_terrain PARAMS ((void));
 
+/* mkunits.c */
+
 extern int make_countries PARAMS ((int calls, int runs));
 extern int make_independent_units PARAMS ((int calls, int runs));
 
+/* mkrivers.c */
+
 extern int make_rivers PARAMS ((int calls, int runs));
 
+/* mkroads.c */
+
 extern int make_roads PARAMS ((int calls, int runs));
 
-extern Obj *make_namer PARAMS ((Obj *sym, Obj *meth));
+/* mknames.c */
 
+extern Obj *make_namer PARAMS ((Obj *sym, Obj *meth));
 extern void init_namers PARAMS ((void));
 extern void make_up_side_name PARAMS ((Side *side));
 extern int name_in_use PARAMS ((Side *side, char *str));
@@ -286,14 +300,12 @@ extern char *name_from_grammar PARAMS ((Obj *grammar));
 extern void gen_name PARAMS ((Obj *nonterm, Obj *rules, int depth, char *rslt));
 extern void gen_from_rule PARAMS ((Obj *rule, Obj *rules, int depth, char *rslt));
 
+/* nlang.c */
+
 extern void init_nlang PARAMS ((void));
 extern void pad_out PARAMS ((char *buf, int n));
 extern char *short_side_title PARAMS ((Side *side));
 extern char *shortest_side_title PARAMS ((Side *side2, char *buf));
-extern char *rel_short_side_title PARAMS ((Side *side, Side *side2, int n));
-extern char *rel_long_side_title PARAMS ((Side *side, Side *side2));
-extern char *narrative_side_desc PARAMS ((Side *side, Side *side2));
-extern char *narrative_side_desc_first PARAMS ((Side *side, Side *side2));
 extern char *long_player_title PARAMS ((char *buf, Player *player, char *thisdisplayname));
 extern char *short_player_title PARAMS ((char *buf, Player *player, char *thisdisplayname));
 extern void side_and_type_name PARAMS ((char *buf, Side *side, int u, Side *side2));
@@ -313,10 +325,14 @@ extern char *feature_name_at PARAMS ((int x, int y));
 extern void temperature_desc PARAMS ((char *buf, int x, int y));
 extern void hp_desc PARAMS ((char *buf, Unit *unit, int label));
 extern void acp_desc PARAMS ((char *buf, Unit *unit, int label));
+extern void morale_desc PARAMS ((char *buf, Unit *unit, int label));
 extern void plan_desc PARAMS ((char *buf, Unit *unit));
-extern void task_desc PARAMS ((char *buf, Task *task));
+extern void task_desc PARAMS ((char *buf, Side *side, Task *task));
 extern void time_desc PARAMS ((char *buf, int time, int maxtime));
 extern char *summarize_units PARAMS ((char *buf, int *ucnts));
+extern void notify_doctrine PARAMS ((Side *side, char *spec));
+extern void report_combat PARAMS ((Unit *unit, Unit *atker, char *str));
+extern void report_take PARAMS ((Side *side, Unit *unit, int needed, short *rslts));
 extern char *exit_commentary PARAMS ((Side *side));
 extern char *ordinal_suffix PARAMS ((int n));
 extern char *plural_form PARAMS ((char *word));
@@ -335,14 +351,20 @@ extern int missing_images PARAMS ((char *buf));
 extern void notify_all_of_resignation PARAMS ((Side *side, Side *side2));
 extern int short_side_title_plural_p PARAMS ((Side *side));
 extern char *short_side_title_with_adjective PARAMS ((Side *side, char *adjective));
-
-extern void set_initial_date PARAMS ((char *str));
-extern long parse_date PARAMS ((char *datestr));
-extern int find_event_type PARAMS ((Obj *sym));
 extern char *side_score_desc PARAMS ((char *buf, Side *side, Scorekeeper *sk));
+extern void location_desc PARAMS ((char *buf, Side *side, Unit *unit, int u, int x, int y));
+extern void destination_desc PARAMS ((char *buf, Side *side, int x, int y, int z));
+extern int supply_desc PARAMS ((char *buf, Unit *unit, int mrow));
+extern char *sidemask_desc PARAMS ((char *buf, SideMask sidemask));
+extern void latlong_desc PARAMS ((char *buf, int x, int y, int xf, int yf, int which));
+extern void others_here_desc PARAMS ((char *buf, Unit *unit));
+extern void occupants_desc PARAMS ((char *buf, Unit *unit));
+extern void set_initial_date PARAMS ((char *str));
+extern int find_event_type PARAMS ((Obj *sym));
+
+/* module.c */
 
 extern void doublecheck_state PARAMS ((Side *side));
-extern long extension_value PARAMS ((Obj *extensions, char *key, long dflt));
 extern void set_u_internal_name PARAMS ((int u, char *s));
 extern void set_u_type_name PARAMS ((int u, char *s));
 extern void set_m_type_name PARAMS ((int m, char *s));
@@ -352,10 +374,14 @@ extern Side *coerce_to_side PARAMS ((Obj *x));
 extern int coerce_to_unit_id PARAMS ((Obj *x));
 extern Unit *coerce_to_unit PARAMS ((Obj *x));
 
+/* write.c */
+
 extern void init_write PARAMS ((void));
 extern int write_entire_game_state PARAMS ((char *fname));
 extern int write_game_module PARAMS ((Module *module));
 
+/* (should organize) */
+
 extern int total_gain PARAMS ((Side *side, int u));
 extern int total_loss PARAMS ((Side *side, int u));
 
@@ -372,3 +398,16 @@ extern int get_synth_method_uses PARAMS ((int methkey, int *calls, int *runs));
 extern void set_initial_date PARAMS ((char *str));
 extern int turns_between PARAMS ((char *str1, char *str2));
 extern int visible_to PARAMS ((Unit *unit, Unit *unit2));
+
+extern int construction_possible PARAMS ((int u2));
+extern int can_build_or_help PARAMS ((Unit *unit));
+extern int can_build_or_help PARAMS ((Unit *unit));
+extern int can_produce PARAMS ((Unit *unit));
+extern void add_to_unit_hp PARAMS ((Unit *unit, int hp));
+extern void try_sharing PARAMS ((Unit *from, Unit *to, int m));
+extern int number_member PARAMS ((int x, Obj *lis));
+extern int wind_value PARAMS ((Unit *unit, int angle, int force, Obj *effect, int maxval));
+extern char *shortest_escaped_name PARAMS ((int u));
+
+extern int side_owns_occupant PARAMS ((Side *side, Unit *unit));
+extern char *goal_desc PARAMS ((char *buf, Goal *goal));
