@@ -1,5 +1,5 @@
 /* Interpretation of generic GDL images for Xconq.
-   Copyright (C) 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1994-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -14,13 +14,15 @@ any later version.  See the file COPYING.  */
 #include "lisp.h"
 #include "imf.h"
 
+extern void syntax_error PARAMS ((Obj *x, char *msg));
+
 /* RGB above this value should be considered white. */
 
-#define WHITE_THRESHOLD (65535 - 100)
+#define WHITE_THRESHOLD (65535 - 256)
 
 /* RGB below this value should be considered black. */
 
-#define BLACK_THRESHOLD (    0 + 100)
+#define BLACK_THRESHOLD (    0 + 255)
 
 /* (should remove these fixed limits someday) */
 
@@ -28,8 +30,6 @@ any later version.  See the file COPYING.  */
 
 #define MAXIMAGEPALETTES 100
 
-#define MAXIMAGECOLORS 1000
-
 enum {
     K_MONO_,
     K_MASK_,
@@ -38,17 +38,17 @@ enum {
 };
 
 static ImageFamily *new_imf PARAMS ((char *name));
-static int colornamecmp PARAMS ((char *str1, char *str2));
 static int bitmaps_match PARAMS ((int w, int h, Obj *lispdata, char *rawdata));
 static int color_matches_mono PARAMS ((Image *img));
 static void write_pixmap PARAMS ((FILE *fp, int w, int h, int aw, int ah,
-				 int pixelsize, int rowbytes,
+				 int pixelsize,
 				 Obj *palette, int *rawpalette, int numcolors,
 				 Obj *lispdata, char *rawdata));
 static void write_bitmap PARAMS ((FILE *fp, char *subtyp, int w, int h,
 				 Obj *data, char *rawdata));
 static void write_palette_contents PARAMS ((FILE *fp, Obj *palette,
 					    int *rawpalette, int numcolors));
+static void write_color PARAMS ((FILE *fp, int n, int r, int g, int b));
 
 /* This is the array and count of known image families. */
 
@@ -62,11 +62,7 @@ ImagePalette **palettes;
 
 int numpalettes = 0;
 
-/* This is the array and count of named colors. */
-
-ImageColor **colors;
-
-int numcolors = 0;
+/* Head of the linked list of image files. */
 
 ImageFile *image_files;
 
@@ -127,6 +123,22 @@ char *name;
     return TRUE;
 }
 
+/* Bash invalid chars in a prospective imf name. */
+
+void
+validify_imf_name(buf)
+char *buf;
+{
+    char *tmp;
+
+    for (tmp = buf; *tmp; ++tmp) {
+	if (tmp == buf && *tmp == '-')
+	  *tmp = 'Z';
+	if (!isalnum(*tmp) && *tmp != '-')
+	  *tmp = '-';
+    }
+}
+
 /* Given a name, find or create an image family with that name. */
 
 ImageFamily *
@@ -152,7 +164,7 @@ char *name;
 	if (numimages >= MAXIMAGEFAMILIES) {
 	    return NULL;
 	}
-	imf = new_imf(name);
+	imf = new_imf(copy_string(name));
 	if (imf != NULL) {
 	    images[numimages++] = imf;
 	}
@@ -186,15 +198,25 @@ int loadnow;
 void (*callback) PARAMS ((ImageFamily *imf, int loadnow));
 {
     int done = FALSE, rslt, first = TRUE;
-    char buf1[80], buf2[80];
-    ImageFamily *imf;
+    char buf[BUFSIZE], *buf1, *buf2, *tmp;
+    ImageFamily *imf = NULL;
     ImageFile *imfile;
     
     while (!done) {
-	rslt = fscanf(fp, "%s %s\n", buf1, buf2);
-	if (rslt != 2)
+	/* Get a line from the file and parse it. */
+	if (fgets(buf, BUFSIZE-1, fp)) {
+	  buf1 = buf;
+	  buf2 = strchr(buf, ' ');
+	  if (buf2 == NULL)
+	    break;
+	  *buf2 = '\0';
+	  ++buf2;
+	  tmp = strchr(buf2, '\n');
+	  if (tmp)
+	    *tmp = '\0';
+	} else
 	  break;
-	else if (strcmp(buf1, ".") == 0
+	if (strcmp(buf1, ".") == 0
 		 && strcmp(buf2, ".") == 0)
 	  done = TRUE;
 	else if (first) {
@@ -209,7 +231,7 @@ void (*callback) PARAMS ((ImageFamily *imf, int loadnow));
 		break;
 	    }
 	} else {
-	    imf = get_imf(copy_string(buf1));
+	    imf = get_imf(buf1);
 	    if (imf != NULL) {
 		imfile = get_image_file(buf2);
 		imf->location = imfile;
@@ -273,8 +295,6 @@ void (*imf_callback) PARAMS ((ImageFamily *imf, int loadnow));
 	  (*imf_callback)(imf, TRUE);
     } else if (match_keyword(head, K_PALETTE)) {
 	interp_palette(form);
-    } else if (match_keyword(head, K_COLOR)) {
-	interp_color(form);
     } else {
 	/* Ignore any non-image forms, we might be reading from a 
 	   normal game design. */
@@ -304,33 +324,41 @@ get_img(imf, w, h)
 ImageFamily *imf;
 int w, h;
 {
-    Image *img, *previmg = NULL;
+    Image *img, *nimg, *previmg;
 
     for (img = imf->images; img != NULL; img = img->next) {
 	if (w == img->w && h == img->h)
 	  return img;
-	previmg = img;
     }
     /* Not found; create a new image and add it to the family. */
-    img = (Image *) xmalloc(sizeof(Image));
-    img->w = w;  img->h = h;
+    nimg = (Image *) xmalloc(sizeof(Image));
+    nimg->w = w;  nimg->h = h;
     /* Default min/max limits to actual size. */
-    img->minw = img->maxw = w;  img->minh = img->maxh = h;
-    img->embedx = img->embedy = -1;
-    img->embedw = img->embedh = -1;
-    img->monodata = img->colrdata = img->maskdata = lispnil;
-    img->palette = lispnil;
-    img->actualw = w;  img->actualh = h;
-    img->pixelsize = img->rowbytes = 0;
-    img->notes = lispnil;
+    nimg->minw = nimg->maxw = w;  nimg->minh = nimg->maxh = h;
+    nimg->embedx = nimg->embedy = -1;
+    nimg->embedw = nimg->embedh = -1;
+    nimg->monodata = nimg->colrdata = nimg->maskdata = lispnil;
+    nimg->palette = lispnil;
+    nimg->actualw = w;  nimg->actualh = h;
+    nimg->notes = lispnil;
+    nimg->bboxw = w;  nimg->bboxh = h;
     /* Rely on zeroing of xmalloc blocks to avoid clearing other fields. */
-    /* Link at front of list of images. */
-    if (previmg != NULL)
-      previmg->next = img;
-    else
-      imf->images = img;
+    /* Link in order by size. */
+    previmg = NULL;
+    for (img = imf->images; img != NULL; img = img->next) {
+	if ((nimg->w < img->w) || (nimg->w == img->w && nimg->h < img->h))
+	  break;
+	previmg = img;
+    }
+    if (previmg != NULL) {
+	nimg->next = previmg->next;
+	previmg->next = nimg;
+    } else {
+	nimg->next = imf->images;
+	imf->images = nimg;
+    }
     ++(imf->numsizes);
-    return img;
+    return nimg;
 }
 
 Image *
@@ -360,7 +388,7 @@ Obj *form;
 	}
 	return imf;
     } else {
-	/* garbage form */
+	run_warning("imf name must be a string");
     }
     return NULL;
 }
@@ -372,23 +400,26 @@ Obj *clauses;
 {
     Obj *rest, *clause;
 
-    for (rest = clauses; rest != lispnil; rest = cdr(rest)) {
+    for_all_list(clauses, rest) {
 	clause = car(rest);
 	if (consp(clause)) {
 	    if (symbolp(car(clause))) {
 		if (match_keyword(car(clause), K_NOTES)) {
 		    imf->notes = cadr(clause);
-		    /* (should complain about non-nil cddr?) */
+		    syntax_error(clause, "extra junk after property value");
 		} else {
-		    /* (should complain about syntax) */
+		    syntax_error(clause, "unknown image family property");
 		}
-	    } else {
+	    } else if (consp(car(clause))) {
 		interp_image(imf, car(clause), cdr(clause));
+	    } else {
+		syntax_error(clause, "not image or image family property");
 	    }
 	} else {
-	    /* garbage? */
+	    syntax_error(clause, "bogus clause");
 	}
     }
+    compute_image_bboxes(imf);
 }
 
 void
@@ -396,7 +427,7 @@ interp_image(imf, size, parts)
 ImageFamily *imf;
 Obj *size, *parts;
 {
-    int w, h, imtype, emx, emy;
+    int w, h, imtype, emx, emy, emw, emh;
     char *name;
     Image *img;
     Obj *head, *rest, *typ, *prop, *proptype, *datalist;
@@ -405,9 +436,18 @@ Obj *size, *parts;
     img = get_img(imf, w, h);
     if (img == NULL)
       run_error("no image?");
+    /* Kind of a hack. */
+    img->minw = img->w / 4;  img->minh = img->h / 4;
+    img->maxw = img->w * 4;  img->maxh = img->h * 4;
+    if (img->w == 1 && img->h == 1) {
+	/* A color is more like a tile than an icon. */
+	img->istile = TRUE;
+	img->palette = cons(cons(new_number(0), parts), lispnil);
+	return;
+    }
     if (match_keyword(car(cddr(size)), K_TILE))
       img->istile = TRUE;
-    for (rest = parts; rest != lispnil; rest = cdr(rest)) {
+    for_all_list(parts, rest) {
 	head = car(rest);
 	typ = car(head);
 	imtype = K_OTHER_;
@@ -431,9 +471,16 @@ Obj *size, *parts;
 	      run_warning("Changing embed x,y from %d,%d to %d,%d in %dx%d image of \"%s\"",
 			  img->embedx, img->embedy, emx, emy, w, h, imf->name);
 	    img->embedx = emx;  img->embedy = emy;
+	} else if (match_keyword(typ, K_EMBED_SIZE)) {
+	    emw = c_number(cadr(head));  emh = c_number(caddr(head));
+	    if ((img->embedw >= 0 && emw != img->embedw)
+		|| (img->embedh >= 0 && emh != img->embedh))
+	      run_warning("Changing embed w,h from %d,%d to %d,%d in %dx%d image of \"%s\"",
+			  img->embedw, img->embedh, emw, emh, w, h, imf->name);
+	    img->embedw = emw;  img->embedh = emh;
 	} else if (match_keyword(typ, K_NOTES)) {
 	    img->notes = cadr(head);
-	    /* (should complain about non-nil cddr?) */
+	    syntax_error(head, "extra junk after image notes property");
 	} else {
 	    run_warning("unknown image property in \"%s\"", imf->name);
 	}
@@ -449,8 +496,6 @@ Obj *size, *parts;
 		img->actualh = c_number(caddr(prop));
 	    } else if (match_keyword(proptype, K_PIXEL_SIZE)) {
 		img->pixelsize = c_number(cadr(prop));
-	    } else if (match_keyword(proptype, K_ROW_BYTES)) {
-		img->rowbytes = c_number(cadr(prop));
 	    } else if (match_keyword(proptype, K_PALETTE)) {
 		if (img->palette != lispnil && !equal(cdr(prop), img->palette))
 		  run_warning("Changing palette in %dx%d image of \"%s\"",
@@ -484,11 +529,89 @@ Obj *size, *parts;
 	    break;
 	}
     }
-    /* Kind of a hack. */
-    img->minw = img->w / 4;  img->minh = img->h / 4;
-    img->maxw = img->w * 4;  img->maxh = img->h * 4;
 }
 
+void
+compute_image_bboxes(imf)
+ImageFamily *imf;
+{
+    Image *img;
+
+    if (imf == NULL)
+      return;
+    for (img = imf->images; img != NULL; img = img->next) {
+	compute_image_bbox(img);
+    }
+}
+
+void
+compute_image_bbox(img)
+Image *img;
+{
+    int numbytes, i, j = 0, byte, x, y, x1, x2, k;
+    int xmin, ymin, xmax, ymax;
+    char *data = NULL;
+    Obj *datalist, *next;
+
+    datalist = img->maskdata;
+    numbytes = img->h * computed_rowbytes(img->w, 1);
+    x = y = 0;
+    xmin = img->w;  ymin = img->h;
+    xmax = 0;  ymax = 0;
+    for (i = 0; i < numbytes; ++i) {
+	if (img->maskdata != lispnil) {
+	    if (data == NULL || data[j] == '\0') {
+		next = car(datalist);
+		if (!stringp(next)) {
+		    syntax_error(datalist, "garbage in image data list");
+		    return;
+		}
+		data = c_string(next);
+		j = 0;
+		datalist = cdr(datalist);
+	    }
+	    /* Just skip over slashes, which are for readability only. */
+	    if (data[j] == '/')
+	      ++j;
+	    byte = hextoi(data[j]) * 16 + hextoi(data[j+1]);
+	    j += 2;
+	} else if (img->rawmaskdata != NULL) {
+	    byte = img->rawmaskdata[i];
+	} else {
+	    byte = 0xff;
+	}
+	if (byte != 0) {
+	    /* Find the most-significant and least-significant bits in
+	       the mask byte. */
+	    x1 = x2 = -1;
+	    k = 0;
+	    while (byte != 0) {
+		if (byte % 2 != 0 && x2 < 0)
+		  x2 = x + 7 - k;
+		byte /= 2;
+		if (byte == 0 && x1 < 0)
+		  x1 = x + 7 - k;
+		++k;
+	    }
+	    xmin = min(x1, xmin);  ymin = min(y, ymin);
+	    xmax = max(x2, xmax);  ymax = max(y, ymax);
+	}
+	x += 8;
+	if (x >= img->w) {
+	    x = 0;
+	    ++y;
+	}
+    }
+    /* Compute position and size of bounding box. */
+    if (xmin <= xmax && ymin <= ymax) {
+	img->bboxx = xmin;  img->bboxy = ymin;
+	img->bboxw = xmax - xmin;  img->bboxh = ymax - ymin;
+    }
+}
+
+/* Given a list of strings, interpret the hex digits and put the
+   results at the given address. */
+
 void
 interp_bytes(datalist, numbytes, destaddr, jump)
 Obj *datalist;
@@ -500,10 +623,16 @@ char *destaddr;
 
     for (i = 0; i < numbytes; ++i) {
 	if (data == NULL || data[j] == '\0') {
-	    if (!stringp(car(datalist)))
-	      return; /* (should warn somehow?) */
-	    data = c_string(car(datalist));
-	    j = 0;
+	    if (datalist == lispnil) {
+		return;
+	    } else if (stringp(car(datalist))) {
+		data = c_string(car(datalist));
+		j = 0;
+	    } else {
+		syntax_error(datalist, "Non-string in image data list");
+		/* Have to give up now. */
+		return;
+	    }
 	    datalist = cdr(datalist);
 	}
 	/* Just skip over slashes, which are for readability only. */
@@ -552,6 +681,7 @@ char *name;
     imp = (ImagePalette *) xmalloc(sizeof(ImagePalette));
     imp->name = name;
     imp->notes = lispnil;
+    imp->palette = lispnil;
     return imp;
 }
 
@@ -601,136 +731,6 @@ char *name;
     return NULL;
 }
 
-ImageColor *
-interp_color(form)
-Obj *form;
-{
-    Obj *elts;
-    ImageColor *imc;
-
-    if (stringp(cadr(form))) {
-	imc = get_imc(c_string(cadr(form)));
-	elts = cddr(form);
-	if (consp(car(elts))
-	    && symbolp(car(car(elts)))) {
-	    if (match_keyword(car(car(elts)), K_NOTES)) {
-		imc->notes = cadr(car(elts));
-	    } else {
-	    }
-	    elts = cdr(elts);
-	}
-	imc->r = c_number(car(elts));
-	imc->g = c_number(car(cdr(elts)));
-	imc->b = c_number(car(cddr(elts)));
-	return imc;
-    }
-    return NULL;
-}
-
-ImageColor *
-new_image_color(name)
-char *name;
-{
-    ImageColor *imc;
-
-    imc = (ImageColor *) xmalloc(sizeof(ImageColor));
-    imc->name = name;
-    imc->notes = lispnil;
-    return imc;
-}
-
-char *
-canonical_color_name(str)
-char *str;
-{
-    return str;
-}
-
-/* Given a name, find or create an image color with that name. */
-
-ImageColor *
-get_imc(name)
-char *name;
-{
-    ImageColor *imc = NULL;
-
-    if (name == NULL)
-      return NULL;
-    if (colors == NULL)
-      colors =
-	(ImageColor **) xmalloc(MAXIMAGECOLORS * sizeof(ImageColor *));
-    if ((imc = find_imc(name)) == NULL) {
-	if (numcolors >= MAXIMAGECOLORS)
-	  return NULL;
-	imc = new_image_color(canonical_color_name(name));
-	if (imc != NULL) {
-	    imc->r = imc->g = imc->b = 0;
-	    colors[numcolors++] = imc;
-	}
-    }
-    return imc;
-}
-
-/* Find the image color of the given name, if it exists. */
-
-ImageColor *
-find_imc(name)
-char *name;
-{
-    int i;
-
-    for (i = 0; i < numcolors; ++i) {
-	if (colornamecmp(name, colors[i]->name) == 0)
-	  return colors[i];
-    }
-    return NULL;
-}
-
-/* X-style color names have several variants, but we only want one of them,
-   so this matches the variants with each other. */
-
-static int
-colornamecmp(str1, str2)
-char *str1, *str2;
-{
-    while (*str1 != '\0' && *str2 != '\0') {
-	if (*str1 == *str2) {
-	    ++str1;  ++str2;
-	} else if (isalpha(*str1) && isalpha(*str2)
-		   && tolower(*str1) == tolower(*str2)) {
-	    ++str1;  ++str2;
-	} else if (*str1 == 'a' && *str2 == 'e'
-		   && *(str1+1) == 'y' && *(str2+1) == 'y') {
-	    ++str1;  ++str2;
-	} else if (*str1 == 'e' && *str2 == 'a'
-		   && *(str1+1) == 'y' && *(str2+1) == 'y') {
-	    ++str1;  ++str2;
-#if 0
-	} else if (*str1 == ' ') {
-	    ++str1;
-	} else if (*str2 == ' ') {
-	    ++str2;
-#endif
-	} else {
-	    return *str1 - *str2;
-	}
-    }
-    if (*str1 == '\0') {
-	if (*str2 == '\0') {
-	    return 0;
-	} else {
-	    return 1;
-	}
-    } else {
-	if (*str2 == '\0') {
-	    return -1;
-	} else {
-	    /* can never happen, but humor the compiler */
-	    return 0;
-	}
-    }
-}
-
 
 /* Try to find the best of multiple images for the given bounding box. */
 /* Don't return anything that won't fit in min space. */
@@ -771,6 +771,91 @@ int w, h;
     return (best ? best : fallback);
 }
 
+Image *
+smallest_image(imf)
+ImageFamily *imf;
+{
+    Image *img, *smallest = NULL;
+
+    if (imf == NULL)
+      return NULL;
+    for (img = imf->images; img != NULL; img = img->next) {
+	if (smallest == NULL || (img->w < smallest->w && img->h < smallest->h))
+	  smallest = img;
+    }
+    if (smallest != NULL) {
+	return smallest;
+    }
+    return NULL;
+}
+
+/* Compute the right location for the given emblem and unit images. */
+
+static int tmpbw;  /* work around Think C bug */
+
+int
+emblem_position(uimg, ename, eimg, sw, sh, exxp, eyyp, ewp, ehp)
+Image *uimg, *eimg;
+char *ename;
+int sw, sh, *exxp, *eyyp, *ewp, *ehp;
+{
+    int ex, ey, ew, eh, bx, by, bw, bh, overlap;
+
+    if (uimg
+	&& uimg->embedname
+	&& ename != NULL
+	&& strcmp(uimg->embedname, ename) == 0) {
+	/* Correct emblem is part of the unit's image, don't need to draw. */
+	return FALSE;
+    }
+    /* (should use emblem bbox to help calc) */
+    /* Get the size of the emblem, either from the image or by computing
+       a reasonable default. */
+    if (uimg && uimg->embedw > 0 && uimg->embedh > 0) {
+	ew = uimg->embedw;  eh = uimg->embedh;
+    } else {
+	ew = min(sw, max(8, sw / 4));  eh = min(sh, max(8, sh / 4));
+    }
+    /* Position the emblem, either explicitly, or default to UR corner
+	   (note that we need the emblem's width to do this) */
+    if (uimg && uimg->embedx >= 0 && uimg->embedy >= 0) {
+	ex = uimg->embedx;  ey = uimg->embedy;
+    } else if (uimg && (uimg->bboxw != uimg->w || uimg->bboxh != uimg->h)) {
+	overlap = FALSE;
+	/* Scale bounding box by space given to image. */
+	bx = (uimg->bboxx * sw) / uimg->w;  by = (uimg->bboxy * sh) / uimg->h;
+	tmpbw = (uimg->bboxw * sw) / uimg->w;
+	bh = (uimg->bboxh * sh) / uimg->h;
+	bw = tmpbw;
+	/* Position the emblem outside the image's bbox if possible,
+	   moving in if necessary to stay inside the image's allowed
+	   area (sw x sh). */
+	ex = bx + bw;
+	if (ex + ew > sw) {
+	    /* Emblem too wide to fit between unit bbox and edge of
+	       area; butt it against edge of area, note the
+	       overlap. */
+	    ex = sw - ew;
+	    overlap = TRUE;
+	}
+	if (ex < 0)
+	  ex = 0;
+	ey = by;
+	if (overlap)
+	  ey -= eh;
+	if (ey + eh > sh)
+	  ey = sh - eh;
+	if (ey < 0)
+	  ey = 0;
+    } else {
+	ex = sw - ew;  ey = 0;
+    }
+    /* Return the results. */
+    *exxp = ex;  *eyyp = ey;
+    *ewp = ew;  *ehp = eh;
+    return TRUE;
+}
+
 /* The comparison function for the image list just does "strcmp" order
    and *requires* that all image families be named and named uniquely. */
 
@@ -811,26 +896,6 @@ sort_all_palettes()
     qsort(&(palettes[0]), numpalettes, sizeof(ImagePalette *), palette_name_compare);
 }
 
-/* The comparison function for the color list just does "strcmp" order
-   and *requires* that all image palettes be named and named uniquely. */
-
-static int
-color_name_compare(imc1, imc2)
-#ifdef THINK_C
-const
-#endif
-void *imc1, *imc2;
-{
-    return strcmp((*((ImageColor **) imc1))->name,
-		  (*((ImageColor **) imc2))->name);
-}
-
-void
-sort_all_colors()
-{
-    qsort(&(colors[0]), numcolors, sizeof(ImageColor *),  color_name_compare);
-}
-
 /* Check Lisp-format and binary-format data for consistency. */
 
 void
@@ -871,7 +936,7 @@ char *rawdata;
     int i, j = 0, rowbytes, numbytes, byte, jump = 0;
     char *datastr = NULL;
 
-    rowbytes = (w + 7) / 8;
+    rowbytes = computed_rowbytes(w, 1);
     numbytes =  h * rowbytes;
     for (i = 0; i < numbytes; ++i) {
 	if (datastr == NULL || datastr[j] == '\0') {
@@ -891,6 +956,38 @@ char *rawdata;
     return TRUE;
 }
 
+/* Write the imf directory for the given set of images. */
+
+void
+write_imf_dir(filename, images, num)
+char *filename;
+ImageFamily **images;
+int num;
+{
+    int i;
+    char *loc;
+    ImageFamily *imf;
+    FILE *fp;
+
+    fp = fopen(filename, "w");
+    if (fp != NULL) {
+	fprintf(fp, "ImageFamilyName FileName\n");
+	for (i = 0; i < num; ++i) {
+	    imf = images[i];
+	    loc = "???";
+	    if (imf->location && !empty_string(imf->location->name))
+	      loc = imf->location->name;
+	    fprintf(fp, "%s %s\n", imf->name, loc);
+	    /* (to write imf files, should scan through images once for
+	       each file, writing all images found that are in that file) */
+	}
+	fprintf(fp, ". .\n");
+	fclose(fp);
+    } else {
+	run_warning("could not open file \"%s\" for writing", filename);
+    }
+}
+
 /* Write out the entire image family. */
 
 void
@@ -898,9 +995,10 @@ write_imf(fp, imf)
 FILE *fp;
 ImageFamily *imf;
 {
+    Obj *palent;
     Image *img;
     
-    if (imf == NULL)
+    if (fp == NULL || imf == NULL)
       return;
     if (imf->name == NULL) {
 	fprintf(fp, "; garbage image family?\n");
@@ -918,11 +1016,12 @@ ImageFamily *imf;
 	    || img->colrdata != lispnil
 	    || img->rawmonodata != NULL
 	    || img->rawmaskdata != NULL
-	    || img->rawcolrdata != NULL) {
+	    || img->rawcolrdata != NULL
+	    || (img->w == 1 && img->h == 1)) {
 	    fprintf(fp, "(%s \"%s\"", keyword_name(K_IMF), imf->name);
 	    fprintf(fp, " (");
 	    fprintf(fp, "(%d %d", img->w, img->h);
-	    if (img->istile)
+	    if (img->istile && !(img->w == 1 && img->h == 1))
 	      fprintf(fp, " %s", keyword_name(K_TILE));
 	    fprintf(fp, ")");
 	    if (img->embedname) {
@@ -942,11 +1041,27 @@ ImageFamily *imf;
 		fprintlisp(fp, img->notes);
 		fprintf(fp, ")\n ");
 	    }
-	    if ((img->colrdata != lispnil || img->rawcolrdata)
-		&& !color_matches_mono(img)) {
+	    /* Write a single color if that's what this image is. */
+	    if (img->w == 1 && img->h == 1) {
+		if (img->rawpalette != NULL) {
+		    write_color(fp, -1,
+				img->rawpalette[1], img->rawpalette[2], img->rawpalette[3]);
+		} else if (img->palette != lispnil) {
+		    palent = cdr(car(img->palette));
+		    if (stringp(car(palent)) || symbolp(car(palent))) {
+			fprintf(fp, " %s", c_string(car(palent)));
+		    } else {
+			write_color(fp, -1,
+				    c_number(car(palent)),
+				    c_number(cadr(palent)),
+				    c_number(caddr(palent)));
+		    }
+		}
+	    } else if ((img->colrdata != lispnil || img->rawcolrdata)
+			&& !color_matches_mono(img)) {
 		fprintf(fp, "\n  ");
 		write_pixmap(fp, img->w, img->h, img->actualw, img->actualh,
-			     img->pixelsize, img->rowbytes,
+			     img->pixelsize,
 			     img->palette, img->rawpalette, img->numcolors,
 			     img->colrdata, img->rawcolrdata);
 	    }
@@ -965,102 +1080,137 @@ ImageFamily *imf;
     }
 }
 
+/* Study an ostensibly color image to see if its color table includes
+   black and white only (white first, then black), and if its data is
+   the same as the mono version of the image. */
+
 static int
 color_matches_mono(img)
 Image *img;
 {
     int i, cj, mj, rowbytes, numbytes, cbyte, mbyte, jump = 0;
+    int col[2], red[2], grn[2], blu[2];
     char *cdatastr = NULL, *mdatastr = NULL;
-    Obj *clispdata = img->colrdata, *mlispdata = img->monodata;
+    Obj *clispdata = img->colrdata, *mlispdata = img->monodata, *palette;
 
     if (img->pixelsize != 1)
       return FALSE;
+
     /* No match possible if not a black-white-only palette. */
-    if (!((img->numcolors == 2
-	   && img->rawpalette != NULL
-	   && img->rawpalette[0+0] == 0
-	   && img->rawpalette[0+1] > WHITE_THRESHOLD
-	   && img->rawpalette[0+2] > WHITE_THRESHOLD
-	   && img->rawpalette[0+3] > WHITE_THRESHOLD
-	   && img->rawpalette[4+0] == 1
-	   && img->rawpalette[4+1] < BLACK_THRESHOLD
-	   && img->rawpalette[4+2] < BLACK_THRESHOLD
-	   && img->rawpalette[4+3] < BLACK_THRESHOLD)
-	  || (img->palette != lispnil
-	      && c_number(car(car(img->palette))) == 0
-	      && c_number(cadr(car(img->palette))) > WHITE_THRESHOLD
-	      && c_number(caddr(car(img->palette))) > WHITE_THRESHOLD
-	      && c_number(caddr(cdr(car(img->palette)))) > WHITE_THRESHOLD
-	      && c_number(car(cadr(img->palette))) == 1
-	      && c_number(cadr(cadr(img->palette))) < BLACK_THRESHOLD
-	      && c_number(caddr(cadr(img->palette))) < BLACK_THRESHOLD
-	      && c_number(caddr(cdr(cadr(img->palette)))) < BLACK_THRESHOLD)))
+    if (img->numcolors > 2)
       return FALSE;
+
+    if (img->rawpalette != NULL) {
+	for (i = 0; i < 2; i++) {
+	    col[i] = img->rawpalette[4*i+0];
+	    red[i] = img->rawpalette[4*i+1];
+	    grn[i] = img->rawpalette[4*i+2];
+	    blu[i] = img->rawpalette[4*i+3];
+	}
+    } else if (img->palette != lispnil) {
+	palette = img->palette;
+	parse_lisp_palette_entry(car(palette), &col[0],
+				 &red[0], &grn[0], &blu[0]);
+	if (cdr(palette) == lispnil) {
+	    /* If only one color in the palette, say the other one is
+	       black. */
+	    col[1] = 1;
+	    red[1] = grn[1] = blu[1] = 0;
+	    /* If the one color is black, say it's white. */
+	    if (col[0] == 0
+		&& red[0] < BLACK_THRESHOLD
+		&& grn[0] < BLACK_THRESHOLD
+		&& blu[0] < BLACK_THRESHOLD) {
+		col[0] = 0;
+		red[0] = grn[0] = blu[0] = 65535;
+	    }
+	} else {
+	    /* Parse the second entry in the palette. */
+	    parse_lisp_palette_entry(cadr(palette), &col[1],
+				     &red[1], &grn[1], &blu[1]);
+	}
+    } else {
+	return FALSE;
+    }
+
+    if (!(col[0] == 0
+	  && red[0] > WHITE_THRESHOLD
+	  && grn[0] > WHITE_THRESHOLD
+	  && blu[0] > WHITE_THRESHOLD
+	  && col[1] == 1
+	  && red[1] < BLACK_THRESHOLD
+	  && grn[1] < BLACK_THRESHOLD
+	  && blu[1] < BLACK_THRESHOLD))
+	return FALSE;
+
     /* Now compare the contents. */
-    rowbytes = (img->w + 7) / 8;
+    rowbytes = computed_rowbytes(img->w, 1);
     numbytes =  img->h * rowbytes;
     cj = mj = 0;
     for (i = 0; i < numbytes; ++i) {
-      if (clispdata != lispnil) {
-	if (cdatastr == NULL || cdatastr[cj] == '\0') {
-	    if (!stringp(car(clispdata)))
-	      break;
-	    cdatastr = c_string(car(clispdata));
-	    cj = 0;
-	    clispdata = cdr(clispdata);
+	/* Extract one byte of the color image. */
+	if (clispdata != lispnil) {
+	    if (cdatastr == NULL || cdatastr[cj] == '\0') {
+		if (!stringp(car(clispdata)))
+		  break;
+		cdatastr = c_string(car(clispdata));
+		cj = 0;
+		clispdata = cdr(clispdata);
+	    }
+	    if (cdatastr[cj] == '/')
+	      ++cj;
+	    cbyte = hextoi(cdatastr[cj]) * 16 + hextoi(cdatastr[cj+1]);
+	    cj += 2;
+	} else if (img->rawcolrdata != NULL) {
+	    cbyte = (img->rawcolrdata)[i];
+	} else {
+	    return FALSE;
 	}
-	if (cdatastr[cj] == '/')
-	  ++cj;
-	cbyte = hextoi(cdatastr[cj]) * 16 + hextoi(cdatastr[cj+1]);
-	cj += 2;
-      } else if (img->rawcolrdata != NULL) {
-	cbyte = (img->rawcolrdata)[i];
-      } else {
-	return FALSE;
-      }
-      if (mlispdata != lispnil) {
-	if (mdatastr == NULL || mdatastr[mj] == '\0') {
-	    if (!stringp(car(mlispdata)))
-	      break;
-	    mdatastr = c_string(car(mlispdata));
-	    mj = 0;
-	    mlispdata = cdr(mlispdata);
+	/* Extract one byte of the mono image. */
+	if (mlispdata != lispnil) {
+	    if (mdatastr == NULL || mdatastr[mj] == '\0') {
+		if (!stringp(car(mlispdata)))
+		  break;
+		mdatastr = c_string(car(mlispdata));
+		mj = 0;
+		mlispdata = cdr(mlispdata);
+	    }
+	    if (mdatastr[mj] == '/')
+	      ++mj;
+	    mbyte = hextoi(mdatastr[mj]) * 16 + hextoi(mdatastr[mj+1]);
+	    mj += 2;
+	} else if (img->rawmonodata != NULL) {
+	    mbyte = (img->rawmonodata)[i];
+	} else {
+	    return FALSE;
 	}
-	if (mdatastr[mj] == '/')
-	  ++mj;
-	mbyte = hextoi(mdatastr[mj]) * 16 + hextoi(mdatastr[mj+1]);
-	mj += 2;
-      } else if (img->rawmonodata != NULL) {
-	mbyte = (img->rawmonodata)[i];
-      } else {
-	return FALSE;
-      }
-      if (cbyte != mbyte)
-	return FALSE;
+	/* Compare the bytes. */
+	if (cbyte != mbyte)
+	  return FALSE;
     }
     return TRUE;
 }
 
 static void
-write_pixmap(fp, w, h, actualw, actualh, pixelsize, rowbytes,
+write_pixmap(fp, w, h, actualw, actualh, pixelsize,
 	     palette, rawpalette, numcolors, lispdata, rawdata)
 FILE *fp;
-int w, h, actualw, actualh, pixelsize, rowbytes, *rawpalette, numcolors;
+int w, h, actualw, actualh, pixelsize, *rawpalette, numcolors;
 Obj *palette, *lispdata;
 char *rawdata;
 {
-    int dolisp, i, j = 0, numbytes, byte, jump = 0;
+    int dolisp, i, j = 0, rowbytes, numbytes, byte;
     char *datastr = NULL;
 
     actualw = (actualw != 0 ? actualw : w);
     actualh = (actualh != 0 ? actualh : h);
-    dolisp = (lispdata != lispnil);	
+    dolisp = (lispdata != lispnil);
+    rowbytes = computed_rowbytes(actualw, pixelsize);
     numbytes = actualh * rowbytes;
     fprintf(fp, "(%s", keyword_name(K_COLOR));
     if (actualw != w || actualh != h)
       fprintf(fp, " (%s %d %d)", keyword_name(K_ACTUAL), actualw, actualh);
     fprintf(fp, " (%s %d)", keyword_name(K_PIXEL_SIZE), pixelsize);
-    fprintf(fp, " (%s %d)", keyword_name(K_ROW_BYTES), rowbytes);
     if (palette != lispnil || (rawpalette && numcolors))
       write_palette_contents(fp, palette, rawpalette, numcolors);
     fprintf(fp, "\n   \"");
@@ -1080,8 +1230,6 @@ char *rawdata;
 	    if (datastr[j] == '/')
 	      ++j;
 	    byte = hextoi(datastr[j]) * 16 + hextoi(datastr[j+1]);
-	    if (jump == 1 || (jump > 0 && i % jump == 0))
-	      i += jump;
 	    j += 2;
 	} else {
 	    byte = rawdata[i];
@@ -1099,12 +1247,12 @@ int w, h;
 Obj *lispdata;
 char *rawdata;
 {
-    int dolisp, i, j = 0, rowbytes, numbytes, byte, jump = 0;
+    int dolisp, i, j = 0, rowbytes, numbytes, byte;
     char *datastr = NULL;
 
-    /* Lisp data overrides byte data. */	
+    /* Lisp data overrides raw byte data. */	
     dolisp = (lispdata != lispnil);	
-    rowbytes = (w + 7) / 8;
+    rowbytes = computed_rowbytes(w, 1);
     numbytes =  h * rowbytes;
     fprintf(fp, "(%s", subtyp);
     if (w > 16 || h > 16)
@@ -1123,11 +1271,10 @@ char *rawdata;
 		j = 0;
 		lispdata = cdr(lispdata);
 	    }
+	    /* Ignore any slashes, they're for human readability. */
 	    if (datastr[j] == '/')
 	      ++j;
 	    byte = hextoi(datastr[j]) * 16 + hextoi(datastr[j+1]);
-	    if (jump == 1 || (jump > 0 && i % jump == 0))
-	      i += jump;
 	    j += 2;
 	} else {
 	    byte = rawdata[i];
@@ -1143,38 +1290,57 @@ FILE *fp;
 Obj *palette;
 int *rawpalette, numcolors;
 {
-    int len, i;
-    Obj *color;
+    int len, i, col, red, grn, blu;
+    Obj *restpal;
 
     len = (palette != lispnil ? length(palette) : numcolors);
     if (len > 2)
       fprintf(fp, "\n  ");
     fprintf(fp, " (%s", keyword_name(K_PALETTE));
-    if (palette != lispnil) {
-	for (; palette != lispnil; palette = cdr(palette)) {
-	    color = car(palette);
+    if (symbolp(palette)) {
+	fprintf(fp, " %s", c_string(palette));
+    } else if (palette != lispnil) {
+	for_all_list(palette, restpal) {
+	    parse_lisp_palette_entry(car(restpal), &col, &red, &grn, &blu);
 	    if (len > 2)
 	      fprintf(fp, "\n   ");
-	    fprintf(fp, " (%d %d %d %d)",
-		    c_number(car(color)),
-		    c_number(car(cdr(color))),
-		    c_number(car(cdr(cdr(color)))),
-		    c_number(car(cdr(cdr(cdr(color))))));
+ 	    write_color(fp, col, red, grn, blu);
+	}
+    } else if (rawpalette != NULL) {
+	for (i = 0; i < numcolors; i++) {
+	    if (len > 2)
+	      fprintf(fp, "\n   ");
+	    write_color(fp, rawpalette[4*i],
+			rawpalette[4*i+1], rawpalette[4*i+2], rawpalette[4*i+3]);
 	}
     } else {
- 	for (i = 0; i < numcolors; i++) {
-	    if (len > 2)
-	      fprintf(fp, "\n   ");
- 	    fprintf(fp, " (%d %d %d %d)",
- 		    rawpalette[4*i],
- 		    rawpalette[4*i+1],
- 		    rawpalette[4*i+2],
- 		    rawpalette[4*i+3]);
- 	}
+	fprintf(fp, " #| no palette? |# ");
     }
     fprintf(fp, ")");
 }
 
+static void
+write_color(fp, n, r, g, b)
+FILE *fp;
+int n, r, g, b;
+{
+    char *colorname;
+
+    if (n >= 0)
+      fprintf(fp, " (%d", n);
+    colorname = find_color_name(r, g, b);
+    if (!empty_string(colorname)) {
+	/* Write color name.  Note that we write as a symbol, so that
+	   each instance of "white" doesn't become a separate string. */
+	fprintf(fp, " %s", colorname);
+    } else {
+	/* Write individual color components. */
+	fprintf(fp, " %d %d %d", r, g, b);
+    }
+    if (n >= 0)
+      fprintf(fp, ")");
+}
+
 void
 write_imp(fp, imp)
 FILE *fp;
@@ -1193,27 +1359,73 @@ ImagePalette *imp;
 	fprintlisp(fp, imp->notes);
 	fprintf(fp, ")\n ");
     }
-    /* (should dump out palette entries here) */
+    write_palette_contents(fp, imp->palette, NULL, 0);
     fprintf(fp, ")\n");
 }
 
-void
-write_imc(fp, imc)
-FILE *fp;
-ImageColor *imc;
+/* Given rgb components, return names of standard colors if the match is close. */
+
+char *
+find_color_name(r, g, b)
+int r, g, b;
 {
-    if (imc == NULL)
-      return;
-    if (imc->name == NULL) {
-	fprintf(fp, "; garbage color?\n");
-	return;
+    if (r > WHITE_THRESHOLD
+	&& g > WHITE_THRESHOLD
+	&& b > WHITE_THRESHOLD)
+      return "white";
+    else if (r < BLACK_THRESHOLD
+	&& g < BLACK_THRESHOLD
+	&& b < BLACK_THRESHOLD)
+      return "black";
+    else if (r > WHITE_THRESHOLD
+	&& g < BLACK_THRESHOLD
+	&& b < BLACK_THRESHOLD)
+      return "red";
+    else if (r < BLACK_THRESHOLD
+	&& g > WHITE_THRESHOLD
+	&& b < BLACK_THRESHOLD)
+      return "green";
+    else if (r < BLACK_THRESHOLD
+	&& g < BLACK_THRESHOLD
+	&& b > WHITE_THRESHOLD)
+      return "blue";
+    else
+      return NULL;
+}
+
+void
+parse_lisp_palette_entry(palentry, col, red, grn, blu)
+Obj *palentry;
+int *col, *red, *grn, *blu;
+{
+    Obj *colorcomp;
+    char *colorname;
+
+    *col = c_number(car(palentry));
+    colorcomp = cdr(palentry);
+    if (symbolp(car(colorcomp)) || stringp(car(colorcomp))) {
+	colorname = c_string(car(colorcomp));
+	*red = *grn = *blu = 0;
+	if (strcmp(colorname, "white") == 0) {
+	    *red = *grn = *blu = 65535;
+	} else if (strcmp(colorname, "black") == 0) {
+	    /* done */
+	} else if (strcmp(colorname, "red") == 0) {
+	    *red = 65535;
+	} else if (strcmp(colorname, "green") == 0) {
+	    *grn = 65535;
+	} else if (strcmp(colorname, "blue") == 0) {
+	    *blu = 65535;
+	} else {
+	    init_warning("No color named \"%s\" found, substituting gray",
+			 colorname);
+	    *red = *grn = *blu = 128 * 256;
+	}
+    } else if (numberp(car(colorcomp))) {
+	*red = c_number(car(colorcomp));
+	*grn = c_number(cadr(colorcomp));
+	*blu = c_number(caddr(colorcomp));
+    } else {
+	init_warning("palette color info is not a name or set of numbers, ignoring");
     }
-    fprintf(fp, "(%s \"%s\"",
-	    keyword_name(K_COLOR), imc->name);
-    if (imc->notes != lispnil) {
-	fprintf(fp, "\n  (%s ", keyword_name(K_NOTES));
-	fprintlisp(fp, imc->notes);
-	fprintf(fp, ")\n ");
-    }
-    fprintf(fp, " %d %d %d)\n", imc->r, imc->g, imc->b);
 }
