@@ -1,13 +1,17 @@
 /* Definitions for Xconq images.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
-/* An image family is like a finder icon family, but allows multiple kinds
-   of images of an arbitrary set of sizes. */
+/* An image family is like a Mac Finder icon family, but allows
+   multiple kinds of images of an arbitrary set of sizes.  Individual
+   images in a family may have both "lisp" form and "raw" form, the
+   former being used for reading and writing, while the raw form is
+   an intermediary for conversion to and from platform-specific
+   representations. */
 
 typedef struct a_image {
     int w, h;			/* Nominal size of the image */
@@ -21,15 +25,16 @@ typedef struct a_image {
     Obj *colrdata;		/* Color data, in GDL form */
     Obj *maskdata;		/* Mask data, in GDL form */
     int actualw, actualh;
-    int pixelsize;
-    int rowbytes;
-    Obj *palette;
+    int pixelsize;		/* Number of bits per pixel */
+    Obj *palette;		/* Color palette, in list form */
     Obj *notes;			/* designer notes about the image */
     char *rawmonodata;		/* Monochrome data, as array of bytes */
     char *rawcolrdata;		/* Color data, as array of bytes */
     char *rawmaskdata;		/* Mask data, as array of bytes */
-    int *rawpalette;
+    int *rawpalette;		/* Color palette, in raw form */
     int numcolors;		/* Number of colors in raw palette */
+    int bboxx, bboxy;		/* Position of actual data within image */
+    int bboxw, bboxh;		/* Dimensions of actual data within image */
     char *hook;			/* Pointer to interface-specific data */
     struct a_image *next;	/* Pointer to next image in family */
 } Image;
@@ -40,28 +45,21 @@ typedef struct a_image_family {
     struct a_image_file *location;  /* File or whatever to look for data */
     Obj *notes;			/* designer notes about the image family */
     int numsizes;		/* Number of images in the list */
-    Image *images;
+    Image *images;		/* Pointer to chain of images */
 } ImageFamily;
 
 typedef struct a_image_palette {
     char *name;			/* Name of the palette */
     struct a_image_file *location;  /* File or whatever to look for data */
     Obj *notes;			/* designer notes about the palette */
+    Obj *palette;		/* List of colors */
     int numcolors;		/* Number of colors in the palette */
 } ImagePalette;
 
-typedef struct a_image_color {
-    char *name;			/* Name of the color */
-    struct a_image_file *location;  /* File or whatever to look for data */
-    Obj *notes;			/* designer notes about the color */
-    short defined;
-    short r, g, b;
-} ImageColor;
-
 typedef struct a_image_file {
-    char *name;
-    int loaded;
-    struct a_image_file *next;
+    char *name;			/* Name of the file */
+    int loaded;			/* True if it has already been loaded */
+    struct a_image_file *next;	/* Link to the next file. */
 } ImageFile;
 
 extern ImageFamily **images;
@@ -72,12 +70,10 @@ extern ImagePalette **palettes;
 
 extern int numpalettes;
 
-extern ImageColor **colors;
-
-extern int numcolors;
-
 extern ImageFile *image_files;
 
+#define computed_rowbytes(w, pixelsize) (((w * pixelsize) + 7) / 8)
+
 #define hextoi(c) (((c) >= '0' && (c) <= '9') ? ((c) - '0') : ((c) - 'a' + 10))
 
 typedef void (*readimf_hook) PARAMS ((ImageFamily *, int));
@@ -94,11 +90,6 @@ extern ImagePalette *new_image_palette PARAMS ((char *name));
 extern ImagePalette *get_imp PARAMS ((char *name));
 extern ImagePalette *find_imp PARAMS ((char *name));
 
-extern char *canonical_color_name PARAMS ((char *str));
-extern ImageColor *new_image_color PARAMS ((char *name));
-extern ImageColor *get_imc PARAMS ((char *name));
-extern ImageColor *find_imc PARAMS ((char *name));
-
 extern ImageFile *get_image_file PARAMS ((char *name));
 extern void load_image_families PARAMS ((FILE *fp, int loadnow,
 					 readimf_hook callback));
@@ -113,16 +104,28 @@ extern void interp_image PARAMS ((ImageFamily *imf, Obj *size, Obj *parts));
 extern void interp_bytes PARAMS ((Obj *datalist, int numbytes, char *destaddr,
 			  int jump));
 extern ImagePalette *interp_palette PARAMS ((Obj *form));
-extern ImageColor *interp_color PARAMS ((Obj *form));
 
 extern Image *best_image PARAMS ((ImageFamily *imf, int w, int h));
+extern Image *smallest_image PARAMS ((ImageFamily *imf));
+extern int emblem_position PARAMS ((Image *uimg, char *ename, Image *eimg,
+				    int sw, int sh,
+				    int *exxp, int *eyyp, int *ewp, int *ehp));
 
 extern void sort_all_images PARAMS ((void));
 extern void sort_all_palettes PARAMS ((void));
-extern void sort_all_colors PARAMS ((void));
 
 extern void check_imf PARAMS ((ImageFamily *imf));
 
 extern void write_imf PARAMS ((FILE *fp, ImageFamily *imf));
 extern void write_imp PARAMS ((FILE *fp, ImagePalette *imp));
-extern void write_imc PARAMS ((FILE *fp, ImageColor *imc));
+
+extern void make_generic_image_data PARAMS ((ImageFamily *imf));
+
+extern void validify_imf_name PARAMS ((char *buf));
+extern void compute_image_bboxes PARAMS ((ImageFamily *imf));
+extern void compute_image_bbox PARAMS ((Image *img));
+extern void write_imf_dir PARAMS ((char *filename, ImageFamily **images,
+				   int num));
+extern char *find_color_name PARAMS ((int r, int g, int b));
+extern void parse_lisp_palette_entry PARAMS ((Obj *palentry, int *c,
+					      int *r, int *g, int *b));
