@@ -1,36 +1,37 @@
 /* General Xconq initialization.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
-   Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
-/* Initialization is complicated, because Xconq needs lots of setup for
-   maps, units, sides, and the like.  The data must also be able to come
-   from saved games, scenarios, bare maps in files, type definitions,
-   or be synthesized if necessary. */
+/* Initialization is complicated, because Xconq needs lots of setup
+   for maps, units, sides, and the like.  The data must also be able
+   to come from saved games, scenarios, bare maps in files, type
+   definitions, or be synthesized if necessary. */
 
-/* The general theory is that everything gets set to a known empty state,
-   then all files are read, then all the synthesis methods get run.
-   Files/readers and synth methods are each responsible for preventing
-   fatal conflicts. */
+/* The general theory is that everything gets set to a known empty
+   state, then all files are read, then all the synthesis methods get
+   run.  Files/readers and synth methods are each responsible for
+   preventing fatal conflicts. */
 
 #include "conq.h"
-extern void add_library_path PARAMS ((char *path));
-extern void prealloc_debug PARAMS ((void));
+extern char *curdatestr;
+#include "kernel.h"
 #include "version.h"
 
 static void cache_possible_types PARAMS ((void));
+static void kill_excess_units PARAMS ((void));
 static void trial_assign_player PARAMS ((int i, Player *player));
 static void init_view_2 PARAMS ((Side *side, int terrainset));
 static void init_view_cell PARAMS ((int x, int y));
 static void maybe_init_view_cell PARAMS ((int x, int y));
 static int adj_seen_terrain PARAMS ((int x, int y, Side *side));
+static void init_ranged_views PARAMS ((void));
 
-/* This is true when the set of types has been defined.  Subsequently only
-   scenarios based on those types can be loaded. */
+/* This is true when the set of types has been defined.  Subsequently
+   only scenarios based on those types can be loaded. */
 
 int typesdefined = FALSE;
 
@@ -49,6 +50,10 @@ short tmpttype;
 
 Assign *assignments = NULL;
 
+/* True when no AIs should be set up at game start. */
+
+short initially_no_ai;
+
 /* The list of pathnames to library directories. */
 
 LibraryPath *xconq_libs;
@@ -187,6 +192,7 @@ init_data_structures()
     init_tasks();
     init_ai_types();
     init_write();
+    curdatestr = xmalloc(BUFSIZE);
 }
 
 /* Build the default list of synthesis methods. */
@@ -221,8 +227,6 @@ set_g_synth_methods_default()
 
 /* In theory, if a game passes these tests, then Xconq will never crash. */
 
-/* (This should always be run, might find errors in compiled game.) */
-
 void
 check_game_validity()
 {
@@ -248,6 +252,33 @@ check_game_validity()
 		     g_separation_min(), g_separation_max());
     }
 
+    /* Check all method names in the synthesis list. */
+    {
+      int i, methkey, found = FALSE;
+      Obj *synthlist = g_synth_methods(), *methods, *method;
+
+      for_all_list(synthlist, methods) {
+	method = car(methods);
+	if (symbolp(method)) {
+	    methkey = keyword_code(c_string(method));
+	    for (i = 0; synthmethods[i].key >= 0; ++i) {
+		if (methkey == synthmethods[i].key) {
+		    found = TRUE;
+		    break;
+		}
+	    }
+	} else if (consp(method)) {
+	    if (stringp(car(method))) {
+		/* External program. */
+		run_error("No external synth programs yet");
+	    }
+	}
+	if (!found) {
+	    sprintlisp(spbuf, method, BUFSIZE);
+	    init_warning("bad synthesis method %s, ignoring", spbuf);
+	}
+      }
+    }
     /* (Need more general game checks.) */
 
     max_zoc_range = -1;
@@ -268,10 +299,6 @@ check_game_validity()
 	    }
 	}
     }
-    /* Large ZOC ranges are not implemented. */
-    if (max_zoc_range > 1) {
-	init_warning("ZOC range goes up to %d, may be very inefficient", max_zoc_range);
-    }
     /* (Eventually check material types also.) */
     /* Check that all terrain names and chars are distinct. */
     for_all_terrain_types(t1) {
@@ -389,16 +416,23 @@ calculate_globals()
     }	
 }
 
-/* Clean up all the objects and cross-references. */
+/* Clean up all the objects and cross-references. NOTE: this may be
+   called multiple times!  It should not have any additional effects
+   if called more than once. */
 
 void
 patch_object_references()
 {
-    int numhere, numoccs;
-    Unit *unit, *transport;
+    int numhere, numoccs, t, id;
+    Unit *unit, *transport, *unit2;
     Side *side;
     Obj *utref, *utorig;
 
+    /* This cannot be called as part of prealloc_debug(), since that
+       can occur before any types are defined.  This should happen
+       after all unit types have been defined but before a first save;
+       this is as good a place as any. */
+    shortest_unique_name(0);
     /* Use read-in ids to fill in side slots that point to other objects. */
     for_all_sides(side) {
 	if (side->playerid > 0) {
@@ -414,11 +448,9 @@ patch_object_references()
     for_all_units(unit) {
 	/* It's possible that dead units got read in, so check. */
 	if (alive(unit)) {
-	    if (unit->transport != NULL) {
+	    if (unit->transport_id != lispnil) {
 		transport = NULL;
-		utref = utorig = (Obj *) unit->transport;
-		/* For safety's sake, null out the slot. */
-		unit->transport = NULL;
+		utref = utorig = unit->transport_id;
 		/* We have a Lisp object; use it to identify a particular
 		   unit as the transport. */
 		if (symbolp(utref) && boundp(utref)) {
@@ -434,20 +466,31 @@ patch_object_references()
 		    if (transport == NULL)
 		      init_warning("could not find a transport named \"%s\" for %s",
 				   c_string(utref), unit_desig(unit));
+		} else if (symbolp(utref)) {
+		    transport = find_unit_by_symbol(utref);
+		    if (transport == NULL)
+		      init_warning("could not find a transport with sym \"%s\" for %s",
+				   c_string(utref), unit_desig(unit));
 		} else {
 		    /* not a recognized way to refer to a unit */
-		    sprintlisp(tmpbuf, utorig);
+		    sprintlisp(tmpbuf, utorig, BUFSIZE);
 		    init_warning("could not find transport %s for %s",
 				 tmpbuf, unit_desig(unit));
 		}
-		/* it is important to make sure that unit->x, and unit->y 
-		   are negative at this point.  Otherwise, the coverage will
-		   be messed up for units put into transports that have not yet
-		   been placed.  They will be covered for entering the cell,
-		   and again when the transport enters the cell. */
+		/* It is important to make sure that unit->x, and
+		   unit->y are negative at this point.  Otherwise, the
+		   coverage will be messed up for units put into
+		   transports that have not yet been placed.  They
+		   will be covered for entering the cell, and again
+		   when the transport enters the cell. */
 		if (transport != NULL) {
-		    /* (also check that this is a valid transport type?) */
-		    enter_transport(unit, transport);
+		    if (transport != unit->transport) {
+			if (unit->transport != NULL) {
+			    leave_transport(unit);
+			}
+			/* (also check that this is a valid transport type?) */
+			enter_transport(unit, transport);
+		    }
 		} else {
 		    /* (could let the unit enter the cell, or could
 		       make it infinitely postponed) */
@@ -456,7 +499,11 @@ patch_object_references()
 		/* Check that the unit's location is meaningful. */
 		if (!inside_area(unit->x, unit->y)) {
 		    if (inside_area(unit->prevx, unit->prevy)) {
-			if (can_occupy_cell(unit, unit->prevx, unit->prevy)) {
+			t = terrain_at(unit->prevx, unit->prevy);
+			if ((can_occupy_cell(unit, unit->prevx, unit->prevy)
+			     && !ut_vanishes_on(unit->type, t)
+			     && !ut_wrecks_on(unit->type, t))
+			    || can_occupy_conn(unit, unit->prevx, unit->prevy, unit->z)) {
 			    enter_cell(unit, unit->prevx, unit->prevy);
 			} else {
 			    numhere = numoccs = 0;
@@ -473,9 +520,12 @@ patch_object_references()
 				  ++numoccs;
 			    }
 #if 0
-			    /* Try having all the existing units enter the transport. */
-			    /* (but doesn't work if only some units should go into transport) */
-			    if (!inside_area(unit->x, unit->y) && numoccs == numhere) {
+			    /* Try having all the existing units enter
+                               the transport. */
+			    /* (but doesn't work if only some units
+                               should go into transport) */
+			    if (!inside_area(unit->x, unit->y)
+				&& numoccs == numhere) {
 				for_all_stack(unit->prevx, unit->prevy, unit2) {
 				    enter_transport(unit2, unit);
 				}
@@ -483,19 +533,23 @@ patch_object_references()
 			    }
 #endif
 			    if (!inside_area(unit->x, unit->y)) {
-				init_warning("No room for %s at %d,%d",
+				init_warning("Can't put %s at %d,%d (%s)",
 					     unit_desig(unit),
-					     unit->prevx, unit->prevy);
+					     unit->prevx, unit->prevy,
+					     (can_occupy_cell(unit, unit->prevx, unit->prevy)
+					      ? "deadly terrain" : "no room"));
 			    }
 			}
 			/* This prevents attempts to do a second enter_cell
 			   during initialization. */
 			unit->prevx = unit->prevy = -1;
 		    } else if (unit->prevx == -1 && unit->prevy == -1) {
-		    	/* This will be left alone - should have pos filled in later. */
+		    	/* This will be left alone - should have pos
+                           filled in later. */
 		    } else if (unit->cp >= 0) {
-			/* Warn, but only if there's no good reason for the unit to have
-			   an offworld position. */
+			/* Warn, but only if there's no good reason
+			   for the unit to have an offworld
+			   position. */
 			if (area.fullwidth == 0) {
 			    init_warning("%s is at offworld location, left there",
 					 unit_desig(unit));
@@ -516,13 +570,14 @@ patch_object_references()
 		unit->number = 0;
 	    }
 	    if (completed(unit)) {
-		init_unit_actorstate(unit, TRUE);
-		/* Restore acp that wasn't written out because it was the most normal
-		   value. */
+		if (unit->act == NULL)
+		  init_unit_actorstate(unit, TRUE);
+		/* Restore acp that wasn't written out because it was
+		   the most normal value. */
 		if (unit->act && unit->act->acp < u_acp_min(unit->type))
 		  unit->act->acp = u_acp(unit->type);
-		/* Restore initacp that wasn't written out because it was the normal
-		   value. */
+		/* Restore initacp that wasn't written out because it
+		   was the normal value. */
 		if (unit->act
 		    && unit->act->acp > 0
 		    && unit->act->initacp == 0)
@@ -532,12 +587,26 @@ patch_object_references()
 		    init_unit_plan(unit);
 		}
 	    }
+	    /* Maybe fill in a formation's leader with a unit. */
+	    if (unit->plan != NULL
+		&& unit->plan->formation != NULL) {
+		id = unit->plan->formation->args[0];
+		if (id > 0) {
+		    unit->plan->funit = find_unit(id);
+		    if (unit->plan->funit == NULL) {
+			init_warning("%s missing leader #%d, cancelling formation",
+				     unit_desig(unit), id);
+			unit->plan->formation = NULL;
+		    }
+		}
+	    }
 	} else {
-	    /* Dead units need to be disentangled from anything that might
-	       have been done to them.  For instance, a module might include
-	       a standard collection of units, but then follow up by removing
-	       some of those units, and can do it by setting hp == 0.  We want
-	       this to work consistently and reliably. */
+	    /* Dead units need to be disentangled from anything that
+	       might have been done to them.  For instance, a module
+	       might include a standard collection of units, but then
+	       follow up by removing some of those units, and can do
+	       it by setting hp == 0.  We want this to work
+	       consistently and reliably. */
 	    /* Null this out, any possible unit reference is useless. */
 	    unit->transport = NULL;
 	    if (inside_area(unit->x, unit->y)) {
@@ -609,16 +678,21 @@ make_trial_assignments()
 	}
     }
     /* Add default players for sides with none (in separate loop so player
-       creation doesn't confuse player list traversal above). */
+       creation doesn't confuse player list traversal above).  Don't
+       add for any sides not actually participating in the game. */
     for (i = 0; i < numsides; ++i) {
 	if (assignments[i].side != NULL
+	    && assignments[i].side->ingame
 	    && assignments[i].player == NULL) {
 	    player = add_player();
 	    /* Default players are always AIs (at least for now). */
-	    player->aitypename = "mplayer";
+	    if (!initially_no_ai)
+	      player->aitypename = "mplayer";
 	    trial_assign_player(i, player);
 	}
     }
+    /* This warning can happen if there are more players than active
+       sides. */
     for_all_players(player) {
 	if (player->side == NULL) {
 	    init_warning("Player %s not given a side", player_desig(player));
@@ -652,7 +726,7 @@ Player *player;
 Side *
 make_up_a_side()
 {
-	extern Obj *side_defaults;
+    extern Obj *side_defaults;
 
     Side *side = create_side();
 
@@ -683,6 +757,42 @@ Side *side;
       side->minadvantage = g_advantage_min();
     if (side->maxadvantage == 0)
       side->maxadvantage = g_advantage_max();
+    if (side->maxadvantage == 0) {
+	int i, methkey, found = FALSE;
+	Obj *synthlist = g_synth_methods(), *methods, *method;
+
+	for_all_list(synthlist, methods) {
+	    method = car(methods);
+	    if (symbolp(method)) {
+		methkey = keyword_code(c_string(method));
+		for (i = 0; synthmethods[i].key >= 0; ++i) {
+		    if (methkey == synthmethods[i].key) {
+			found = TRUE;
+			break;
+		    }
+		}
+	    }
+	    if (found) {
+		if (synthmethods[i].key == K_MAKE_COUNTRIES) {
+		    int u, var = FALSE;
+
+		    for_all_unit_types(u) {
+			if (u_start_with(u) > 0) {
+			    var = TRUE;
+			    break;
+			}
+		    }
+		    if (var)
+		      side->maxadvantage = 10;
+		}
+	    }
+	}
+    }
+    /* If we're not getting any guidance from global defaults, just set to 1. */
+    if (side->minadvantage == 0)
+      side->minadvantage = 1;
+    if (side->maxadvantage == 0)
+      side->maxadvantage = side->minadvantage;
 }
 
 /* Add a side and a player to go with it (used by interfaces). */
@@ -760,11 +870,15 @@ run_synth_methods()
     int i, methkey, found = FALSE, rslt;
     Obj *synthlist, *methods, *method, *parms;
 
+    /* Make sure no dead units get saved. */
+    flush_dead_units();
+    /* Make a consistent ordering of units. */
+    sort_units(TRUE);
     synthlist = g_synth_methods();
     Dprintf("Will run syntheses ");
     Dprintlisp(synthlist);
     Dprintf("\n");
-    for (methods = synthlist; methods != lispnil; methods = cdr(methods)) {
+    for_all_list(synthlist, methods) {
 	method = car(methods);
 	if (symbolp(method)) {
 	    methkey = keyword_code(c_string(method));
@@ -789,7 +903,7 @@ run_synth_methods()
 	    }
 	}
 	if (!found) {
-	    sprintlisp(spbuf, method);
+	    sprintlisp(spbuf, method, BUFSIZE);
 	    init_warning("bad synthesis method %s, ignoring", spbuf);
 	}
     }
@@ -815,9 +929,22 @@ int
 make_weather(calls, runs)
 int calls, runs;
 {
-    int x, y, winddir;
-    extern int maxwindforce;	
+    int x, y, winddir, t, var;
+    extern int maxclouds;	
 
+    if (maxclouds > 0) {
+	if (!clouds_defined()) {
+	    allocate_area_clouds();
+	    for_all_cells(x, y) {
+		t = terrain_at(x, y);
+		/* Vary the cloud cover randomly across the range of possibilities. */
+		var = t_clouds_max(t) - t_clouds_min(t);
+		if (var > 0)
+		  var = xrandom(var + 1);
+		set_raw_cloud_at(x, y, t_clouds_min(t) + var);
+	    }
+	}
+    }
     if (maxwindforce > 0) {
 	if (!winds_defined()) {
 	    allocate_area_winds();
@@ -883,6 +1010,7 @@ final_init()
        hereafter will be fatal. */
     check_consistency();
     cache_possible_types();
+    kill_excess_units();
     /* Count the units initially present. */
     init_side_balance();
     /* Fix up garbled view data. */
@@ -893,11 +1021,12 @@ final_init()
     create_game_help_nodes();
     /* Set up the scores to be attached to each side. */
     init_scores();
-    /* Calculate which random event methods will be run. */
-    init_random_events();
     final_init_world();
+    init_run();
     /* Start the recording of history. */
     start_history();
+    /* Compute and cache AI/planning data. */
+    ai_init_shared();
 #ifdef DEBUGGING
     /* Make sure that any debugging-related allocation is done. */
     if (Debug || DebugG || DebugM)
@@ -911,6 +1040,22 @@ final_init()
 #endif /* DEBUGGING */
 }
 
+static void
+kill_excess_units()
+{
+    Unit *unit;
+
+    for_all_units(unit) {
+	/* Offworld units with no scheduled appearance should evaporate. */
+	if (!inside_area(unit->x, unit->y)
+	    && unit_appear_turn(unit) < 0) {
+	    kill_unit(unit, -1);
+	}
+    }
+    /* Make the units' storage available. */
+    flush_dead_units();
+}
+
 /* Load up any player-specified configuration data. */
 
 void
@@ -962,7 +1107,7 @@ init_all_views()
 	}
     }
     finish_lengthy_process();
-    init_area_views();
+    init_ranged_views();
 }
 
 static void
@@ -1017,8 +1162,7 @@ int x, y;
 		set_cover(tmpside, x, y, 1);
 	    	return;
 	    }
-	    chance = (indep(unit) ? u_already_seen_indep(u)
-		      : u_already_seen(u));
+	    chance = already_seen_chance(tmpside, unit);
 	    if (probability(chance)) {
 	    	see_exact(tmpside, x, y);
 	    	/* This view might be overwritten by a view of
@@ -1031,6 +1175,32 @@ int x, y;
     set_wind_view(tmpside, x, y, raw_wind_at(x, y));
 }
 
+int
+already_seen_chance(side, unit)
+Side *side;
+Unit *unit;
+{
+    int u = unit->type, val;
+
+    if (indep(unit)) {
+	if (side != NULL
+	    && side->already_seen_indep != NULL) {
+	    val = side->already_seen_indep[u];
+	    if (val >= 0)
+	      return val;
+	}
+	return u_already_seen_indep(u);
+    } else {
+	if (side != NULL
+	    && side->already_seen != NULL) {
+	    val = side->already_seen[u];
+	    if (val >= 0)
+	      return val;
+	}
+	return u_already_seen(u);
+    }
+}
+
 static void
 maybe_init_view_cell(x, y)
 int x, y;
@@ -1067,8 +1237,8 @@ Side *side;
 
 /* Do ranged initial views from units. */
 
-void
-init_area_views()
+static void
+init_ranged_views()
 {
     int rad, x, y, pop, dir, x1, y1, i = 0;
     Unit *unit;
@@ -1288,6 +1458,8 @@ assign_players_to_sides()
 	announce_progress((100 * i) / numsides);
 	side = assignments[i].side;
 	player = assignments[i].player;
+	if (player == NULL)
+	  continue;
 	canonicalize_player(player);
 	/* Fix any mistaken advantages. */
 	/* This is a warning here because properly-done interfaces shouldn't
@@ -1302,15 +1474,20 @@ assign_players_to_sides()
 			 player->advantage, player_desig(player), side->maxadvantage);
 	    player->advantage = side->maxadvantage;
 	}
+	side->rid = player->rid;
 	/* Call the interface code to initialize the side's display, if
 	   it wants to use one (the interface has to decide). */
 	if (side_wants_display(side)) {
 	    ++numdisplayswanted;
-	    init_ui(side);
-	    if (side_has_display(side)) {
-		++numdisplays;
+	    if (side->rid == my_rid) {
+		init_ui(side);
+	    } else {
+		init_remote_ui(side);
 	    }
 	}
+	if (side_has_display(side)) {
+	    ++numdisplays;
+	}
 	/* Count the desired AIs, for setup below. */
 	if (side_wants_ai(side)) {
 	    ++numais;
@@ -1368,7 +1545,6 @@ prealloc_debug()
     side_desig(NULL);
     player_desig(NULL);
     unit_desig(NULL);
-    shortest_unique_name(0);
 }
 
 /* Return the program version. */
