@@ -1,5 +1,5 @@
 /* Name generation for Xconq.
-   Copyright (C) 1991, 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1991-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -13,11 +13,18 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 
-int totalsideweights;
+Obj *choose_from_weighted_list PARAMS ((Obj *lis, int *totalweightp));
+
+Obj *namerlist;
+
+Obj *lastnamer;
+
+static int totalsideweights;
 
 void
 init_namers()
 {
+    namerlist = lastnamer = lispnil;
     totalsideweights = 0;
 }
 
@@ -52,6 +59,39 @@ set_g_side_lib_default()
     set_g_side_lib(sidenamelist);
 }
 
+Obj *
+choose_from_weighted_list(lis, totalweightp)
+Obj *lis;
+int *totalweightp;
+{
+    int n, sofar;
+    Obj *rest, *head, *rslt;
+
+    if (*totalweightp <= 0) {
+	for_all_list(lis, rest) {
+	    head = car(rest);
+	    *totalweightp +=
+	      ((consp(head) && numberp(car(head))) ? c_number(car(head)) : 1);
+	}
+    }
+    n = xrandom(*totalweightp);
+    sofar = 0;
+    rslt = lispnil;
+    for_all_list(lis, rest) {
+	head = car(rest);
+	sofar += ((consp(head) && numberp(car(head))) ? c_number(car(head)) : 1);
+	if (sofar > n) {
+	    rslt = head;
+	    break;
+	}
+    }
+    /* Remove the weighting if present. */
+    if (consp(rslt) && numberp(car(rslt))) {
+	rslt = cdr(rslt);
+    }
+    return rslt;
+}
+
 /* Pick a side name that is not already being used. */
 
 void
@@ -59,38 +99,19 @@ make_up_side_name(side)
 Side *side;
 {
     int uniq = FALSE, tries = 0, n, sofar;
-    Obj *sidelib, *subobj, *subelts, *lis, *filler;
+    Obj *sidelib, *subobj, *subelts, *head, *filler;
 
-    if ((sidelib = g_side_lib()) == lispnil)
+    sidelib = g_side_lib();
+    if (sidelib == lispnil)
       return;
-    /* (what if all weights were explicitly 0?) */
-    if (totalsideweights == 0) {
-	for (lis = sidelib; lis != lispnil; lis = cdr(lis)) {
-	    totalsideweights +=
-	      (numberp(car(car(lis))) ? c_number(car(car(lis))) : 1);
-	}
-    }
     filler = lispnil;
     while (tries++ < 100 * numsides) {
-	n = xrandom(totalsideweights);
-	sofar = 0;
-	subobj = lispnil;
-	for (lis = sidelib; lis != lispnil; lis = cdr(lis)) {
-	    sofar += (numberp(car(car(lis))) ? c_number(car(car(lis))) : 1);
-	    if (sofar > n) {
-		subobj = car(lis);
-		break;
-	    }
-	}
-	/* (should scan and preprocess subobj before using it) */
-	/* Remove the weighting if present. */
-	if (numberp(car(subobj))) {
-	    subobj = cdr(subobj);
-	}
+	subobj = choose_from_weighted_list(sidelib, &totalsideweights);
 	uniq = TRUE;
-	for (subelts = subobj; subelts != lispnil; subelts = cdr(subelts)) {
-	    if (stringp(cadr(car(subelts)))) {
-		if (name_in_use(side, c_string(cadr(car(subelts))))) {
+	for_all_list(subobj, subelts) {
+	    head = car(subelts);
+	    if (stringp(cadr(head))) {
+		if (name_in_use(side, c_string(cadr(head)))) {
 		    uniq = FALSE;
 		    break;
 		}
@@ -133,8 +154,16 @@ Obj *
 make_namer(sym, meth)
 Obj *sym, *meth;
 {
-    Obj *namer = new_pointer(sym, (char *) meth);
+    Obj *namer = new_pointer(sym, (char *) meth), *tmpobj;
 
+    /* Append the new namer to the end of the list of namers. */
+    tmpobj = cons(namer, lispnil);
+    if (lastnamer != lispnil) {
+	set_cdr(lastnamer, tmpobj);
+	lastnamer = tmpobj;
+    } else {
+	namerlist = lastnamer = tmpobj;
+    }
     return namer;
 }
 
@@ -235,10 +264,10 @@ assign_unit_number(unit)
 Unit *unit;
 {
     if (u_assign_number(unit->type)
-	&& unit->side != NULL
-	&& unit->number == 0) {
-	/* Give it the next available number and increment. */
-	unit->number = (unit->side->counts)[unit->type]++;
+	&& unit->side != NULL) {
+	/* If unnumbered, give it the next available number and increment. */
+	if (unit->number == 0)
+	  unit->number = (unit->side->counts)[unit->type]++;
     } else {
 	/* Note that this will erase any already-assigned number,
 	   if the type is one that is not supposed to be numbered. */
