@@ -1,6 +1,5 @@
 /* Unit generation for Xconq.
-   Copyright (C) 1986, 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995
-   Stanley T. Shebs.
+   Copyright (C) 1986-1989, 1991-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -8,9 +7,10 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
+Obj *choose_from_weighted_list PARAMS ((Obj *lis, int *totalweightp));
+#include "kernel.h"
 
-static void set_people_on_side PARAMS ((int x, int y));
-static void expand_country PARAMS ((int x, int y));
+static void add_preexisting_units PARAMS ((Side *side, int runs));
 static int country_is_complete PARAMS ((Side *side, int runs));
 static int average_numunits PARAMS ((int u));
 static void count_cells PARAMS ((int x, int y));
@@ -22,6 +22,11 @@ static int find_unit_place PARAMS ((int u, int cx, int cy, int *xp, int *yp));
 static void mung_terrain PARAMS ((int x, int y, int u));
 static Unit *find_occupant_place PARAMS ((Side *side, int u, int x0, int y0));
 static int at_country_units_max PARAMS ((Side *side, int u));
+static void place_unit PARAMS ((int u, Side *side, int x, int y,
+				Unit *transport));
+static void expand_countries PARAMS ((void));
+static void expand_country PARAMS ((int x, int y));
+static void set_people_on_side PARAMS ((int x, int y));
 
 /* Tmp array for counting terrain. */
 
@@ -53,127 +58,27 @@ int sidedeltahalf;
 
 static int growth;
 
-/* Set the people inside the country to be on the country's side.
-   Flip a coin along the edges, to make the border more interesting.
-   Also don't always overwrite existing people. */
-
-static void
-set_people_on_side(x, y)
-int x, y;
-{
-    if ((distance(tmpside->startx, tmpside->starty, x, y) <= tmpradius
-         || flip_coin())
-        && probability(t_country_people(terrain_at(x, y)))
-        && (people_side_at(x, y) == NOBODY || flip_coin())) {
-	set_people_side_at(x, y, side_number(tmpside));
-    }
-}
-
-/* Expand a country into the given cell. */
-
-static void
-expand_country(x, y)
-int x, y;
-{
-    int u, t = terrain_at(x, y), ours, theirs, prob;
-    Unit *unit;
-    
-    if (people_side_at(x, y) == NOBODY) {
-	prob = t_country_growth(t);
-    } else {
-	prob = t_country_takeover(t);
-    }
-    if (probability(prob)) {
-	/* Claim the cell, attempt to own it. */
-	ours = theirs = FALSE;
-	if (unit_at(x, y) == NULL) {
-	    /* If completely empty, we claim it. */
-	    ours = TRUE;
-	    for_all_unit_types(u) {
-	    	tmputype = u;
-		if (xrandom(10000) < u_unit_growth(u)
-		    && !at_country_units_max(tmpside, u)
-		    && probability(ut_favored(u, t))
-		    && valid_unit_place(x, y)) {
-		    unit = create_unit(u, TRUE);
-		    if (unit != NULL) {
-			set_unit_side(unit, tmpside);
-			set_unit_origside(unit, tmpside);
-			init_supply(unit);
-			enter_cell(unit, x, y);
-		    	break;
-		    }
-		}
-		if (xrandom(10000) < u_indep_growth(u)
-		    && !at_country_units_max(tmpside, u)
-		    && probability(ut_favored(u, t))
-		    && valid_unit_place(x, y)) {
-		    unit = create_unit(u, TRUE);
-		    if (unit != NULL) {
-			init_supply(unit);
-			enter_cell(unit, x, y);
-		    	break;
-		    }
-		}
-	    }
-	} else {
-	    /* Somebody is here already. */
-	    for_all_stack(x, y, unit) {
-		if (unit->side != tmpside) {
-		    if (unit->side != NULL) {
-			/* Maybe take over another side's unit. */
-			if (xrandom(10000) < u_unit_takeover(unit->type)
-			    && !at_country_units_max(tmpside, unit->type)
-			    ) {
-			    set_unit_side(unit, tmpside);
-			    set_unit_origside(unit, tmpside);
-			    ours = TRUE;
-			} else {
-			    /* Indicate that we don't have possession of every
-			       unit in this cell. */
-			    theirs = TRUE;
-			}
-		    } else {
-			/* Maybe take over an independent unit. */
-			if (xrandom(10000) < u_indep_takeover(unit->type)
-			    && !at_country_units_max(tmpside, unit->type)
-			    ) {
-			    set_unit_side(unit, tmpside);
-			    set_unit_origside(unit, tmpside);
-			    ours = TRUE;
-			}
-		    }
-		}
-	    }
-	}
-	/* If cell is claimed, and has no opposition, then country gets it. */
-	if (ours && !theirs && probability(t_country_people(t))) {
-	    set_people_side_at(x, y, side_number(tmpside));
-	}
-	/* Count this cell as one we expanded into. */
-	++growth;
-    }
-    /* The cell should become known even if not occupied, but we can't
-       do that here because view structures don't exist yet. */
-}
+static int totnumtodo;
+static int totnumdone;
 
 #define announce_unit_progress(n) \
-  announce_progress(sideprogress + sidedeltahalf + (sidedeltahalf * (n)) / totnumtodo)
+  
 
 /* Place all the units belonging to countries. */
 
+int any_side_possible_units = -1;
+
 int
 make_countries(calls, runs)
 int calls, runs;
 {
-    int x0, y0, u, t, i, tot, x, y;
+    int alldone, x0, y0, u, t, i, tot, x, y;
     int advantage, sideadvantage, favor, x1, y1, dir;
     int canbeinopen[MAXUTYPES];  /* true if type need not be occ at start */
-    int numtodo[MAXUTYPES], numindeptodo[MAXUTYPES], totnumtodo, totnumdone;
+    int numtodo[MAXUTYPES], numindeptodo[MAXUTYPES];
     int numleft[MAXUTYPES], numindepleft[MAXUTYPES], totleft, numlisted, numfails;
     int checkmins = FALSE;
     int dopeoplesides = FALSE, dopeopleindeps = FALSE;
-    int maxradius;
     int numdone = 0;
     Unit *unit, *transport;
     Side *side;
@@ -182,6 +87,20 @@ int calls, runs;
     /* Run this always, unless something important is missing. */
     if (!terrain_defined())
       return FALSE;
+    /* Add any units already in the world that should belong to a side. */
+    for_all_sides(side) {
+	add_preexisting_units(side, runs);
+    }
+    /* Scan countries to see if done already. */
+    alldone = TRUE;
+    for_all_sides(side) {
+	if (!country_is_complete(side, runs)) {
+	    alldone = FALSE;
+	    break;
+	}
+    }
+    if (alldone)
+      return FALSE;
     /* Calculate whether the minimum required terrain types are present. */
     totnumcells = (int *) xmalloc(numttypes * sizeof(int));
     for_all_terrain_types(t) {
@@ -266,9 +185,10 @@ int calls, runs;
 		numtodo[u] = numindeptodo[u] = 0;
 		if (type_allowed_on_side(u, side)) {
 		    if (runs > 0) {
-			/* If the side is just coming into the game, give it an
-			   average number of each type that the existing sides
-			   started out with, so that the new side has some parity. */
+			/* If the side is just coming into the game,
+			   give it an average number of each type that
+			   the existing sides started out with, so
+			   that the new side has some parity. */
 			if (u_start_with(u) > 0 || u_indep_near_start(u) > 0)
 			  numtodo[u] = (average_numunits(u) * advantage) / sideadvantage;
 		    } else {
@@ -282,19 +202,12 @@ int calls, runs;
 		}
 		numleft[u] = numindepleft[u] = 0;
 	    }
-	    /* First do units actually belonging to the side initially. */
+	    /* First place units actually belonging to the side initially. */
 	    for_all_unit_types(u) {
 		if (canbeinopen[u]) {
 		    for (i = 0; i < numtodo[u]; ++i) {
 			if (find_unit_place(u, x0, y0, &x, &y)) {
-			    unit = create_unit(u, TRUE);
-			    if (unit != NULL) {
-				set_unit_side(unit, side);
-				set_unit_origside(unit, side);
-				init_supply(unit);
-				enter_cell(unit, x, y);
-	   			announce_unit_progress(++totnumdone);
-			    }
+			    place_unit(u, side, x, y, NULL);
 			} else {
 			    /* If can't find places for this type, give up. */
 		            numleft[u] = numtodo[u] - i;
@@ -307,12 +220,7 @@ int calls, runs;
 	    for_all_unit_types(u) {
 		for (i = 0; i < numindeptodo[u]; ++i) {
 		    if (find_unit_place(u, x0, y0, &x, &y)) {
-			unit = create_unit(u, TRUE);
-			if (unit != NULL) {
-			    init_supply(unit);
-			    enter_cell(unit, x, y);
-			    announce_unit_progress(++totnumdone);
-			}
+			place_unit(u, NULL, x, y, NULL);
 		    } else {
 			/* If can't find places for this type, give up. */
 			numindepleft[u] = numindeptodo[u] - i;
@@ -328,14 +236,7 @@ int calls, runs;
 		    for (i = 0; i < numtodo[u]; ++i) {
 			if ((transport = find_occupant_place(side, u, x0, y0))
 			    != NULL) {
-			    unit = create_unit(u, TRUE);
-			    if (unit != NULL) {
-				set_unit_side(unit, side);
-				set_unit_origside(unit, side);
-				init_supply(unit);
-				enter_transport(unit, transport);
-	   			announce_unit_progress(++totnumdone);
-			    }
+			    place_unit(u, side, x0, y0, transport);
 			} else {
 		   	    numleft[u] = numtodo[u] - i;
 			    break;
@@ -362,7 +263,7 @@ int calls, runs;
 			     tmpbuf2, (numfails >= 5 ? " etc" : ""));
 	    }
 	}
-	/* Now set the side of the people in this country. */
+	/* Set the side of the people in this country. */
 	tmpside = side;
 	apply_to_area(side->startx, side->starty, tmpradius + 1,
 		      set_people_on_side);
@@ -376,64 +277,7 @@ int calls, runs;
     }
     finish_lengthy_process();
     /* Grow each country out to a maximum radius. */
-    announce_lengthy_process("Growing countries");
-    maxradius = min(area.width, g_radius_max());
-    for_all_sides(side)
-      side->finalradius = maxradius;
-    for (i = baseradius; i < maxradius; ++i) {
-    	announce_progress((100 * (i - baseradius)) / (maxradius - baseradius));
-	for_all_sides(side) {
-	    /* If side still allowed to grow, expand it. */
-	    if (side->finalradius == maxradius) {
-	    	tmpside = side;
-	    	growth = 0;
-	    	apply_to_ring(side->startx, side->starty, i - 2, i,
-			      expand_country);
-		/* If no actual growth happened in a step, the country
-		   might have reached its natural size. */
-	    	if (growth == 0 && probability(g_growth_stop())) {
-		    side->finalradius = i;
-	    	}
-	    }
-	}
-    }
-    /* Do a couple "consolidation" steps to fill in "jaggies". */
-    for (i = 0; i < 2; ++i) {
-	for_all_sides(side) {
-	    tmpside = side;
-	    apply_to_ring(side->startx, side->starty,
-			  baseradius, side->finalradius - 1,
-			  expand_country);
-	}
-    }
-    /* Also remove isolated populations surrounded by one other side. */
-    if (people_sides_defined()) {
-	for_all_interior_cells(x, y) {
-	    int pop, pop2, majoritypop;
-	    
-	    if ((pop = people_side_at(x, y)) != NOBODY) {
-		majoritypop = NOBODY;
-		for_all_directions(dir) {
-		    if (interior_point_in_dir(x, y, dir, &x1, &y1)) {
-			if ((pop2 = people_side_at(x1, y1)) == pop)
-			  goto nextcell;
-			if (majoritypop == NOBODY) {
-			    majoritypop = pop2;
-			} else {
-			    if (pop2 != majoritypop)
-			      goto nextcell;
-			}
-		    }
-		}
-		if (majoritypop != NOBODY) {
-		    set_people_side_at(x, y, majoritypop);
-		}
-	    }
-	  nextcell:
-	    pop = pop;  /* a dummy statement */
-	}
-    }
-    finish_lengthy_process();
+    expand_countries();
     /* Warn about any difficulties encountered. */
     if (badcountryplaces > 0) {
     	init_warning("%d of %d sides have undesirable locations",
@@ -455,7 +299,8 @@ int runs;
     int sideadvantage = max(1, side->advantage);
     int advantage = (side->player ? side->player->advantage : sideadvantage);
 
-    /* If side no longer in game, then we don't want to return country synth on it. */
+    /* If side no longer in game, then we don't want to do country
+       synth on it. */
     if (!side->ingame)
       return TRUE;
     totunits = 0;
@@ -468,9 +313,9 @@ int runs;
 	    ++numunits[unit->type];
 	}
     }
-    /* See if we're re-running side generation for the benefit of adding a
-       new side, use the total counts of units to distinguish the new side
-       from the old sides. */ 
+    /* See if we're re-running side generation for the benefit of
+       adding a new side, use the total counts of units to distinguish
+       the new side from the old sides. */
     if (runs > 0)
       return (totunits > 0);
     for_all_unit_types(u) {
@@ -480,6 +325,84 @@ int runs;
     return TRUE;
 }
 
+static void
+add_preexisting_units(side, runs)
+Side *side;
+int runs;
+{
+    int tries, total;
+    Obj *possunits, *chosen, *lis, *rest, *head;
+    Unit *unit;
+
+    if (any_side_possible_units < 0) {
+	any_side_possible_units = FALSE;
+	for_all_units(unit) {
+	    lis = get_x_property(unit, K_SIDES);
+	    if (lis != lispnil) {
+		any_side_possible_units = TRUE;
+	    }
+	}
+    }
+    if (!country_is_complete(side, runs)) {
+	if ((side->possible_units != lispnil && consp(side->possible_units))
+	    || any_side_possible_units) {
+	    /* Collect a list of possible units for this side. */
+	    possunits = side->possible_units;
+	    if (any_side_possible_units) {
+		for_all_units(unit) {
+		    lis = get_x_property(unit, K_SIDES);
+		    if ((stringp(lis) || symbolp(lis))
+			&& side == find_side_by_name(c_string(lis))) {
+			possunits = cons(new_number(unit->id), possunits);
+		    } else if (listp(lis)) {
+			for_all_list(lis, rest) {
+			    head = car(rest);
+			    if ((stringp(head) || symbolp(lis))
+				&& side == find_side_by_name(c_string(head))) {
+				possunits = cons(new_number(unit->id), possunits);
+			    }
+			    /* Note that we ignore bogus-looking data in the
+			       list, it may be side info for some other game. */
+			}
+		    } else {
+			init_warning("Unit %s sides prop invalid, ignoring",
+				     unit_desig(unit));
+		    }
+		}
+	    }
+	    tries = 1000;
+	    /* (should pay attention to need for unit type before collecting) */
+	    while (possunits != lispnil && !country_is_complete(side, runs)) {
+		/* Just to ensure no infinite loop... */
+		if (--tries < 0)
+		  break;
+		total = 0;
+		chosen = choose_from_weighted_list(possunits, &total);
+		possunits = remove_from_list(chosen, possunits);
+		if (Debug) {
+		    Dprintf("For %s, chose ", side_desig(side));
+		    Dprintlisp(chosen);
+		    Dprintf(" from ");
+		    Dprintlisp(possunits);
+		    Dprintf(" possible units\n");
+		}
+		if (stringp(chosen)) {
+		    unit = find_unit_by_name(c_string(chosen));
+		} else if (numberp(chosen)) {
+		    unit = find_unit(c_number(chosen));
+		} else {
+		    unit = NULL;
+		}
+		if (unit != NULL) {
+		    set_unit_side(unit, side);
+		    set_unit_origside(unit, side);
+		    init_supply(unit);
+		}
+	    }
+	}
+    }
+}
+
 /* Compute the average number of units of a given type that are currently
    in the game. */
 
@@ -646,7 +569,7 @@ int x, y;
 }
 
 /* Find a place somewhere in the designated area, following constraints
-   on terrain and adjacency.  Returns success of placement. */
+   on terrain and adjacency.  Returns success of the search. */
 
 static int
 find_unit_place(u, cx, cy, xp, yp)
@@ -731,6 +654,191 @@ int u, x0, y0;
     return NULL;
 }
 
+static void
+place_unit(u, side, x, y, transport)
+int u, x, y;
+Side *side;
+Unit *transport;
+{
+    Unit *unit;
+
+    unit = create_unit(u, TRUE);
+    if (unit != NULL) {
+	if (side != NULL) {
+	    set_unit_side(unit, side);
+	    set_unit_origside(unit, side);
+	}
+	init_supply(unit);
+	if (transport != NULL)
+	  enter_transport(unit, transport);
+	else
+	  enter_cell(unit, x, y);
+	announce_progress(sideprogress + sidedeltahalf + (sidedeltahalf * (++totnumdone)) / totnumtodo);
+    }
+}
+
+static void
+expand_countries()
+{
+    int i, maxradius, growth, x, y, dir, x1, y1;
+    int pop, pop2, majoritypop, broken;
+    Side *side;
+
+    announce_lengthy_process("Growing countries");
+    maxradius = min(area.width, g_radius_max());
+    for_all_sides(side)
+      side->finalradius = maxradius;
+    for (i = baseradius; i < maxradius; ++i) {
+    	announce_progress((100 * (i - baseradius)) / (maxradius - baseradius));
+	for_all_sides(side) {
+	    /* If side still allowed to grow, expand it. */
+	    if (side->finalradius == maxradius) {
+	    	tmpside = side;
+	    	growth = 0;
+	    	apply_to_ring(side->startx, side->starty, i - 2, i,
+			      expand_country);
+		/* If no actual growth happened in a step, the country
+		   might have reached its natural size. */
+	    	if (growth == 0 && probability(g_growth_stop())) {
+		    side->finalradius = i;
+	    	}
+	    }
+	}
+    }
+    /* Do a couple "consolidation" steps to fill in "jaggies". */
+    for (i = 0; i < 2; ++i) {
+	for_all_sides(side) {
+	    tmpside = side;
+	    apply_to_ring(side->startx, side->starty,
+			  baseradius, side->finalradius - 1,
+			  expand_country);
+	}
+    }
+    /* Also remove isolated populations surrounded by one other side. */
+    if (people_sides_defined()) {
+	for_all_interior_cells(x, y) {
+	    if ((pop = people_side_at(x, y)) != NOBODY) {
+		majoritypop = NOBODY;
+		broken = FALSE;
+		for_all_directions(dir) {
+		    if (interior_point_in_dir(x, y, dir, &x1, &y1)) {
+			if ((pop2 = people_side_at(x1, y1)) == pop) {
+			    broken = TRUE;
+			    break;
+			} else if (majoritypop == NOBODY) {
+			    majoritypop = pop2;
+			} else if (pop2 != majoritypop) {
+			    broken = TRUE;
+			    break;
+			}
+		    }
+		}
+		if (broken)
+		  break;
+		if (majoritypop != NOBODY)
+		  set_people_side_at(x, y, majoritypop);
+	    }
+	}
+    }
+    finish_lengthy_process();
+}
+
+/* Expand a country into the given cell. */
+
+static void
+expand_country(x, y)
+int x, y;
+{
+    int u, t = terrain_at(x, y), ours, theirs, prob, prob2;
+    int advantage, sideadvantage;
+    Unit *unit;
+    
+    if (people_side_at(x, y) == NOBODY) {
+	prob = t_country_growth(t);
+    } else {
+	prob = t_country_takeover(t);
+    }
+    if (probability(prob)) {
+	/* Claim the cell, attempt to own it. */
+	ours = theirs = FALSE;
+	if (unit_at(x, y) == NULL) {
+	    /* If completely empty, we claim it. */
+	    ours = TRUE;
+	    for_all_unit_types(u) {
+	    	tmputype = u;
+		sideadvantage = max(1, tmpside->advantage);
+		advantage = (tmpside->player ? tmpside->player->advantage : sideadvantage);
+		/* Note that while theoretically this multiplication won't work
+		   if the product is > 100%, in practice the growth probabilities are
+		   going to be well under 10%. */
+		prob2 = advantage * u_unit_growth(u);
+		if (xrandom(10000) < prob2
+		    && !at_country_units_max(tmpside, u)
+		    && probability(ut_favored(u, t))
+		    && valid_unit_place(x, y)) {
+		    unit = create_unit(u, TRUE);
+		    if (unit != NULL) {
+			set_unit_side(unit, tmpside);
+			set_unit_origside(unit, tmpside);
+			init_supply(unit);
+			enter_cell(unit, x, y);
+		    	break;
+		    }
+		}
+		prob2 = advantage * u_indep_growth(u);
+		if (xrandom(10000) < prob2
+		    && !at_country_units_max(tmpside, u)
+		    && probability(ut_favored(u, t))
+		    && valid_unit_place(x, y)) {
+		    unit = create_unit(u, TRUE);
+		    if (unit != NULL) {
+			init_supply(unit);
+			enter_cell(unit, x, y);
+		    	break;
+		    }
+		}
+	    }
+	} else {
+	    /* Somebody is here already. */
+	    for_all_stack(x, y, unit) {
+		if (unit->side != tmpside) {
+		    if (unit->side != NULL) {
+			/* Maybe take over another side's unit. */
+			if (xrandom(10000) < u_unit_takeover(unit->type)
+			    && !at_country_units_max(tmpside, unit->type)
+			    ) {
+			    set_unit_side(unit, tmpside);
+			    set_unit_origside(unit, tmpside);
+			    ours = TRUE;
+			} else {
+			    /* Indicate that we don't have possession of every
+			       unit in this cell. */
+			    theirs = TRUE;
+			}
+		    } else {
+			/* Maybe take over an independent unit. */
+			if (xrandom(10000) < u_indep_takeover(unit->type)
+			    && !at_country_units_max(tmpside, unit->type)
+			    ) {
+			    set_unit_side(unit, tmpside);
+			    set_unit_origside(unit, tmpside);
+			    ours = TRUE;
+			}
+		    }
+		}
+	    }
+	}
+	/* If cell is claimed, and has no opposition, then country gets it. */
+	if (ours && !theirs && probability(t_country_people(t))) {
+	    set_people_side_at(x, y, side_number(tmpside));
+	}
+	/* Count this cell as one we expanded into. */
+	++growth;
+    }
+    /* The cell should become known even if not occupied, but we can't
+       do that here because view structures don't exist yet. */
+}
+
 static int
 at_country_units_max(side, u)
 Side *side;
@@ -814,3 +922,19 @@ int calls, runs;
     finish_lengthy_process();
     return TRUE;
 }
+
+/* Set the people inside the country to be on the country's side.
+   Flip a coin along the edges, to make the border more interesting.
+   Also don't always overwrite existing people. */
+
+static void
+set_people_on_side(x, y)
+int x, y;
+{
+    if ((distance(tmpside->startx, tmpside->starty, x, y) <= tmpradius
+         || flip_coin())
+        && probability(t_country_people(terrain_at(x, y)))
+        && (people_side_at(x, y) == NOBODY || flip_coin())) {
+	set_people_side_at(x, y, side_number(tmpside));
+    }
+}
