@@ -1,5 +1,5 @@
 /* Definitions for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1992-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -49,6 +49,8 @@ any later version.  See the file COPYING.  */
 #include <Devices.h>
 #endif /* MPW */
 
+#include <QDOffscreen.h>
+
 #include <GestaltEqu.h>
 #include <PPCToolbox.h>
 #include <AppleEvents.h>
@@ -103,20 +105,20 @@ enum {
 #define kCreateFolder true
 #define kDontCreateFolder false
 
-#define kSystemFolderType				'macs'		/* the system folder */
+#define kSystemFolderType				'macs'	/* the system folder */
 #define kDesktopFolderType				'desk'		/* the desktop folder; objects in this folder show on the desk top. */
 #define kTrashFolderType				'trsh'		/* the trash folder; objects in this folder show up in the trash */
-#define kWhereToEmptyTrashFolderType	'empt'		/* the "empty trash" folder; Finder starts empty from here down */
+#define kWhereToEmptyTrashFolderType	'empt'	/* the "empty trash" folder; Finder starts empty from here down */
 
-#define kPrintMonitorDocsFolderType		'prnt'		/* Print Monitor documents */
+#define kPrintMonitorDocsFolderType		'prnt'	/* Print Monitor documents */
 
 #define kStartupFolderType				'strt'		/* Finder objects (applications, documents, DAs, aliases, to...) to open at startup go here */
-#define kAppleMenuFolderType			'amnu'		/* Finder objects to put into the Apple menu go here */
-#define kControlPanelFolderType 		'ctrl'		/* Control Panels go here (may contain INITs) */
+#define kAppleMenuFolderType			'amnu'	/* Finder objects to put into the Apple menu go here */
+#define kControlPanelFolderType 			'ctrl'		/* Control Panels go here (may contain INITs) */
 #define kExtensionFolderType			'extn'		/* Finder extensions go here */
-#define	kFontsFolderType				'font'		/* Fonts go here */
+#define kFontsFolderType				'font'		/* Fonts go here */
 #define kPreferencesFolderType			'pref'		/* preferences for applications go here */
-#define kTemporaryFolderType			'temp'		/* temporary files go here (deleted periodically, but don't rely on it.) */
+#define kTemporaryFolderType			'temp'	/* temporary files go here (deleted periodically, but don't rely on it.) */
 };
 
 pascal OSErr FindFolder(short vRefNum,OSType folderType,Boolean createFolder,
@@ -182,8 +184,6 @@ enum tooltype {
    attached displays. */
 
 typedef struct a_ui {
-	short followaction;	   /* move to where a change has occured */
-	int itertime;
 	int active;
 } UI;
 
@@ -192,44 +192,42 @@ typedef struct a_ui {
 #define MAXSELECTIONS 500
 
 typedef struct a_map {
-	VP *vp;                 /* This map's generic view parameters */
-	short osx, osy;         /* Tracks shifted origin so can be restored */
+	short id;				/* Identifying number for the map */
+	VP *vp;                 		/* This map's generic view parameters */
+	short osx, osy;         		/* Tracks shifted origin so can be restored */
 	WindowPtr window;
 	ControlHandle hscrollbar;
 	ControlHandle vscrollbar;
+	/* Offscreen graphics support. */
+	CGrafPtr windowPortPtr;
+	GDHandle deviceHdl;
+	GWorldPtr gworldPortPtr;
+	QDErr qdErr;
+	short offsetx;			/* Displacement (right, down) in pixels of map window within its gworld */
+	short offsety;			/* offsetx = offsety = 0 means the window is in the center of its gworld */
 	/* How to draw the map. */
 	short conw;
 	short toplineh;
 	short topunith;
 	short toph;
-	short drawterrain;		/* Display solid color terrain on the map? */
-	short auxterraintypes[MAXTTYPES];
-	short drawauxterrain[MAXTTYPES];
-	short drawgrid;		    /* Draw outlines around cells? */
-	short drawcellpats;		/* Draw terrain patterns? */
-	short drawothermaps;	/* Draw outlines of other maps? */
-	short drawunits;		/* Draw units on the map? */
-	short drawnames;		/* Draw unit names/numbers on the map? */
-	short oldesttoshow;	    /* the relative time of the oldest data */
-	short agetofade;		/* point at which to gray out the display */
-	short newesttoshow;	    /* the relative time of the newest data */
-	short fadeterrain;		/* fade world features as well as units? */
-	short lowestaltitude;   /* the lowest altitude of units to display */
-	short highestaltitude;	/* the highest altitude of units to display */
-	short drawborders;
-	short drawconnections;
-	short drawpeople;		/* Draw people sides on the map? */
-	short drawelevations;	/* Draw elevations on the map? */
-	short drawmaterials[MAXMTYPES];
-	short nummaterialstodraw;
-	short drawlighting;		/* Draw day/night on the map? */
-	short drawtemperature;	/* Draw temperatures on the map? */
-	short drawwinds;
-	short drawclouds;
-	short drawstorms;
-	short drawplans;
-	short drawai;
-	short drawcover;
+	short bufx;				/* Actual size of horizontal offscreen scrolling buffer */
+	short bufy;				/* Actual size of vertical offscreen scrolling buffer */
+	short max_bufx;			/* Max size of horizontal offscreen scrolling buffer as set in dialog */
+	short max_bufy;			/* Max size of vertical offscreen scrolling buffer as set in dialog */
+	short featureborders;	/* Draw feature/province borders */
+	short featurenames;		/* Draw feature/province names */
+	short simple_borders;	/* Do NOT draw borders to sea & empty land */
+	short shorelines;		/* Draw 3D shorelines */
+	short sidecolors;		/* Use side colors for icons */
+	short iconmasks;		/* Use masks when drawing units */
+	short boxmasks;			/* Use masks when drawing grouping boxes */
+	short textmasks;		/* Use masks when drawing text */
+	short optimize_fonts;	/* Use special font set for maps */
+	short solid_color_terrain;	/* Draw terrain in solid colors */
+	short draw_topline;		/* Draw top line */
+	short draw_topunit;		/* Draw top unit info */	
+	short erase_names;		/* Draw row in update_cell_display to erase name of moving unit */	
+	short see_all;
 	short autoselect;
 	Unit *curunit;
 	short moveonclick;
@@ -250,9 +248,11 @@ typedef struct a_map {
 #define MAXINLIST 500
 
 typedef struct a_list {
-    SideMask sides;
+	short sidecolors;		/* Use side colors for units */
+	SideMask sides;			/* Sides to show the units of */
+	short completed_units;
+	short incomplete_units;
 	int mainsortmi;
-	int listglimpsed;              /* list glimpsed units also? */
 	enum sortkeys sortkeys[MAXSORTKEYS];  /* attributes to sort list elements on */
 	UnitVector *contents;
 	int numunits;
@@ -260,9 +260,9 @@ typedef struct a_list {
 	int lastvisible;
 	int firstvisfield;
 	int lastvisfield;
-	int largeicons;
-	int shouldreorg;
-	int entryspacing;
+	short largeicons;
+	short shouldreorg;
+	short entryspacing;
 	WindowPtr window;
 	ControlHandle hscrollbar;
 	ControlHandle vscrollbar;
@@ -277,6 +277,11 @@ typedef struct a_unit_closeup {
 	struct a_unit_closeup *next;
 } UnitCloseup;
 
+/* Iteration over all windows. */
+
+#define for_all_windows(w)  \
+  for ((w) = FrontWindow(); (w) != NULL; (w) = ((GrafPtr) ((WindowPeek)(w))->nextWindow))
+
 /* Iteration over all of a side's maps. */
 
 #define for_all_maps(m)  \
@@ -324,6 +329,7 @@ enum {
 enum {
 	NO_MODAL,
 	MOVE_TO_MODAL,
+	ATTACK_MODAL,
 	FIRE_MODAL,
 	FIRE_INTO_MODAL,
 	SET_FORMATION_MODAL,
@@ -393,7 +399,7 @@ extern ImageFamily **uimages;
 extern ImageFamily **timages;
 extern ImageFamily **eimages;
 
-extern ImageColor **tcolors;
+extern MacImage **tcolors;
 
 extern WindowPtr helpwin;
 extern struct a_helpnode *curhelpnode;
@@ -437,8 +443,9 @@ extern int sbarwid;
 
 extern WindowPtr *winmenuwins;
 
-extern int default_draw_grid;		   /* Display grid on the map? */
-extern int default_draw_names;		   /* Display unit names/numbers on the map? */
+extern int default_draw_grid;
+extern int default_draw_names;
+extern int default_draw_latlong;
 
 extern int map_modal;
 
@@ -490,7 +497,8 @@ extern CursHandle opencrosscursor;
 extern CursHandle watchcursor;
 extern CursHandle firecursor;
 
-extern char *curdatestr;
+extern CursHandle sendcursor;
+extern CursHandle receivecursor;
 
 extern RGBColor graycolor, blackcolor;
 
@@ -507,7 +515,7 @@ extern int tophgt;
 
 extern int first_windows;
 
-extern int daynight;
+extern short daynight;
 extern int grid_matches_unseen;
 
 extern BitMap *bordbitmaps;
@@ -528,6 +536,109 @@ extern int Profile;
 
 extern TEHandle command_text;
 
+extern TEHandle run_length_text;
+
+extern int gameinited;
+extern int position_set_modally;
+extern Point modal_point;
+extern Map *modal_map;
+
+extern short pref_color_unit_images;
+extern short pref_solid_color_terrain;
+
+extern short small_font_id;
+extern short small_font_size;
+extern short small_line_spacing;
+extern short large_font_id;
+extern short large_font_size;
+extern short large_line_spacing;
+extern short title_font_id;
+extern short title_font_size;
+extern short title_line_spacing;
+
+extern int gamewinw;
+extern int topunithgt;
+
+extern int default_latlong_interval;
+
+extern int use_colornames;			/* Save and read colors and colorschemes by name */
+extern int use_RGB_values;			/* Save and read RGB colors in XCol resource */
+extern int show_instructions;		/* Show instructions window at start */
+extern int fullsize_map;			/* Display the first map at full screen size */
+extern int unified_cicns;			/* Use BitMap as mask for color PixMap */
+
+extern int default_draw_topline;		/* Draw top line box (toggle function declared elsewhere) */
+extern int default_draw_topunit;		/* Draw unit info box (toggle function declared elsewhere) */
+extern int default_drawothermaps;	/* Draw other-map boxes */
+extern int default_sidecolors;		/* Use side colors for unit icons */					
+extern int default_iconmasks;		/* Draw unit icon masks */
+extern int default_boxmasks;		/* Draw masks for grouping boxes */
+extern int default_textmasks;		/* Draw text masks */
+extern int default_featureborders;	/* Draw feature borders */
+extern int default_featurenames;		/* Draw feature names */		
+extern int default_shorelines;		/* Draw 3D shorelines */
+extern int default_simple_borders;	/* Do not draw borders to sea or empty land */
+extern int default_optimize_fonts;	/* Use optimized set of font sizes */
+extern int default_erase_names;		/* Erase names of moving units */
+extern int default_power;			/* Default power of new map */
+extern int default_max_bufx;		/* Default size of horizonal offscreen scrolling buffer */
+extern int default_max_bufy;		/* Default size of vertical offscreen scrolling buffer */
+
+#define FEATURES 10						/* Should equal the number of feature colors below! */
+
+#define forecolor 		featColor[1]		/* Note: differs from ForeColor !!! */
+#define maskcolor 		featColor[2]		/* Default mask color for icons, text and boxes */
+#define textcolor 		featColor[3]		/* Text color (except for feature names) */
+#define gridcolor 		featColor[4]		/* Color of the grid */
+#define unseencolor 		featColor[5]		/* Color of unseen land */
+#define shorecolor 		featColor[6]		/* Shore line color */
+#define featurecolor 		featColor[7]		/* Feature name and border color */
+#define frontcolor 		featColor[8]		/* Front line color used by draw_country_borders */
+#define meridiancolor 	featColor[9]		/* Color of meridians */
+#define contourcolor 		featColor[10]		/* Color of contour lines */
+
+#define gridcolor_name	featColorName[4]	/* Used to set grid_matches_unseen */
+#define unseencolor_name	featColorName[5]	/* Used to set grid_matches_unseen */
+
+extern RGBColor *featColor; /*[FEATURES+1];		/* This array is used to store the feature colors */
+extern RGBColor *sideColor; /*[MAXSIDES+1][4];	/* This array is used to store the side colors */
+									/* side 0 is the independents, hence MAXSIDES + 1 */
+									/* sideColor[][0] is not currently used */
+
+extern char **featColorItemName; /*[FEATURES+1][32];	/* Name of item for which featColor is set */
+extern char **featColorName; /*[FEATURES+1][32];		/* Name of featColor value (red etc) */
+extern char **sideColorName; /*[MAXSIDES+1][4][32];		/* Name of sideColor value (red etc)*/
+extern char **default_featColorName; /*[FEATURES+1][32];	/* Default feature color values (by name) */
+extern char **default_colorscheme; /*[MAXSIDES+1][96];	/* Default side colorscheme (red,pink,blue) */
+extern char **colorscheme; /*[MAXSIDES+1][96];			/* Side colorscheme (red,pink,blue) */
+
+#define get_sideColor(s,n) (sideColor[4 * (s) + (n)])
+
+#define get_sideColorName(s,n) (sideColorName[4 * (s) + (n)])
+
+#define set_sideColor(s,n,c) (sideColor[4 * (s) + (n)] = (c))
+
+#define set_sideColorName(s,n,str) (sideColorName[4 * (s) + (n)] = (str))
+
+/* These are the default numbers for color usage */ 
+
+extern int default_main_icon_color;			/* Number (1-3) of main sideColor used for unit icons */ 
+extern int default_half_icon_color;			/* Number (1-3) of sideColor used for right half of icons */
+extern int default_icon_mask_color;			/* Number (1-3) of sideColor used to draw icon masks */
+extern int default_split_icon;				/* Use two colors for left and right half if TRUE */				
+
+/* These are customized numbers for each player */
+
+extern int *main_icon_color; /*[MAXSIDES +1];		/* Number (1-3) of main sideColor used for unit icons */ 
+extern int *half_icon_color; /*[MAXSIDES +1];		/* Number (1-3) of sideColor used for right half of icons */
+extern int *icon_mask_color; /*[MAXSIDES +1];		/* Number (1-3) of sideColor used to draw icon masks */
+extern int *split_icon; /*[MAXSIDES +1];			/* Use two colors for left and right half if TRUE */				
+
+/* These shorts are defined in macconq.c and used in maccolors.c */
+
+extern short images_refnum;	/* frefNum of lib-mac : Images file */ 
+extern short prefs_refnum;		/* frefNum of Xconq Preferences file */
+
 /* Function declarations. */
 
 /* Note that although the Xconq kernel must conditionalize its prototypes,
@@ -536,10 +647,7 @@ extern TEHandle command_text;
 extern int main(void);
 extern int splash_dialog(void);
 extern void connect_game_dialog(void);
-extern void init_toolbox(void);
 extern void recalc_depths(void);
-extern void init_rects(void);
-extern void init_ae(void);
 extern void get_files(void);
 extern int open_preferences(void);
 extern void close_preferences(void);
@@ -581,6 +689,8 @@ extern void stagger_window(WindowPtr win, int *lasthp, int *lastvp);
 extern void set_end_of_game_interaction_modes(void);
 extern void draw_selection_animation(Map *map, Unit *unit);
 
+extern void exit_macconq(void);
+
 /* Map-handling prototypes. */
 
 extern int at_all_visible(Map *map, int x, int y);
@@ -635,29 +745,28 @@ extern void force_map_update(Map *map);
 extern void destroy_map(Map *map);
 extern void activate_map(Map *map, int activate);
 extern void print_map(Map *map);
-extern void oneliner(Map *map, int sx, int sy);
 
-extern void draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int e, int mod);
+extern void draw_unit_image(WindowPtr win, int sx, int sy, int sw, int sh, int u, int s, int mod, int emblem);
 extern void draw_side_emblem(WindowPtr win, int ex, int ey, int ew, int eh, int e, int style);
-extern void draw_cell_block(int sx, int sy, int n, int power, int t, int t2, int angle);
+extern void draw_cell_block(int sx, int sy, int n, int power, int t, int over, int angle, int dosolid);
 extern void calc_best_terrain_images(void);
-extern void draw_hex_region(int sx, int sy, int power, int dogrid, int t, int t2, int angle);
-extern void draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle);
-extern void draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle);
+extern void draw_hex_region(int sx, int sy, int power, int t, int over, int angle, int dogrid, int dosolid);
+extern void draw_border_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle, int over, int dosolid);
+extern void draw_connection_line_multiple(WindowPtr win, int sx, int sy, int bitmask, int power, int t, int angle, int over, int dosolid);
 extern void draw_terrain_sample(WindowPtr win, Rect tmprect, int t);
 extern void draw_country_borders(WindowPtr win, int sx, int sy, int bitmask, int power, int shade, int angle);
+extern void draw_feature_borders(WindowPtr win, int sx, int sy, int bitmask, int power);
 extern void draw_ai_region_borders(WindowPtr win, int sx, int sy, int bitmask, int power);
 extern int draw_elevation_here(int x, int y);
 extern void draw_elevation(int sx, int sy, int power, int elev);
-extern int draw_temperature_here(int x, int y);
 extern void draw_temperature(int sx, int sy, int power, int temp);
-extern int draw_winds_here(int x, int y);
 extern void draw_winds(int sx, int sy, int power, int rawwind);
 extern int draw_clouds_here(int x, int y);
+extern void draw_cloud_block(int sx, int sy, int i, int power, int cloudtype, int angle);
 extern void draw_clouds(int sx, int sy, int power, int cloudtype);
 extern void draw_coverage(int sx, int sy, int power, int cov, int altcov);
-extern void draw_unit_name(Unit *unit, int sx, int sy, int sw, int sh);
-extern void draw_legend_text(int sx, int sy, int sh, char *legend, int just);
+extern void draw_unit_name(Unit *unit, int sx, int sy, int sw, int sh, int masks, int optimize);
+extern void draw_legend_text(int sx, int sy, int sh, char *legend, int just, int masks, int optimize);
 extern void draw_blast_image(WindowPtr win, int sx, int sy, int sw, int sh, int blasttype);
 extern void clear_blast_image(WindowPtr win, int sx, int sy, int sw, int sh, int blasttype);
 extern int picture_width(PicHandle pichandle);
@@ -680,6 +789,7 @@ extern void toggle_map_lighting(Map *map);
 extern void toggle_map_coverage(Map *map);
 extern void toggle_map_names(Map *map);
 extern void toggle_map_people(Map *map);
+extern void toggle_map_control(Map *map);
 extern void toggle_map_elevations(Map *map);
 extern void toggle_map_materials(Map *map, int m);
 extern void toggle_map_aux_terrain(Map *map, int t);
@@ -710,7 +820,7 @@ extern void select_previous_mover(Map *map);
 extern void select_next_awake_mover(Map *map);
 extern void select_previous_awake_mover(Map *map);
 extern void select_another(Map *map, Unit *(*fn)(Side *side, Unit *unit));
-extern void scroll_best_map_to_unit(Unit *unit);
+extern void scroll_best_map_to_unit(Unit *unit, int bringtofront);
 extern void scroll_to_unit(Map *map, Unit *unit);
 extern void magnify_to_fit(Map *map, int x1, int y1, int x2, int y2);
 
@@ -730,6 +840,8 @@ extern void zoom_list(List *list, int part);
 extern void adjust_list_decor(List *list);
 extern void do_mouse_down_list(List *list, Point mouse, int mods);
 extern void set_list_sorting(List *list, enum sortkeys newkey, int mi);
+extern void toggle_list_completed(List *list);
+extern void toggle_list_incomplete(List *list);
 extern void toggle_list_large_icons(List *list);
 extern void update_unit_in_lists(Unit *unit);
 extern int unit_position_in_list(List *list, Unit *unit);
@@ -742,8 +854,6 @@ extern void activate_list(List *list, int activate);
 extern void print_list(List *list);
 extern void destroy_list(List *list);
 
-extern void scroll_best_map_to_unit(Unit *unit);
-
 extern void init_patterns(void);
 extern void init_icons(void);
 extern void init_cursors(void);
@@ -755,11 +865,9 @@ extern int variants_dialog(void);
 extern int open_game_dialog(void);
 extern int open_game_from_fsspec(FSSpec *fsspec);
 extern int open_game_from_name_and_volume(Str255 name, int vrefnum);
-extern int launch_game(void);
 extern void handle_player_setup_event(EventRecord *event);
 extern int hit_player_setup_dialog(int itemhit, Point mouse);
 extern void launch_game_2(void);
-extern void add_remote_player(char *name);
 extern void check_for_missing_images(void);
 extern void open_progress_dialog(void);
 extern void close_progress_dialog(void);
@@ -769,6 +877,7 @@ extern void interp_named_color(char *name, enum grays *grayvar, RGBColor *colorv
 extern void init_terrain_colors(void);
 extern void init_unit_images(void);
 extern void init_all_emblem_images(void);
+extern void reinit_all_emblem_images(void);
 extern void init_emblem_images(Side *side2);
 
 extern int construction_ever_possible(void);
@@ -784,7 +893,7 @@ extern void build_terrain_type_menu(void);
 extern void build_ai_type_menu(void);
 extern void build_feature_menu(void);
 extern void build_optional_terrain_type_menu(void);
-extern void update_feature_menu(Feature *feature);
+extern void sanitize_for_menu(char *str, Str255 outstr);
 extern void do_menu_command(long which);
 extern void do_about_box(void);
 extern void save_the_game(int askname, int quitting);
@@ -801,18 +910,17 @@ extern int do_fire_command(void);
 extern int do_fire_into_command(void);
 extern void do_profile(void);
 extern void do_trace(void);
-extern void do_detonate_command(void);
 extern void adjust_menus(void);
 
 extern int add_unit_position(Unit *unit);
 extern void set_focus(Map *map, int x, int y);
-extern void message_dialog(void);
+extern void message_dialog(int sidenum);
 extern void command_dialog(void);
 extern void do_keyboard_command(int key);
 extern void execute_named_command(char *cmdstr);
-extern void describe_commands(int arg, char *key, char *buf);
+extern void describe_commands(int arg, char *key, TextBuffer *buf);
 struct cmdtab;
-extern void describe_command_table(int arg, char *key, char *buf, struct cmdtab *cmdtab);
+extern void describe_command_table(int arg, char *key, TextBuffer *buf, struct cmdtab *cmdtab);
 extern void toggle_profiling(void);
 
 extern char *get_string_from_item(Handle itemhandle);
@@ -867,12 +975,8 @@ extern int do_mouse_down_unit_closeup(UnitCloseup *unitcloseup, Point mouse, int
 extern void destroy_unit_closeup(UnitCloseup *unitcloseup);
 
 extern void create_history_window(void);
-extern void calc_history_layout(void);
-extern void update_total_hist_lines(void);
-extern void set_history_scrollbar(void);
 extern void draw_history(void);
-extern void draw_historical_event(HistEvent *hevt, int y);
-extern void draw_historical_date(HistEvent *hevt, int y);
+extern void draw_historical(HistEvent *hevt, int y, int drawevt);
 extern void do_mouse_down_history(Point mouse, int mods);
 extern void move_history_scrollbar(int h, int v);
 extern void grow_history(int h, int v);
@@ -906,10 +1010,11 @@ extern int do_key_down_command(int key);
 extern void do_mouse_down_command(Point mouse, int mods);
 
 extern void help_dialog(HelpNode *helpnode);
-extern void describe_menus(int arg, char *key, char *buf);
-extern void describe_mouse(int arg, char *key, char *buf);
-extern void describe_keyboard(int arg, char *key, char *buf);
-extern void describe_help(int arg, char *key, char *buf);
+extern void describe_menus(int arg, char *key, TextBuffer *buf);
+extern void describe_mouse(int arg, char *key, TextBuffer *buf);
+extern void describe_keyboard(int arg, char *key, TextBuffer *buf);
+extern void describe_text_commands(int arg, char *key, TextBuffer *buf);
+extern void describe_help(int arg, char *key, TextBuffer *buf);
 extern void create_help_window(void);
 extern void set_help_content(HelpNode *helpnode);
 extern void draw_help(void);
@@ -919,10 +1024,11 @@ extern void do_mouse_down_help(Point mouse, int mods);
 extern void grow_help(int h, int v);
 extern void zoom_help(int part);
 
+#ifdef DESIGNERS
 extern void enable_designing(int forsure);
 extern void disable_designing(void);
 extern void init_design_cursors(void);
-extern CursPtr adjust_designer_cursor(Point mouse, RgnHandle region);
+extern CursHandle adjust_designer_cursor(Point mouse, RgnHandle region);
 extern void create_design_window(void);
 extern void position_design_window(void);
 extern void draw_design_window(void);
@@ -937,6 +1043,7 @@ extern void border_on_drag(Map *map, int h0, int v0, int mods, int paintmode);
 extern void connect_on_drag(Map *map, int h0, int v0, int mods, int paintmode);
 extern void designer_save_dialog(void);
 extern void designer_reshape_dialog(Module *module);
+#endif /* DESIGNERS */
 
 extern void update_unit_in_maps(Unit *unit);
 extern void beep(void);
@@ -956,7 +1063,61 @@ extern void play_sound(char *soundname);
 extern void query_position_modally(int mode);
 extern void set_position_modally(void);
 
-extern int gameinited;
-extern int position_set_modally;
-extern Point modal_point;
-extern Map *modal_map;
+extern void enable_command(void);
+
+extern void invert_map(Map *map, int x, int y, int r);
+extern Map *front_map(void);
+
+extern void update_all_windows(void);
+extern void update_all_map_windows(void);
+extern void update_all_visible_windows(void);
+extern void update_all_visible_map_windows(void);
+
+extern void scroll_map_window(Map *map, int dx, int dy);	/* Scroll map window within its gworld */
+extern void copy_from_gworld(Map *map, Rect destRect);	/* Copy rect from gworld to screen */
+extern void update_resized_map(Map *map);			/* Update map and then redraw its gworld */
+extern void update_gworld(Map *map);					/* Update the gworld (and redraw the map)*/
+extern void set_offscreen_buffers(Map *map);			/* Set size of offscreen scrolling buffers */
+extern int handle_numeric_keypad(EventRecord *event);	/* Enables special use of numeric keypad */
+extern int cell_is_in_gworld(Map *map, int x, int y);		/* Test if cell is within current gworld */
+
+extern void toggle_map_sidecolors(Map *map);		
+extern void toggle_iconmasks(Map *map);
+extern void toggle_boxmasks(Map *map);
+extern void toggle_textmasks(Map *map);
+extern void toggle_featureborders(Map *map);
+extern void toggle_featurenames(Map *map);
+extern void toggle_shorelines(Map *map);
+extern void toggle_simple_borders(Map *map);
+extern void toggle_optimize_fonts(Map *map);
+extern void toggle_erase_names(Map *map);
+extern void toggle_draw_latlong(Map *map);			/* The associated default is declared elsewhere */
+extern void toggle_solid_color_terrain(Map *map);	/* The associated default is declared elsewhere */
+extern void toggle_list_sidecolors(List *list);		/* Uses the same default as toggle_map_sidecolors */
+
+extern void read_xcol(void);				/* Read RGB colors form XCol resource */
+extern void write_xcol(void);				/* Write RGB colors to XCol resource */
+extern void check_screen_depths(void);		/* Check for more than 256 colors */
+extern void set_default_colors(void);			/* Set default colors */
+extern void init_side_colors(void);			/* Grab memory, set colors to black, names to undefined */
+extern void set_side_colors(int e);			/* Side colors Dialog called by Colors menu */
+extern void set_default_side_colors(int e);		/* Side colors Dialog called by Prefs popup */
+extern void set_default_feature_color(int e);	/* Feature color Dialog called by Prefs popup */
+extern void set_feature_color(int e);			/* Feature color Dialog called by Colors menu */
+extern void set_terrain_color(Str255 tname);	/* Feature color Dialog for terrain */
+extern void build_colors_menu(void);			/* Build the Colors menu */
+extern void build_sides_menu(void);			/* Build popup menu for prefs */
+extern void build_features_menu(void);		/* Build popup menu for prefs */
+extern void init_unit_cicns(void);			/* Intitialize cicns by a dummy PlotCIcon call */
+extern void init_emblem_cicns(void);			/* Intitialize cicns ny a dummy PlotCIcon call */
+extern void dispose_unit_cicns(void);			/* Trash every unit cicn in use */
+extern void dispose_emblem_cicns(void);		/* Trash every emblem cicn in use */
+extern void dissect_colorscheme(int sidenum, 	/* Dissect a colorscheme into sideColorNames */
+		 char* colorscheme);				
+
+/* Functions in maccmd.c now declared here for use elsewhere */
+
+extern void do_recenter(void);
+extern void do_mac_zoom_in(void);
+extern void do_mac_zoom_out(void);
+extern void do_mac_set_map_angle(void);
