@@ -1,5 +1,5 @@
 /* Image families for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995 Stanley T. Shebs.
+   Copyright (C) 1992-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,15 +25,23 @@ any later version.  See the file COPYING.  */
 
 #include "macimf.h"
 
+#define mac_computed_rowbytes(w, pixelsize) (((w * pixelsize + 15) / 16) * 2)
+
 extern int hasColorQD;
 
-CTabHandle interp_ctab(Obj *palette);
-CTabHandle synth_ctab(void);
-void convert_ppat(Image *img, PixPatHandle colrpat);
-void convert_cicn(Image *img, CIconHandle colricon, int *hasmono, int *hasmask);
-Obj *convert_ctab(CTabHandle ctabhandle);
+extern CIconHandle mac_create_cicn(Image *img);
+extern void mac_add_cicn(ImageFamily *imf, Handle handle);
+extern void mac_add_ppat(ImageFamily *imf, Handle handle);
 
-MacImage *
+static MacImage *init_mac_image(Image *img);
+static void mac_interp_bytes(Obj *datalist, int numbytes, Handle desthandle, int jump);
+static CTabHandle interp_ctab(Obj *palette);
+static CTabHandle synth_ctab(void);
+static void convert_ppat(Image *img, PixPatHandle colrpat);
+static void convert_cicn(Image *img, CIconHandle colricon, int *hasmono, int *hasmask);
+static Obj *convert_ctab(CTabHandle ctabhandle);
+
+static MacImage *
 init_mac_image(Image *img)
 {
 	int j;
@@ -41,7 +49,7 @@ init_mac_image(Image *img)
 
 	macimg = (MacImage *) xmalloc(sizeof(MacImage));
 	for (j = 0; j < 8; ++j)
-	  SET_IMG_PAT (macimg, j, '\0');
+	  SET_IMG_PAT (macimg, j, '\F');
 	macimg->generic = img;
 	return macimg;
 }
@@ -62,24 +70,18 @@ get_mac_image(Image *img)
    should be "best first", so as to improve search time. */
 
 void
-mac_load_imf(imf)
-ImageFamily *imf;
+mac_load_imf(ImageFamily *imf)
 {
     int w, h, i, rsize, startlineno = 0, endlineno = 0;
     char tmpstrbuf[100];
     Obj *imfspec;
     Image *img;
     MacImage *macimg;
-    Handle imfhandle, pathandle, sicnhandle, iconhandle, maskhandle;
+    Handle imfhandle, pathandle, sicnhandle, iconhandle, maskhandle, handle;
     PicHandle pichandle;
-    PixPatHandle ppathandle;
-	PixMapHandle pmhandle;
-    CIconHandle cicnhandle;
-    PixMap pmap;
     Rect bounds;
-    Str255 namestr, maskstr, im16x16str, resname;
-	short resid;
-	ResType restype;
+    Str255 namestr, maskstr, im32x32str, im16x16str, im1x1str;
+	short resid;  ResType restype;  Str255 resname;
 
 	/* Can't do anything without a name for the image family. */
 	if (imf == NULL || imf->name == NULL)
@@ -88,13 +90,19 @@ ImageFamily *imf;
 	/* The name of the mask is always formed by appending " mask". */
 	sprintf(tmpstrbuf, "%s mask", imf->name);
 	c2p(tmpstrbuf, maskstr);
-	/* The name of the 16x16 cicn is always formed by appending " 16x16". */
+	/* The name of the 32x32 cicn/ppat is always formed by appending " 32x32". */
+	sprintf(tmpstrbuf, "%s 32x32", imf->name);
+	c2p(tmpstrbuf, im32x32str);
+	/* The name of the 16x16 cicn/ppat is always formed by appending " 16x16". */
 	sprintf(tmpstrbuf, "%s 16x16", imf->name);
 	c2p(tmpstrbuf, im16x16str);
+	/* The name of the solid color is always formed by appending " 1x1". */
+	sprintf(tmpstrbuf, "%s 1x1", imf->name);
+	c2p(tmpstrbuf, im1x1str);
 	/* Look for and load the image family specification resource first. */
 	imfhandle = (Handle) GetNamedResource('XCif', namestr);
 	if (imfhandle != nil) {
-		imfspec = read_form_from_string(copy_string(*imfhandle), &startlineno, &endlineno);
+		imfspec = read_form_from_string(copy_string(*imfhandle), &startlineno, &endlineno, NULL);
 		interp_imf_contents(imf, imfspec);
 	}
 	pichandle = (PicHandle) GetNamedResource('PICT', namestr);
@@ -118,44 +126,10 @@ ImageFamily *imf;
 	    would need separate resource to identify which is which) */
 	/* Pick up cicns, if we can do color. */
 	if (hasColorQD) {
-		cicnhandle = (CIconHandle) GetNamedResource('cicn', namestr);
-		if (cicnhandle != nil) {
-			HLock((Handle) cicnhandle);
-			/* Need to get id so we can use special trap. */
-			GetResInfo((Handle) cicnhandle, &resid, &restype, resname);
-			cicnhandle = GetCIcon(resid);
-			pmap = (*cicnhandle)->iconPMap;
-			bounds = pmap.bounds;
-			w = bounds.right - bounds.left;  h = bounds.bottom - bounds.top;
-			img = get_img(imf, w, h);
-			if (img != NULL) {
-				img->minw = img->minh = 8;
-				img->maxw = img->maxh = 128;
-				macimg = get_mac_image(img);
-				macimg->colricon = (Handle) cicnhandle;
-				/* Mask is built in, don't need to load separately. */
-			}
-			HUnlock((Handle) cicnhandle);
-		}
-		cicnhandle = (CIconHandle) GetNamedResource('cicn', im16x16str);
-		if (cicnhandle != nil) {
-			HLock((Handle) cicnhandle);
-			/* Need to get id so we can use special trap. */
-			GetResInfo((Handle) cicnhandle, &resid, &restype, resname);
-			cicnhandle = GetCIcon(resid);
-			pmap = (*cicnhandle)->iconPMap;
-			bounds = pmap.bounds;
-			w = bounds.right - bounds.left;  h = bounds.bottom - bounds.top;
-			img = get_img(imf, w, h);
-			if (img != NULL) {
-				img->minw = img->minh = 8;
-				img->maxw = img->maxh = 128;
-				macimg = get_mac_image(img);
-				macimg->colricon = (Handle) cicnhandle;
-				/* Mask is built in, don't need to load separately. */
-			}
-			HUnlock((Handle) cicnhandle);
-		}
+		handle = GetNamedResource('cicn', namestr);
+		mac_add_cicn(imf, handle);
+		handle = GetNamedResource('cicn', im16x16str);
+		mac_add_cicn(imf, handle);
 	} else {
 		/* (should at least try to get the mono part, without using CQD traps) */
 	}
@@ -206,54 +180,14 @@ ImageFamily *imf;
 	}
 	/* Pick up color patterns, if we're capable of doing color. */
 	if (hasColorQD) {
-		ppathandle = (PixPatHandle) GetNamedResource('ppat', namestr);
-		if (ppathandle != nil) {
-			/* Need to get the id of the ppat so we can use special trap. */
-			GetResInfo((Handle) ppathandle, &resid, &restype, resname);
-			ppathandle = GetPixPat(resid);
-			pmhandle = (*ppathandle)->patMap;
-			switch ((*ppathandle)->patType) {
-				case 0:
-					/* (should put something in monodata?) */
-					w = h = 8;
-					break;
-				case 1:
-					/* Compute the actual size of the pattern. */
-					bounds = (*pmhandle)->bounds;
-					w = bounds.right - bounds.left;  h = bounds.bottom - bounds.top;
-			}
-			img = get_img(imf, w, h);
-			if (img != NULL) {
-				/* Indicate that we can use this pattern for any size area. */
-				img->minw = img->minh = 1;
-				img->maxw = img->maxh = 9999;
-				img->istile = TRUE;
-				macimg = get_mac_image(img);
-				macimg->colrpat = ppathandle;
-				if (w == 8 && h == 8) {
-					/* Set the monopat. */
-					for (i = 0; i < 8; ++i)
-					  SET_IMG_PAT(macimg, i, QD_PAT_ADDR((*ppathandle)->pat1Data)[i]);
-					macimg->patdefined = 1;
-				}
-			}
-			/* If the pattern is larger than 8x8, we need to make an 8x8 image for
-			   the mono pattern. */
-			if (!(w == 8 && h == 8)) {
-				img = get_img(imf, 8, 8);
-				if (img != NULL) {
-					/* Indicate that we can use this pattern for any size area. */
-					img->minw = img->minh = 1;
-					img->maxw = img->maxh = 9999;
-					img->istile = TRUE;
-					macimg = get_mac_image(img);
-					/* Set the monopat. */
-					for (i = 0; i < 8; ++i)
-					  SET_IMG_PAT(macimg, i, QD_PAT_ADDR((*ppathandle)->pat1Data)[i]);
-					macimg->patdefined = 1;
-				}
-			}
-		}
+		handle = GetNamedResource('ppat', namestr);
+		mac_add_ppat(imf, handle);
+		handle = GetNamedResource('ppat', im1x1str);
+		mac_add_ppat(imf, handle);
+		handle = GetNamedResource('ppat', im16x16str);
+		mac_add_ppat(imf, handle);
+		handle = GetNamedResource('ppat', im32x32str);
+		mac_add_ppat(imf, handle);
 	}
 	/* Load a pattern, which can be used for any size area, but whose "natural" size
 	   is always 8x8. */
@@ -297,19 +231,133 @@ ImageFamily *imf;
 	}
 }
 
+void
+mac_add_cicn(ImageFamily *imf, Handle handle)
+{
+    int n, i, w, h;
+    Image *img;
+    MacImage *macimg;
+    CIconHandle cicnhandle;
+    PixMap pmap;
+    Rect bounds;
+	short resid;  ResType restype;  Str255 resname;
+
+	if (handle == nil)
+	  return;
+	HLock(handle);
+	/* Need to get id so we can use special trap. */
+	GetResInfo(handle, &resid, &restype, resname);
+	cicnhandle = GetCIcon(resid);
+	HLock((Handle) cicnhandle);
+	pmap = (*cicnhandle)->iconPMap;
+	bounds = pmap.bounds;
+	w = bounds.right - bounds.left;  h = bounds.bottom - bounds.top;
+	img = get_img(imf, w, h);
+	if (img != NULL) {
+		img->minw = img->minh = 8;
+		img->maxw = img->maxh = 128;
+		macimg = get_mac_image(img);
+		macimg->colricon = (Handle) cicnhandle;
+		/* Mask is built in, don't need to load separately. */
+	}
+	HUnlock((Handle) cicnhandle);
+	HUnlock(handle);
+}
+
+void
+mac_add_ppat(ImageFamily *imf, Handle handle)
+{
+    int n, i, w, h, val, constpat;
+    Image *img;
+    MacImage *macimg;
+    PixPatHandle ppathandle;
+	PixMapHandle pmhandle;
+    PixMap pmap;
+	CTabHandle ctabhandle;
+	ColorSpec *ctdata;
+    Rect bounds;
+	short resid;  ResType restype;  Str255 resname;
+	if (handle == nil)
+	  return;
+	HLock(handle);
+	/* Need to get the id of the ppat so we can use special trap. */
+	GetResInfo(handle, &resid, &restype, resname);
+	ppathandle = GetPixPat(resid);
+	HLock((Handle) ppathandle);
+	pmhandle = (*ppathandle)->patMap;
+	switch ((*ppathandle)->patType) {
+		case 0:
+			/* (should put something in monodata?) */
+			w = h = 8;
+			break;
+		case 1:
+			/* Compute the actual size of the pattern. */
+			bounds = (*pmhandle)->bounds;
+			w = bounds.right - bounds.left;  h = bounds.bottom - bounds.top;
+			/* Decide if this pattern is a pattern or just a color; color
+			   part must be one color and mono part must be all zeros. */
+			ctabhandle = (*pmhandle)->pmTable;
+			if ((*ctabhandle)->ctSize == 0) {
+				/* See if anything nonzero in mono pattern. */
+				constpat = TRUE;
+				for (i = 0; i < 8; ++i) {
+					val = QD_PAT_ADDR((*ppathandle)->pat1Data)[i];
+					if (val != 0)
+					  constpat = FALSE;
+				}
+				if (constpat) {
+					w = h = 1;
+				}
+			}
+	}
+	img = get_img(imf, w, h);
+	if (img != NULL) {
+		/* Indicate that we can use this pattern for any size area. */
+		img->minw = img->minh = 1;
+		img->maxw = img->maxh = 9999;
+		img->istile = TRUE;
+		macimg = get_mac_image(img);
+		if (w == 1 && h == 1) {
+			/* Collect the one color and record it. */
+			ctabhandle = (*pmhandle)->pmTable;
+			ctdata = (ColorSpec *) &((*ctabhandle)->ctTable);
+			macimg->color.red   = ctdata[0].rgb.red;
+			macimg->color.green = ctdata[0].rgb.green;
+			macimg->color.blue  = ctdata[0].rgb.blue;
+			macimg->colordefined = TRUE;
+		} else {
+			macimg->colrpat = ppathandle;
+			/* A ppat always has an 8x8 mono pattern defined. */
+			img = get_img(imf, 8, 8);
+			if (img != NULL) {
+				/* Indicate that we can use this pattern for any size area. */
+				img->minw = img->minh = 1;
+				img->maxw = img->maxh = 9999;
+				img->istile = TRUE;
+				macimg = get_mac_image(img);
+				/* Set the monopat. */
+				for (i = 0; i < 8; ++i)
+				  SET_IMG_PAT(macimg, i, QD_PAT_ADDR((*ppathandle)->pat1Data)[i]);
+				macimg->patdefined = 1;
+			}
+		}
+	}
+	HUnlock((Handle) ppathandle);
+	HUnlock(handle);
+}
+
 /* Given an image family that already has data for it, in Lisp form,
    make platform-specific images. */
 
 void
-mac_interp_imf(imf)
-ImageFamily *imf;
+mac_interp_imf(ImageFamily *imf)
 {
 	int w, h, i, numbytes, bitrowbytes, actualw, actualh, pixelsize, rowbytes;
+	int macrowbytes;
 	int monodone, colrdone, maskdone;
 	Image *img;
 	MacImage *macimg;
 	Handle sicnhandle, iconhandle, datahandle;
-	Handle monohandle = nil, maskhandle = nil;
     PixPatHandle ppathandle;
     CIconHandle cicnhandle;
     PixMapHandle pmhandle;
@@ -318,7 +366,7 @@ ImageFamily *imf;
 		w = img->w;  h = img->h;
 		actualw = img->actualw;  actualh = img->actualh;
 		pixelsize = img->pixelsize;
-		rowbytes = img->rowbytes;
+		rowbytes = computed_rowbytes(actualw, pixelsize);
 		macimg = get_mac_image(img);
 		/* Mono icons and masks are very similar; digest both here. */
 		monodone = colrdone = maskdone = FALSE;
@@ -371,155 +419,61 @@ ImageFamily *imf;
 			macimg->maskicon = iconhandle;
 			maskdone = TRUE;
 		}
-		/* The mono and mask data are probably going to go into the cicn - save them
-		   into a handle for now. */
-		if (!monodone && img->monodata != lispnil) {
-			if (rowbytes < 1)
-			  rowbytes = ((actualw + 15) / 16) * 2;
-			numbytes = actualh * (actualw <= 8 ? 2 : rowbytes);
-			monohandle = NewHandle(numbytes);
-			memset(*maskhandle, 0, numbytes);
-			/* Read as many bytes as directed. */
-			mac_interp_bytes(img->monodata, numbytes, monohandle, (actualw <= 8 ? 1 : 0));
-			monodone = TRUE;
-		}
-		if (!maskdone && img->maskdata != lispnil) {
-			if (rowbytes < 1)
-			  rowbytes = ((actualw + 15) / 16) * 2;
-			numbytes = actualh * (actualw <= 8 ? 2 : rowbytes);
-			maskhandle = NewHandle(numbytes);
-			memset(*maskhandle, 0, numbytes);
-			/* Read as many bytes as directed. */
-			mac_interp_bytes(img->maskdata, numbytes, maskhandle, (actualw <= 8 ? 1 : 0));
-			maskdone = TRUE;
-		}
-		/* Limit bits per pixel to reasonable values. */
-		if (pixelsize < 1 || pixelsize > 8)
-		  pixelsize = 8;
-		/* A sufficiently small tiling color image can be a color pattern. */
-		if (hasColorQD && actualw <= 64 && actualh <= 64 && img->istile && img->colrdata != lispnil) {
-			/* Make a color pattern. */
+		/* A 1x1 image is just a color - make it into a small color pattern. */
+		if (hasColorQD && w == 1 && h == 1 && img->istile) {
+			/* Make a simple color pattern. */
 			ppathandle = NewPixPat();
 			pmhandle = (*ppathandle)->patMap;
-			SetRect(&((*pmhandle)->bounds), 0, 0, actualw, actualh);
-			if (rowbytes < 1)
-			  rowbytes = (actualw * pixelsize) / 8;
-			(*pmhandle)->rowBytes = rowbytes;
-			(*pmhandle)->pixelSize = pixelsize;
+			SetRect(&((*pmhandle)->bounds), 0, 0, 8, 8);
+			(*pmhandle)->rowBytes = 1;
+			(*pmhandle)->pixelType = 0;
+			(*pmhandle)->pixelSize = 1;
+			(*pmhandle)->cmpCount = 1;
+			(*pmhandle)->cmpSize = pixelsize;
 			(*pmhandle)->pmTable = interp_ctab(img->palette);
-			numbytes = actualh * rowbytes;
-			datahandle = NewHandle(numbytes);
+			datahandle = NewHandle(8);
 			(*ppathandle)->patData = datahandle;
-			mac_interp_bytes(img->colrdata, numbytes, datahandle, 0);
+			HLock(datahandle);
+			memset(*datahandle, 0, 8);
+			HUnlock(datahandle);
 			(*ppathandle)->patXValid = -1;
 			macimg->colrpat = ppathandle;
 			colrdone = maskdone = TRUE;
 		}
-		if (hasColorQD && actualw <= 64 && actualh <= 64
-		    && !img->istile
+		/* A sufficiently small tiling color image can be a color pattern. */
+		if (hasColorQD && w <= 64 && h <= 64 && img->istile && img->colrdata != lispnil) {
+			/* Limit bits per pixel to reasonable values. */
+			if (pixelsize < 1 || pixelsize > 8)
+			  pixelsize = 8;
+			macrowbytes = mac_computed_rowbytes(actualw, pixelsize);
+			/* As a special exception, PixPats can have a rowbytes of 1. */
+			if (actualw <= 8 && pixelsize == 1)
+			  macrowbytes = 1;
+			/* Make a color pattern. */
+			ppathandle = NewPixPat();
+			pmhandle = (*ppathandle)->patMap;
+			SetRect(&((*pmhandle)->bounds), 0, 0, actualw, actualh);
+			(*pmhandle)->rowBytes = macrowbytes;
+			(*pmhandle)->pixelType = 0;
+			(*pmhandle)->pixelSize = pixelsize;
+			(*pmhandle)->cmpCount = 1;
+			(*pmhandle)->cmpSize = pixelsize;
+			(*pmhandle)->pmTable = interp_ctab(img->palette);
+			numbytes = actualh * rowbytes;
+			datahandle = NewHandle(numbytes);
+			(*ppathandle)->patData = datahandle;
+			mac_interp_bytes(img->colrdata, numbytes, datahandle, macrowbytes - rowbytes);
+			(*ppathandle)->patXValid = -1;
+			macimg->colrpat = ppathandle;
+			colrdone = maskdone = TRUE;
+		}
+		if (hasColorQD && w <= 64 && h <= 64 && !img->istile
 		    && (img->colrdata != lispnil
 		        || (macimg->monosicn == nil
 		            && macimg->masksicn == nil
 		            && macimg->monoicon == nil
-		            && macimg->maskicon == nil
-		           ))) {
-		    /* If no color data in evidence, prepare to use mono or mask instead. */
-		    if (img->colrdata == lispnil) {
-		    	pixelsize = 1;
-		    }
-			/* Make a full color icon. */
-			bitrowbytes = ((actualw + 15) / 16) * 2;
-			/* Allocate enough space for the icon and its data/mask bitmaps. */
-			cicnhandle = (CIconHandle) NewHandle(sizeof(CIcon) + 2 * actualh * bitrowbytes);
-			HLock((Handle) cicnhandle);
-			(*cicnhandle)->iconPMap.baseAddr = 0;
-			if (rowbytes < 1)
-			  rowbytes = ((actualw + 15) / 16) * pixelsize * 2;
-			(*cicnhandle)->iconPMap.rowBytes = rowbytes | 0x8000;
-			SetRect(&((*cicnhandle)->iconPMap.bounds), 0, 0, actualw, actualh);
-			(*cicnhandle)->iconPMap.pmVersion = 0;
-			(*cicnhandle)->iconPMap.packType = 0;
-			(*cicnhandle)->iconPMap.packSize = 0;
-			(*cicnhandle)->iconPMap.hRes = 0;
-			(*cicnhandle)->iconPMap.vRes = 0;
-			(*cicnhandle)->iconPMap.pixelType = 0;
-			(*cicnhandle)->iconPMap.pixelSize = pixelsize;
-			(*cicnhandle)->iconPMap.cmpCount = 1;
-			(*cicnhandle)->iconPMap.cmpSize = pixelsize;
-			(*cicnhandle)->iconPMap.planeBytes = 0;
-			(*cicnhandle)->iconPMap.pmTable = interp_ctab(img->palette);
-			(*cicnhandle)->iconPMap.pmReserved = 0;
-			/* Configure the monochrome icon. */
-			SetRect(&((*cicnhandle)->iconBMap.bounds), 0, 0, actualw, actualh);
-			(*cicnhandle)->iconBMap.rowBytes = 0;
-			(*cicnhandle)->iconBMap.baseAddr = NULL;
-			SetRect(&((*cicnhandle)->iconMask.bounds), 0, 0, actualw, actualh);
-			/* Configure the mask bitmap. */
-			(*cicnhandle)->iconMask.rowBytes = bitrowbytes;
-			(*cicnhandle)->iconMask.baseAddr = NULL;
-			numbytes = actualh * rowbytes;
-			datahandle = NewHandle(numbytes);
-			(*cicnhandle)->iconData = datahandle;
-			/* Fill up the datahandle with the color data, or else use mono/mask data
-			   if the color data is missing. */
-			if (img->colrdata != lispnil) {
-				mac_interp_bytes(img->colrdata, numbytes, datahandle, 0);
-			} else if (img->monodata != lispnil) {
-				mac_interp_bytes(img->monodata, numbytes, datahandle, (actualw <= 8 ? 1 : 0));
-				/* We need an ersatz color table. */
-				(*cicnhandle)->iconPMap.pmTable = synth_ctab();
-			} else if (img->maskdata != lispnil) {
-				mac_interp_bytes(img->maskdata, numbytes, datahandle, (actualw <= 8 ? 1 : 0));
-				/* We need an ersatz color table. */
-				(*cicnhandle)->iconPMap.pmTable = synth_ctab();
-			}
-			/* If a mono icon is already set up, use it as the icon's bitmap. */
-			if (macimg->monoicon != nil) {
-				HLock(macimg->monoicon);
-				memcpy(((char *) (*cicnhandle)->iconMaskData) + actualh * bitrowbytes,
-					   *(macimg->monoicon), actualh * bitrowbytes);
-				(*cicnhandle)->iconBMap.rowBytes = bitrowbytes;
-				HUnlock(macimg->monoicon);
-			}
-			/* If a mono sicn is already set up, use it as the icon's bitmap. */
-			if (macimg->monosicn != nil) {
-				HLock(macimg->monosicn);
-				memcpy(((char *) (*cicnhandle)->iconMaskData) + actualh * bitrowbytes,
-					   *(macimg->monosicn), actualh * bitrowbytes);
-				(*cicnhandle)->iconBMap.rowBytes = bitrowbytes;
-				HUnlock(macimg->monosicn);
-			}
-			/* ...or just use the monohandle that we built earlier. */
-			if (monohandle != nil) {
-				HLock(monohandle);
-				memcpy(((char *) (*cicnhandle)->iconMaskData) + actualh * bitrowbytes,
-					   *monohandle, actualh * bitrowbytes);
-				(*cicnhandle)->iconBMap.rowBytes = bitrowbytes;
-				HUnlock(monohandle);
-			}
-			/* If a mask icon is already set up, use it as the icon's mask. */
-			if (macimg->maskicon != nil) {
-				HLock(macimg->maskicon);
-				memcpy((char *) (*cicnhandle)->iconMaskData,
-					   *(macimg->maskicon), actualh * bitrowbytes);
-				HUnlock(macimg->maskicon);
-			}
-			/* If a mask sicn is already set up, use it as the icon's mask. */
-			if (macimg->masksicn != nil) {
-				HLock(macimg->masksicn);
-				memcpy((char *) (*cicnhandle)->iconMaskData,
-					   *(macimg->masksicn), actualh * bitrowbytes);
-				HUnlock(macimg->masksicn);
-			}
-			/* ...or just use the maskhandle that we built earlier. */
-			if (maskhandle != nil) {
-				HLock(maskhandle);
-				memcpy((char *) (*cicnhandle)->iconMaskData,
-					   *maskhandle, actualh * bitrowbytes);
-				HUnlock(maskhandle);
-			}
-			macimg->colricon = (Handle) cicnhandle;
-			HUnlock((Handle) cicnhandle);
+		            && macimg->maskicon == nil))) {
+		    macimg->colricon = (Handle) mac_create_cicn(img);
 			colrdone = TRUE;
 		}
 		if (!monodone && !colrdone) {
@@ -546,11 +500,138 @@ ImageFamily *imf;
 #endif
 }
 
-void
-mac_interp_bytes(datalist, numbytes, desthandle, jump)
-Obj *datalist;
-int numbytes, jump;
-Handle desthandle;
+/* Given an image, make a color icon out of its data. */
+
+CIconHandle
+mac_create_cicn(Image *img)
+{
+	int w, h, i, numbytes, bitrowbytes, macbitrowbytes, actualw, actualh, pixelsize, rowbytes, macrowbytes;
+	MacImage *macimg;
+	Handle sicnhandle, iconhandle, datahandle, monohandle, maskhandle;
+    PixPatHandle ppathandle;
+    CIconHandle cicnhandle;
+    PixMapHandle pmhandle;
+
+	w = img->w;  h = img->h;
+	actualw = img->actualw;  actualh = img->actualh;
+	pixelsize = img->pixelsize;
+    /* If no color data in evidence, prepare to use mono or mask instead. */
+    if (img->colrdata == lispnil)
+	  pixelsize = 1;
+	rowbytes = computed_rowbytes(actualw, pixelsize);
+	macrowbytes = mac_computed_rowbytes(actualw, pixelsize);
+	bitrowbytes = computed_rowbytes(actualw, 1);
+	macbitrowbytes = mac_computed_rowbytes(actualw, 1);
+	macimg = get_mac_image(img);
+	/* Make a full color icon. */
+	/* Allocate enough space for the icon and its data/mask bitmaps. */
+	cicnhandle = (CIconHandle) NewHandle(sizeof(CIcon) + 2 * actualh * macbitrowbytes);
+	if (cicnhandle == nil)
+	  run_error("out of memory");
+	HLock((Handle) cicnhandle);
+	(*cicnhandle)->iconPMap.baseAddr = 0;
+	(*cicnhandle)->iconPMap.rowBytes = macrowbytes | 0x8000;
+	SetRect(&((*cicnhandle)->iconPMap.bounds), 0, 0, actualw, actualh);
+	(*cicnhandle)->iconPMap.pmVersion = 0;
+	(*cicnhandle)->iconPMap.packType = 0;
+	(*cicnhandle)->iconPMap.packSize = 0;
+	(*cicnhandle)->iconPMap.hRes = 0;
+	(*cicnhandle)->iconPMap.vRes = 0;
+	(*cicnhandle)->iconPMap.pixelType = 0;
+	(*cicnhandle)->iconPMap.pixelSize = pixelsize;
+	(*cicnhandle)->iconPMap.cmpCount = 1;
+	(*cicnhandle)->iconPMap.cmpSize = pixelsize;
+	(*cicnhandle)->iconPMap.planeBytes = 0;
+	(*cicnhandle)->iconPMap.pmTable = interp_ctab(img->palette);
+	(*cicnhandle)->iconPMap.pmReserved = 0;
+	/* Configure the monochrome icon. */
+	SetRect(&((*cicnhandle)->iconBMap.bounds), 0, 0, actualw, actualh);
+	(*cicnhandle)->iconBMap.rowBytes = 0;
+	(*cicnhandle)->iconBMap.baseAddr = NULL;
+	SetRect(&((*cicnhandle)->iconMask.bounds), 0, 0, actualw, actualh);
+	/* Configure the mask bitmap. */
+	(*cicnhandle)->iconMask.rowBytes = macbitrowbytes;
+	(*cicnhandle)->iconMask.baseAddr = NULL;
+	numbytes = actualh * macrowbytes;
+	datahandle = NewHandle(numbytes);
+	(*cicnhandle)->iconData = datahandle;
+	/* Fill up the datahandle with the color data, or else use mono/mask data
+	   if the color data is missing. */
+	if (img->colrdata != lispnil) {
+		mac_interp_bytes(img->colrdata, numbytes, datahandle, macrowbytes - rowbytes);
+	} else if (img->monodata != lispnil) {
+		mac_interp_bytes(img->monodata, numbytes, datahandle, macrowbytes - rowbytes);
+		/* Make an ersatz color table. */
+		(*cicnhandle)->iconPMap.pmTable = synth_ctab();
+	} else if (img->maskdata != lispnil) {
+		mac_interp_bytes(img->maskdata, numbytes, datahandle, macrowbytes - rowbytes);
+		/* Make an ersatz color table. */
+		(*cicnhandle)->iconPMap.pmTable = synth_ctab();
+	} else {
+		/* (should do something) */
+	}
+	/* Set up rowbytes and numbytes for both mono and mask parts. */
+    (*cicnhandle)->iconBMap.rowBytes = macbitrowbytes;
+	numbytes = actualh * macbitrowbytes;
+	if (macimg->monoicon != nil) {
+		/* If a mono icon is already set up, use it as the icon's bitmap. */
+		HLock(macimg->monoicon);
+		memcpy(((char *) (*cicnhandle)->iconMaskData) + numbytes, *(macimg->monoicon), numbytes);
+		HUnlock(macimg->monoicon);
+	} else if (macimg->monosicn != nil) {
+		/* If a mono sicn is already set up, use it as the icon's bitmap. */
+		HLock(macimg->monosicn);
+		memcpy(((char *) (*cicnhandle)->iconMaskData) + numbytes, *(macimg->monosicn), numbytes);
+		HUnlock(macimg->monosicn);
+	} else if (img->monodata != lispnil) {
+		/* (should use static temp space here?) */
+		monohandle = NewHandle(numbytes);
+		HLock(monohandle);
+		memset(*monohandle, 0, numbytes);
+		/* Read as many bytes as directed. */
+		mac_interp_bytes(img->monodata, numbytes, monohandle, macbitrowbytes - bitrowbytes);
+		memcpy(((char *) (*cicnhandle)->iconMaskData) + numbytes, *monohandle, numbytes);
+		HUnlock(monohandle);
+	} else {
+		/* No mono data, make it all 1s. */
+		monohandle = NewHandle(numbytes);
+		HLock(monohandle);
+		memset(*monohandle, 0xff, numbytes);
+		memcpy(((char *) (*cicnhandle)->iconMaskData) + numbytes, *monohandle, numbytes);
+		HUnlock(monohandle);
+	}
+	if (macimg->maskicon != nil) {
+		/* If a mask icon is already set up, use it as the icon's mask. */
+		HLock(macimg->maskicon);
+		memcpy((char *) (*cicnhandle)->iconMaskData, *(macimg->maskicon), numbytes);
+		HUnlock(macimg->maskicon);
+	} else if (macimg->masksicn != nil) {
+		/* If a mask sicn is already set up, use it as the icon's mask. */
+		HLock(macimg->masksicn);
+		memcpy((char *) (*cicnhandle)->iconMaskData, *(macimg->masksicn), numbytes);
+		HUnlock(macimg->masksicn);
+	} else if (img->maskdata != lispnil) {
+		maskhandle = NewHandle(numbytes);
+		HLock(maskhandle);
+		memset(*maskhandle, 0, numbytes);
+		/* Read as many bytes as directed. */
+		mac_interp_bytes(img->maskdata, numbytes, maskhandle, macbitrowbytes - bitrowbytes);
+		memcpy((char *) (*cicnhandle)->iconMaskData, *maskhandle, numbytes);
+		HUnlock(maskhandle);
+	} else {
+		/* No mask data, make it black. */
+		maskhandle = NewHandle(numbytes);
+		HLock(maskhandle);
+		memset(*maskhandle, 0xff, numbytes);
+		memcpy((char *) (*cicnhandle)->iconMaskData, *maskhandle, numbytes);
+		HUnlock(maskhandle);
+	}
+	HUnlock((Handle) cicnhandle);
+	return cicnhandle;
+}
+
+static void
+mac_interp_bytes(Obj *datalist, int numbytes, Handle desthandle, int jump)
 {
     HLock(desthandle);
     interp_bytes(datalist, numbytes, *desthandle, jump);
@@ -559,15 +640,18 @@ Handle desthandle;
 
 /* Given a list of color table entries, create and return a color table. */
 
-CTabHandle
-interp_ctab(palette)
-Obj *palette;
+static CTabHandle
+interp_ctab(Obj *palette)
 {
-	int len, i;
+	int len, i, r, g, b;
+	char *colorname;
 	Obj *head, *rest, *color;
 	ColorSpec *ctdata;
 	CTabHandle ctabhandle;
 
+	if (!consp(palette)) {
+		/* (should look up named palette) */
+	}
 	len = length(palette);
 	if (len == 0)
 	  return nil;
@@ -581,15 +665,50 @@ Obj *palette;
 		head = car(rest);
 		ctdata[i].value = c_number(car(head));
 		color = cdr(head);
-		ctdata[i].rgb.red = c_number(car(color));
-		ctdata[i].rgb.green = c_number(cadr(color));
-		ctdata[i].rgb.blue = c_number(car(cddr(color)));
+		if (symbolp(car(color)) || stringp(car(color))) {
+			colorname = c_string(car(color));
+			if (strcmp(colorname, "white") == 0) {
+				ctdata[i].rgb.red   = 65535;
+				ctdata[i].rgb.green = 65535;
+				ctdata[i].rgb.blue  = 65535;
+			} else if (strcmp(colorname, "black") == 0) {
+				ctdata[i].rgb.red   = 0;
+				ctdata[i].rgb.green = 0;
+				ctdata[i].rgb.blue  = 0;
+			} else if (strcmp(colorname, "red") == 0) {
+				ctdata[i].rgb.red   = 65535;
+				ctdata[i].rgb.green = 0;
+				ctdata[i].rgb.blue  = 0;
+			} else if (strcmp(colorname, "green") == 0) {
+				ctdata[i].rgb.red   = 0;
+				ctdata[i].rgb.green = 65535;
+				ctdata[i].rgb.blue  = 0;
+			} else if (strcmp(colorname, "blue") == 0) {
+				ctdata[i].rgb.red   = 0;
+				ctdata[i].rgb.green = 0;
+				ctdata[i].rgb.blue  = 65535;
+			} else {
+				init_warning("No color named \"%s\" found, substituting gray",
+							 colorname);
+				ctdata[i].rgb.red   = i * 5000;
+				ctdata[i].rgb.green = i * 5000;
+				ctdata[i].rgb.blue  = i * 5000;
+			}
+		} else if (numberp(car(color))) {
+			ctdata[i].rgb.red   = c_number(car(color));
+			ctdata[i].rgb.green = c_number(cadr(color));
+			ctdata[i].rgb.blue  = c_number(car(cddr(color)));
+		} else {
+			init_warning("palette color is not a name or set of numbers, ignoring");
+		}
 	}
 	HUnlock((Handle) ctabhandle);
 	return ctabhandle;
 }
 
-CTabHandle
+/* Make up a two-color (white and black) color table. */
+
+static CTabHandle
 synth_ctab()
 {
 	int len;
@@ -619,8 +738,10 @@ void
 make_generic_image_data(ImageFamily *imf)
 {
 	int i, hasmono, hasmask;
+	char *colorname;
 	Image *img;
 	MacImage *macimg;
+	Obj *colorcomp;
 
 	for (img = imf->images; img != NULL; img = img->next) {
 		hasmono = hasmask = FALSE;
@@ -631,6 +752,19 @@ make_generic_image_data(ImageFamily *imf)
 		if (hasColorQD && macimg->colrpat != nil) {
 			convert_ppat(img, macimg->colrpat);
 		}
+		if (hasColorQD && img->w == 1 && img->h == 1 && macimg->colordefined) {
+			colorname = find_color_name(macimg->color.red, macimg->color.green, macimg->color.blue);
+			if (colorname != NULL) {
+				colorcomp = cons(new_string(colorname), lispnil);
+			} else {
+				colorcomp = cons(new_number(macimg->color.red),
+								 cons(new_number(macimg->color.green),
+									  cons(new_number(macimg->color.blue),
+									       lispnil)));
+
+			}
+			img->palette = cons(cons(new_number(0), colorcomp), lispnil);
+		}
 		if (macimg->monoicon != nil && !hasmono) {
 			img->rawmonodata = xmalloc(128);
 			for (i = 0; i < 128; ++i)
@@ -657,14 +791,15 @@ make_generic_image_data(ImageFamily *imf)
 			  img->rawmonodata[i] = ((char *) &(macimg->monopat))[i];
 		}
 	}
+    compute_image_bboxes(imf);
 }
 
 /* Generify a color pattern. */
 
-void
+static void
 convert_ppat(Image *img, PixPatHandle pat)
 {
-	int w, h, i, numbytes;
+	int w, h, i, j, numbytes, rowbytes, macrowbytes, anynonzero;
 	Rect bounds;
 	PixMapHandle pmhandle = (*pat)->patMap;
 	CTabHandle ctabhandle = (*pmhandle)->pmTable;
@@ -681,13 +816,18 @@ convert_ppat(Image *img, PixPatHandle pat)
 			if (w != img->w || h != img->h) {
 				img->actualw = w;  img->actualh = h;
 			}
-			img->rowbytes = (*pmhandle)->rowBytes & 0x3fff;
 			img->pixelsize = (*pmhandle)->pixelSize;
 			img->palette = convert_ctab(ctabhandle);
-			numbytes = h * img->rowbytes;
+			rowbytes = computed_rowbytes(w, img->pixelsize);
+			macrowbytes = (*pmhandle)->rowBytes & 0x3fff;
+			numbytes = h * rowbytes;
 			img->rawcolrdata = xmalloc(numbytes);
-			for (i = 0; i < numbytes; ++i)
-			  img->rawcolrdata[i] = (*patdata)[i];
+			j = 0;
+			for (i = 0; i < numbytes; ++i) {
+				if (rowbytes > 1 && i > 0 && i % rowbytes == 0)
+				  j += (macrowbytes - rowbytes);
+				img->rawcolrdata[i] = (*patdata)[j++];
+			}
 			break;
 		case 2:
 			run_warning("Type 2 (RGB) pattern, ignoring");
@@ -697,10 +837,10 @@ convert_ppat(Image *img, PixPatHandle pat)
 
 /* Generify the data in a color icon. */
 
-void
+static void
 convert_cicn(Image *img, CIconHandle cicnhandle, int *hasmono, int *hasmask)
 {
-	int w, h, i, rowbytes, numbytes;
+	int w, h, i, j, rowbytes, macrowbytes, bitrowbytes, macbitrowbytes, numbytes;
 	Rect bounds;
 	char *baseaddr;
 	PixMap pmap = (*cicnhandle)->iconPMap;
@@ -714,46 +854,54 @@ convert_cicn(Image *img, CIconHandle cicnhandle, int *hasmono, int *hasmask)
 	if (w != img->w || h != img->h) {
 		img->actualw = w;  img->actualh = h;
 	}
-	img->rowbytes = pmap.rowBytes & 0x3fff;
 	img->pixelsize = pmap.pixelSize;
 	img->palette = convert_ctab(ctabhandle);
-	numbytes = h * img->rowbytes;
+	rowbytes = computed_rowbytes(w, img->pixelsize);
+	macrowbytes = pmap.rowBytes & 0x3fff;
+	numbytes = h * rowbytes;
 	img->rawcolrdata = xmalloc(numbytes);
 	datahandle = (*cicnhandle)->iconData;
-	for (i = 0; i < numbytes; ++i)
-	  img->rawcolrdata[i] = (*datahandle)[i];
+	j = 0;
+	for (i = 0; i < numbytes; ++i) {
+		if (i > 0 && (rowbytes == 1 || i % rowbytes == 0))
+		  j += (macrowbytes - rowbytes);
+		img->rawcolrdata[i] = (*datahandle)[j++];
+	}
 	/* Convert the cicn's monochrome icon if defined. */
-	rowbytes = (*cicnhandle)->iconBMap.rowBytes;
-	if (rowbytes > 0) {
-		baseaddr = ((char *) (*cicnhandle)->iconMaskData) + h * rowbytes;
-		numbytes = h * rowbytes;
+	bitrowbytes = computed_rowbytes(w, 1);
+	macbitrowbytes = (*cicnhandle)->iconBMap.rowBytes;
+	if (macbitrowbytes > 0) {
+		baseaddr = ((char *) (*cicnhandle)->iconMaskData) + h * macbitrowbytes;
+		numbytes = h * bitrowbytes;
 		img->rawmonodata = xmalloc(numbytes);
+		j = 0;
 		for (i = 0; i < numbytes; ++i) {
-			img->rawmonodata[i] = baseaddr[i];
-			/* For small icons, jump over a not-strictly-necessary padding char. */
-			if (w <= 8 && rowbytes == 2) ++i;
+			if (i > 0 && (bitrowbytes == 1 || i % bitrowbytes == 0))
+			  j += (macbitrowbytes - bitrowbytes);
+			img->rawmonodata[i] = baseaddr[j++];
 		}
 		*hasmono = TRUE;
 	}
-	/* Write the cicn's mask if one is defined. */
-	rowbytes = (*cicnhandle)->iconMask.rowBytes;
-	if (rowbytes > 0) {
+	/* Convert the cicn's mask if one is defined. */
+	macbitrowbytes = (*cicnhandle)->iconMask.rowBytes;
+	if (macbitrowbytes > 0) {
 		baseaddr = (char *) (*cicnhandle)->iconMaskData;
-		numbytes = h * rowbytes;
+		numbytes = h * bitrowbytes;
 		img->rawmaskdata = xmalloc(numbytes);
+		j = 0;
 		for (i = 0; i < numbytes; ++i) {
-			img->rawmaskdata[i] = baseaddr[i];
-			/* For small icons, jump over a not-strictly-necessary padding char. */
-			if (w <= 8 && rowbytes == 2) ++i;
+			if (i > 0 && (bitrowbytes == 1 || i % bitrowbytes == 0))
+			  j += (macbitrowbytes - bitrowbytes);
+			img->rawmaskdata[i] = baseaddr[j++];
 		}
 		*hasmask = TRUE;
 	}
 	HUnlock((Handle) cicnhandle);
 }
 
-/* Write a color table. */
+/* Generify the data in a color table. */
 
-Obj *
+static Obj *
 convert_ctab(CTabHandle ctabhandle)
 {
 	int c, ctsize;
@@ -789,26 +937,3 @@ convert_ctab(CTabHandle ctabhandle)
 	}
 	return rslt;
 }
-
-void
-mac_load_image_color(imc)
-ImageColor *imc;
-{
-	Handle imchandle;
-    Str255 namestr;
-
-	if (imc->name == NULL)
-	  return;
-	c2p(imc->name, namestr);
-	imchandle = GetNamedResource('XCic', namestr);
-	if (imchandle != nil) {
-		imc->r = ((short *) (*imchandle))[0];
-		imc->g = ((short *) (*imchandle))[1];
-		imc->b = ((short *) (*imchandle))[2];
-		imc->defined = 1;
-	} else {
-		/* Not found. */
-		imc->r = imc->g = imc->b = 0;
-		imc->defined = 0;
-	}
-}
