@@ -1,5 +1,5 @@
 /* Initialization for the Mac interface to Xconq.
-   Copyright (C) 1992, 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1992-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -7,14 +7,12 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING. */
 
 #include "conq.h"
-extern int numremotes;
-extern void low_send(int id, char *buf);
-extern int low_receive(int *id, char *buf, int maxchars, int timeout);
-extern int allbedesigners;
-extern void update_debugging PARAMS ((void));
-extern void toggle_debugging PARAMS ((int *flagp));
+extern void send_version PARAMS ((int rid));
+extern void debugging_string PARAMS ((char *buf));
 #include "macconq.h"
 
+extern char *connection_method_name;
+
 /* Limit on the number of check boxes and sliders in the variants dialog. */
 
 #define MAXCHECKBOXES 7
@@ -27,8 +25,6 @@ extern void toggle_debugging PARAMS ((int *flagp));
 
 #define game_already_loaded() (numutypes > 0)
 
-#define t_color_defined(t) (hasColorQD && tcolors != NULL && tcolors[t] != NULL)
-
 /* Private functions. */
 
 static pascal Boolean filter_splash(DialogPtr dialog, EventRecord *evt, short *itemhit);
@@ -175,15 +171,17 @@ int selectedplayer = -1;
 UserItemUPP draw_player_setup_list_proc;
 UserItemUPP draw_player_setup_advantage_proc;
 
+int you_player;
+
 /* Caches of images and colors. */
 
 ImageFamily **uimages = NULL;
 ImageFamily **timages = NULL;
 ImageFamily **eimages = NULL;
 
-ImageColor **tcolors = NULL;
+MacImage **tcolors = NULL;
 
-Pattern *animation_patterns;
+ Pattern *animation_patterns;
 
 CursHandle readprogressors[NUMcParens];
 CursHandle progressors[NUMcSynth];
@@ -195,6 +193,9 @@ CursHandle opencrosscursor;
 CursHandle firecursor;
 CursHandle watchcursor;
 
+CursHandle sendcursor;
+CursHandle receivecursor;
+
 /* This value is the current progress cursor. */
 
 int curcurs = 0;
@@ -217,8 +218,13 @@ enum grays gridgray = whitegray;
 enum grays unseengray = whitegray;
 enum grays bggray = mediumgray;
 
-RGBColor gridcolor;
-RGBColor unseencolor;
+// --- NOW DEFINED AS featColor[4] and featColor[5] WHICH ARE DECLARED ELSEWHERE --------
+
+//	RGBColor gridcolor;
+//	RGBColor unseencolor;
+
+// ----------------------------------------------------------------------
+
 RGBColor blackcolor;
 
 int grid_matches_unseen = FALSE;
@@ -282,6 +288,10 @@ init_cursors()
 	opencrosscursor = GetCursor(cOpenCross);
 	firecursor = GetCursor(138);
 	watchcursor = GetCursor(watchCursor);
+
+	sendcursor = GetCursor(cSendPacket);
+	receivecursor = GetCursor(cReceivePacket);
+
 	/* Designer-related cursors will be done later, if needed at all. */
 
 	dotdotdotpicture = (PicHandle) GetResource('PICT', 134);
@@ -302,7 +312,6 @@ filter_splash(DialogPtr dialog, EventRecord *evt, short *itemhit)
 			}
 #ifdef DEBUGGING
 			/* A secret way to get debugging at startup. */
-			/* (should have some sort of feedback?) */
 			if (ch == 'D') {
 				toggle_debugging(&Debug);
 			}
@@ -315,6 +324,17 @@ filter_splash(DialogPtr dialog, EventRecord *evt, short *itemhit)
 			if (ch == 'P') {
 				toggle_profiling();
 			}
+			{
+				char buf[BUFSIZE];
+				Str255 tmpstr;
+				short itemtype;  Handle itemhandle;  Rect itemrect;
+
+				GetDItem(dialog, diSplashVersion, &itemtype, &itemhandle, &itemrect);
+				debugging_string(buf);
+				c2p(buf, tmpstr);
+				SetIText(itemhandle, tmpstr);
+				DrawControls(dialog);
+			}
 #endif
 			break;
 	}
@@ -355,7 +375,7 @@ do_splash_box()
 	SelectWindow(win);
 	ModalDialog(filter_splash_proc, &ditem);
 	/* We don't loop around here, just return the ditem and let caller decide what to do. */
-	DisposDialog(win);
+	DisposeDialog(win);
 	return ditem;
 }
 
@@ -553,15 +573,9 @@ display_selected_game()
 	GetDItem(newgamewin, diNewGameBlurb, &itemtype, &itemhandle, &itemrect);
 	TESetSelect(0, 100000, newgametext);
 	TECut(newgametext);
-	if (selected_game /* && selected_game->complete */) {
-		if (selected_game->blurb) {
-			desc = selected_game->blurb;
-		} else {
-			desc = "??? experimental ???";
-		}
-	} else {
-		desc = "";
-	}
+	desc = "(no description)";
+	if (selected_game && !empty_string(selected_game->blurb))
+	  desc = selected_game->blurb;
 	TESetText(desc, strlen(desc), newgametext);
 	TEUpdate(&itemrect, newgametext);
 	/* Load a picture if one can be found. */
@@ -617,7 +631,7 @@ new_game_dialog()
 		newgamewin = GetNewDialog(dNewGame, NULL, (DialogPtr) -1);
 		SetPort(newgamewin);
 		TextFont(newYork);
-/*		TextSize(10); */
+		TextSize(12);
 		/* Set up the app-defined item that lists games. */
 		GetDItem(newgamewin, diNewGameList, &itemtype, &itemhandle, &itemrect);
 		SetDItem(newgamewin, diNewGameList, itemtype, (Handle) draw_game_list_proc, &itemrect);
@@ -641,7 +655,7 @@ new_game_dialog()
 		SetDItem(newgamewin, diNewGameBlurb, itemtype, (Handle) draw_game_blurb_proc, &itemrect);
 		destrect = itemrect;
 		viewrect = itemrect;
-/*		TextSize(10);  */
+		TextSize(12);
 		newgametext = TENew(&destrect, &viewrect);
 		GetDItem(newgamewin, diNewGamePicture, &itemtype, &itemhandle, &itemrect);
 		SetDItem(newgamewin, diNewGamePicture, itemtype, (Handle) draw_game_picture_proc, &itemrect);
@@ -679,7 +693,7 @@ new_game_dialog()
 		}
 	}
 	/* We're done, OK to throw away the dialog. */
-	DisposDialog(newgamewin);
+	DisposeDialog(newgamewin);
 	newgamewin = nil;
 	/* (should release TEs and lists also) */
 }
@@ -789,6 +803,8 @@ variants_dialog()
 	short ditem;
 	short itemtype;  Handle itemhandle;  Rect itemrect;
 
+	if (my_rid > 0)
+	  selected_game = modulelist;
 	if (selected_game == NULL)
 	  init_error("variants on a nonexistent game?");
 	interpret_variants();
@@ -880,7 +896,7 @@ variants_dialog()
 				break;
 		}
 	}
-	DisposDialog(variants_win);
+	DisposeDialog(variants_win);
 	variants_win = nil;
 	return changed;
 }
@@ -953,6 +969,9 @@ interpret_variants()
 				}
 				break;
 			case K_WORLD_SIZE:
+				/* If the area is already set up, it's too late. */
+				if (area.width > 0 || area.height > 0)
+				  break;
 				vary_world = TRUE;
 				/* Start with some defaults. */
 				new_circumference = DEFAULTCIRCUMFERENCE;
@@ -1019,7 +1038,9 @@ interpret_variants()
 	}
 }
 
-/* This is where we actually change the state of the game according to the variants. */
+/* This is where we actually change the state of the game according to the variants.
+   Actually, the kernel does the changing, this just collects the variant values and
+   passes them on. */
 
 static void
 implement_variants()
@@ -1076,10 +1097,6 @@ implement_variants()
 	}
 	/* (should implement the random slider variants) */
 	do_module_variants(selected_game, variants);
-	/* Recheck everything, the variants might have broken something. */
-	if (any_variants) {
-		check_game_validity();
-	}
 }
 
 /* Specially adapted version of xform. */
@@ -1091,7 +1108,6 @@ special_xform(int x, int y, int *sxp, int *syp)
 	*syp = (new_height - y) / new_scale;
 }
 
-
 /* This is a callback that draws the world and its areas. */
 
 /* (Would be really cool to draw accurate curvature over globe here) */
@@ -1444,7 +1460,7 @@ world_shape_dialog()
 				break;
 		}
 	}
-	DisposDialog(world_shape_win);
+	DisposeDialog(world_shape_win);
 	world_shape_win = nil;
 	return changed;
 }
@@ -1501,7 +1517,7 @@ real_time_dialog()
 				break;
 		}
 	}
-	DisposDialog(real_time_win);
+	DisposeDialog(real_time_win);
 	real_time_win = nil;
 	return changed;
 }
@@ -1542,7 +1558,7 @@ more_variants_dialog()
 				break;
 		}
 	}
-	DisposDialog(more_variants_win);
+	DisposeDialog(more_variants_win);
 	more_variants_win = nil;
 	return changed;
 }
@@ -1561,6 +1577,8 @@ draw_player_setup_list(WindowPtr win, short ditem)
 	short itemtype;  Handle itemhandle;  Rect itemrect;
 
 	GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
+	TextFont(newYork);
+	TextSize(12);
 	/* Default to normal size and spacing. */
 	playerh = 22;
 	playerbaseline = 14;
@@ -1571,7 +1589,7 @@ draw_player_setup_list(WindowPtr win, short ditem)
 		playerh = 10;
 		playerbaseline = 8;
 		numvisentries = (itemrect.bottom - itemrect.top) / playerh;
-		/* (should switch to smaller font also?) */
+		TextSize(10);
 	}
 	tmprgn = NewRgn();
 	GetClip(tmprgn);
@@ -1596,7 +1614,6 @@ draw_player_setup_list(WindowPtr win, short ditem)
 			if (empty_string(sidetitle))
 			  sidetitle = "(unnamed)";
 			MoveTo(entryrect.left + 2 + 16 + 2, entryrect.top + playerbaseline);
-			TextFont(newYork);
 			TextFace(0);
 			DrawText(sidetitle, 0, strlen(sidetitle));
 			/* Only draw other info for sides "in" this game. */
@@ -1612,9 +1629,9 @@ draw_player_setup_list(WindowPtr win, short ditem)
 				MoveTo(entryrect.left + 2 + 260, entryrect.top + playerbaseline);
 				TextFace(0);
 				DrawText(spbuf, 0, strlen(spbuf));
-				if (player->password) {
+				if (player->remotewanted) {
 					MoveTo(entryrect.right - 15, entryrect.top + playerbaseline);
-					DrawText((strcmp(player->password, "x") == 0 ? "r" : "R"), 0, 1);
+					DrawText((player->rid > 0 ? "R" : "r"), 0, 1);
 				}
 			}
 			/* (should be able to draw/update selection separately) */
@@ -1637,6 +1654,8 @@ draw_player_setup_list(WindowPtr win, short ditem)
 		}
 		OffsetRect(&entryrect, 0, playerh);
 	}
+	TextFont(0);
+	TextSize(12);
 	PenNormal();
 	SetClip(tmprgn);
 	DisposeRgn(tmprgn);
@@ -1726,6 +1745,18 @@ player_setup_dialog()
 	  draw_player_setup_list_proc = NewUserItemProc(draw_player_setup_list);
 	if (draw_player_setup_advantage_proc == NULL)
 	  draw_player_setup_advantage_proc = NewUserItemProc(draw_player_setup_advantage);
+	if (my_rid > 0 && you_player > 0) {
+		int i;
+		Player *player;
+		for (i = 0; i < numsides; ++i) {
+			player = assignments[i].player;
+			if (player->id == you_player) {
+				player->rid = 2;
+			} else {
+				player->rid = 1;
+			}
+		}
+	}
 	/* Create the window. */
 	if (playersetupwin == nil) {
 		playersetupwin = GetNewDialog(dPlayerSetup, NULL, (DialogPtr) -1L);
@@ -1745,8 +1776,6 @@ player_setup_dialog()
 	return TRUE;
 }
 
-int numremotewaiting;
-
 /* Set the state of the various buttons and controls.  In general, a side/player
    pair must be selected before anything can be done. */	
 
@@ -1756,14 +1785,18 @@ set_player_setup_button_states()
 	int locked = g_player_sides_locked();
 	int player = (selected_a_player() && !locked);
 	int i, numingame = 0;
+	Side *side = selected_side();
+	char buf[255];
 	short itemtype;  Handle itemhandle;  Rect itemrect;
+	WindowPtr win;
+	Str255 tmpstr;
 
 	numremotewaiting = 0;
 	for (i = 0; i < numsides; ++i) {
 		if (assignments[i].side && (assignments[i].side)->ingame) {
 			++numingame;
-			if ((assignments[i].player)->password) {
-				if (strcmp((assignments[i].player)->password, "x") == 0)
+			if ((assignments[i].player)->remotewanted) {
+				if ((assignments[i].player)->rid == 0)
 				  ++numremotewaiting;
 			}
 		}
@@ -1777,10 +1810,19 @@ set_player_setup_button_states()
 	set_player_button(diPlayerSetupOK, numremotewaiting == 0);
 	set_player_button(diPlayerSetupAdd, !locked && (numsides < g_sides_max()));
 	set_player_button(diPlayerSetupRemove, player && !locked && (numsides > g_sides_min()));
-	set_player_button(diPlayerSetupRename, player && !(selected_side()->nameslocked));
+	set_player_button(diPlayerSetupRename, player && !(side->nameslocked));
 	set_player_button(diPlayerSetupAI, player);
 	set_player_button(diPlayerSetupRemote, player);
 	set_player_button(diPlayerSetupExchange, player && numingame > 1);
+
+	GetDItem(playersetupwin, diPlayerSetupAdvantageRange, &itemtype, &itemhandle, &itemrect);
+	if (player && (side->minadvantage != side->maxadvantage)) {
+		sprintf(buf, "(%d - %d)", side->minadvantage, side->maxadvantage);
+	} else {
+		strcpy(buf, "");
+	}
+	c2p(buf, tmpstr);
+	SetIText(itemhandle, tmpstr);
 }
 
 /* Handle any event that we don't want DialogSelect to get. */
@@ -1832,7 +1874,7 @@ hit_player_setup_dialog(int ditem, Point pt)
 {
 	int playersok = FALSE, done = FALSE, changedselected = FALSE, changedall = FALSE;
 	int willhost = FALSE;
-	Player *tmpplayer;
+	Player *player, *tmpplayer;
 	Side *tmpside;
 	short itemtype;  Handle itemhandle;  Rect itemrect;
 
@@ -1842,15 +1884,23 @@ hit_player_setup_dialog(int ditem, Point pt)
 	switch (ditem) {
 		case diPlayerSetupOK:
 			playersok = TRUE;
-			/* Fall through. */
+			done = TRUE;
+			break;
 		case diPlayerSetupCancel:
+			/* Undo any remotes. */
+			for_all_players(player) {
+				player->remotewanted = NULL;
+			}
+			numremotewaiting = 0;
+			numremotes = 0;
 			done = TRUE;
 			break;
 		case diPlayerSetupAdd:
 			add_side_and_player();
 			/* Select the newly-created side/player pair. */
 			select_player(numsides - 1);
-			init_emblem_images(selected_side());
+			init_emblem_images(tmpside);
+			reinit_all_emblem_images();
 			changedall = TRUE;
 			break;
 		case diPlayerSetupRemove:
@@ -1867,7 +1917,7 @@ hit_player_setup_dialog(int ditem, Point pt)
 			tmpside->pluralnoun = NULL;
 			tmpside->adjective = NULL;
 			make_up_side_name(tmpside);
-			init_emblem_images(tmpside);
+			reinit_all_emblem_images();
 			changedselected = TRUE;
 			break;
 		case diPlayerSetupAI:
@@ -1881,11 +1931,15 @@ hit_player_setup_dialog(int ditem, Point pt)
 			break;
 		case diPlayerSetupRemote:
 			/* Toggle whether to wait for human to connect in */
-			/* (should not use password as flag...) */
-			if (tmpplayer->password) {
-				tmpplayer->password = NULL;
+			if (tmpplayer->remotewanted) {
+				tmpplayer->remotewanted = NULL;
 			} else {
-				tmpplayer->password = "x";
+				/* Serial line supports only one remote player. */
+				if (connection_method == 1 && numremotes == 2) {
+					SysBeep(20);
+					break;
+				}
+				tmpplayer->remotewanted = "x";
 				willhost = TRUE;
 			}
 			changedselected = TRUE;
@@ -1920,6 +1974,7 @@ hit_player_setup_dialog(int ditem, Point pt)
 		connection_method_dialog();
 		if (connection_method > 0) {
 			hosting = TRUE;
+			open_remote_connection(connection_method_name);
 		}
 	}
 	if (changedall || changedselected) {
@@ -1930,14 +1985,19 @@ hit_player_setup_dialog(int ditem, Point pt)
 	}
 	if (done) {
 		/* Get rid of this dialog, unlikely to need it again. */
-		DisposDialog(playersetupwin);
+		DisposeDialog(playersetupwin);
 		playersetupwin = nil;
+		/* Give all unassigned players to the master program. */
+		if (numremotes > 0) {
+			for_all_players(player) {
+			  if (player->rid == 0)
+			    player->rid = master_rid;
+			}
+		}
 	}
 	return playersok;
 }
 
-void download_to_player(Player *player);
-
 void
 add_remote_player(char *name)
 {
@@ -1949,10 +2009,14 @@ add_remote_player(char *name)
 	for (i = 0; i < numsides; ++i) {
 		if (assignments[i].side && (assignments[i].side)->ingame) {
 			player = assignments[i].player;
-			if (player->password && strcmp(player->password, "x") == 0) {
-				player->password = name;
+			if (player->remotewanted != NULL && player->rid == 0) {
+				player->displayname = copy_string(name);
+				player->rid = 2; /* for now */
+				send_version(player->rid);
 				download_to_player(player);
 				set_player_setup_button_states();
+				DrawDialog(playersetupwin);
+				draw_default_button(playersetupwin, diPlayerSetupOK);
 				return;
 			}
 		}
@@ -1961,37 +2025,11 @@ add_remote_player(char *name)
 	beep();
 }
 
-void send_assignment PARAMS ((int id, Side *side, Player *player));
-
 void
-send_assignment(id, side, player)
+set_you_player(id)
 int id;
-Side *side;
-Player *player;
 {
-    int remoteid;
-
-	low_send(id, "assign");
-	low_receive(&remoteid, spbuf, BUFSIZE, 0);
-}
-
-/* (should be a generic routine?) */
-void
-download_to_player(Player *player)
-{
-	int i, remoteid;
-
-	/* Have the remote program load the game. */
-	low_send(player->id, "load");
-	low_receive(&remoteid, spbuf, BUFSIZE, 0);
-	/* Send it the variant choices. */
-	low_send(player->id, "variants");
-	low_receive(&remoteid, spbuf, BUFSIZE, 0);
-	/* Send it the current list of side/player assignments. */
-	for (i = 0; i < numsides; ++i) {
-		send_assignment(player->id, assignments[i].side, assignments[i].player);
-	}
-	++numremotes;
+	you_player = id;
 }
 
 /* This dialog opens a player-specified file as a game. */
@@ -2075,15 +2113,15 @@ launch_game()
 		/* Cancelled from variants dialog, go back to wherever. */
 		return FALSE;
 	}
-	/* Load any colors first; we can use them to substitute for patterns. */
-	init_terrain_colors();
 	/* Set up the images we'll be needing. */
 	init_terrain_images();
 	init_unit_images();
+	init_side_colors();
 	/* Trial side/player assignment, will create the default player if nececessary. */
 	make_trial_assignments();
 	/* Get the emblems so they can be displayed in the player dialog. */
 	init_all_emblem_images();
+	init_emblem_cicns();
 	/* Ask for modifications to the trial assignment. */
 	return player_setup_dialog();
 }
@@ -2104,6 +2142,8 @@ launch_game_2()
 	run_synth_methods();
 	final_init();
 	assign_players_to_sides();
+	/* Build the Colors menu */
+	build_colors_menu();
 	/* No more interminable init calcs (we hope), so close the progress bar. */
 	close_progress_dialog();
 	/* Init the display now. */
@@ -2123,7 +2163,7 @@ check_for_missing_images()
 	Str255 tmpstr;
 
 	for_all_terrain_types(t) {
-		if (timages[t]->ersatz && !t_color_defined(t))
+		if (timages[t]->ersatz)
 		  record_missing_image(TTYP, t_type_name(t));
 	}
 	for_all_unit_types(u) {
@@ -2144,7 +2184,7 @@ check_for_missing_images()
 				break;
 			case 2:
 				/* Ugliness is unbearable, get out now. */
-				ExitToShell();
+				exit_macconq();
 				break;
 		}
 	}
@@ -2279,7 +2319,7 @@ close_progress_dialog()
 {
 	/* Restore the normal cursor. */
 	SetCursor(&QD(arrow));
-	DisposDialog(progresswin);
+	DisposeDialog(progresswin);
 	progresswin = nil;
 }
 
@@ -2306,10 +2346,6 @@ init_ui(Side *side)
 		side->ui = (UI *) xmalloc(sizeof(UI));
 		side->ui->active = FALSE;
 		dside = side;
-		/* Make sure this is allocated.  Note that xmalloc clears, so this
-		   will look like it has an empty Pascal string. */
-		if (curdatestr == NULL)
-		  curdatestr = (char *) xmalloc(100);
 		interp_mac_ui_data(find_at_key(dside->uidata, "mac"));
 	}
 }
@@ -2322,6 +2358,16 @@ init_display()
 {
 	int p;
 	Handle handle;
+	
+// --- ADDED TO PREVENT HEAP FRAGMENTATION (SEE BELOW) -------------------------
+
+	#if 0
+
+	Ptr	iconPtr;
+
+	#endif
+
+// ---------------------------------------------------------------------
 
 	/* If dside never got assigned, try some hasty repairs. */
 	if (dside == NULL) {
@@ -2355,20 +2401,66 @@ init_display()
 		bordbitmaps[p].rowBytes = 0;
 		connbitmaps[p].rowBytes = 0; 
 	}
+
+/*
+
+These two locked handles are loaded late and therefore mess up the heap. 
+Better to use pointers instead, which are loaded low. Suggested changes 
+are shown below. However, everything has now been commented out since 
+these ICNs are not used in the current version (7.1.92).
+
+*/
+
+// --- REWRITTEN CODE -----------------------------------
+
+#if 0
+
 	handle = GetResource('ICN#', 140);
 	if (handle != nil) {
+
+		/* Get a pointer (which is loaded low) */
+		iconPtr = NewPtr(GetHandleSize(handle));
+	
 		HLock(handle);
-		bordbitmaps[4].baseAddr = *handle;
+
+		/* Copy the ICN data to the pointer */
+		memcpy(iconPtr, *handle, GetHandleSize(handle));
+
+		HUnlock(handle);
+
+		/* Use the pointer instead */
+		bordbitmaps[4].baseAddr = iconPtr;
+//		bordbitmaps[4].baseAddr = *handle;
+
 		bordbitmaps[4].rowBytes = 4;
 		SetRect(&(bordbitmaps[4].bounds), 0, 0, 32, 32);
 	}
+	
 	handle = GetResource('ICN#', 141);
 	if (handle != nil) {
+
+		/* Get a pointer (which is loaded low) */
+		iconPtr = NewPtr(GetHandleSize(handle));
+	
 		HLock(handle);
-		connbitmaps[4].baseAddr = *handle;
+
+		/* Copy the ICN data to the pointer */
+		memcpy(iconPtr, *handle, GetHandleSize(handle));
+
+		HUnlock(handle);
+
+		/* Use the pointer instead */
+		connbitmaps[4].baseAddr = iconPtr;
+//		connbitmaps[4].baseAddr = *handle;
+
 		connbitmaps[4].rowBytes = 4;
 		SetRect(&(connbitmaps[4].bounds), 0, 0, 32, 32);
 	}
+	
+#endif
+
+// ------------------------------------------------------------------------------
+
 	/* Init assorted list heads and window pointers. */
 	maplist = NULL;
 	listlist = NULL;
@@ -2391,11 +2483,16 @@ init_display()
 		/* The notice window is always useful. */
 		create_notice_window();
 		ShowWindow(noticewin);
+
 		/* This will be the main playing display, so it should be up. */
-		create_map(5);
+		create_map(default_power);
+
 		/* This needs to come up so that players unfamiliar with the game
 		   will have some guidance. */
-		instructions_dialog();
+		if (show_instructions) {
+			instructions_dialog();
+		}
+
 		first_windows = FALSE;
 	}
 #ifdef DESIGNERS
@@ -2417,9 +2514,11 @@ init_unit_images()
 	uimages = (ImageFamily **) xmalloc(numutypes * sizeof(ImageFamily *));
 
 	for_all_unit_types(u) {
-		uimages[u] = get_unit_type_images(dside, u, mac_interp_imf, mac_load_imf,
+		uimages[u] = get_unit_type_images(dside, u, (pref_color_unit_images ? "c" : "s"),
+										  mac_interp_imf, mac_load_imf,
 										  add_default_unit_image);
 	}
+	init_unit_cicns();
 }
 
 /* The default unit image is a box with the name of the unit inside. */
@@ -2438,6 +2537,7 @@ add_default_unit_image(ImageFamily *imf, int u, char *name)
       return;
 	img->istile = FALSE;
 	macimg = get_mac_image(img);
+	/* (should define what grafport is in use) */
 	tmprgn = NewRgn();
 	GetClip(tmprgn);
 	SetRect(&tmprect, 0, 0, 32, 32);
@@ -2455,33 +2555,6 @@ add_default_unit_image(ImageFamily *imf, int u, char *name)
 	macimg->monopict = pichandle;
 }
 
-/* Allocate space and load a named color for each terrain type
-   that defines one. */
-
-void
-init_terrain_colors()
-{
-	int t;
-
-	if (hasColorQD) {
-		tcolors = (ImageColor **) xmalloc(numttypes * sizeof(ImageColor *));
-		for_all_terrain_types(t) {
-			if (!empty_string(t_color(t))) {
-				tcolors[t] = new_image_color(t_color(t));
-				mac_load_image_color(tcolors[t]);
-			}
-		}
-	}
-	interp_named_color(g_grid_color(), &gridgray, &gridcolor);
-	interp_named_color(g_unseen_color(), &unseengray, &unseencolor);
-	/* If the grid color and the unseen terrain color are the same, then we can
-	   draw more efficiently. */
-	if (strcmp(g_grid_color(), g_unseen_color()) == 0
-	    && empty_string(g_unseen_image_name()))
-	  grid_matches_unseen = TRUE;
-	blackcolor.red = blackcolor.green = blackcolor.blue = 0;
-}
-
 /* Set up the collection of terrain images/patterns, making a default pattern
    if necessary. */
 
@@ -2489,6 +2562,13 @@ void
 init_terrain_images()
 {
 	int t;
+	Image *timg;
+	MacImage *mactimg;
+
+	if (strcmp(gridcolor_name, unseencolor_name) == 0
+	    && empty_string(g_unseen_image_name()))
+	  grid_matches_unseen = TRUE;
+	blackcolor.red = blackcolor.green = blackcolor.blue = 0;
 
 	timages = (ImageFamily **) xmalloc(numttypes * sizeof(ImageFamily *));
 
@@ -2496,6 +2576,19 @@ init_terrain_images()
 		timages[t] = get_terrain_type_images(dside, t, mac_interp_imf, mac_load_imf,
 											 add_default_terrain_image);
 	}
+	if (hasColorQD) {
+		tcolors = (MacImage **) xmalloc(numttypes * sizeof(MacImage *));
+		for_all_terrain_types(t) {
+			for (timg = timages[t]->images; timg != NULL; timg = timg->next) {
+				mactimg = (MacImage *) timg->hook;
+				/* 1x1 images are actually colors. */
+				if (timg->w == 1 && timg->h == 1 && mactimg != NULL && mactimg->colordefined) {
+					tcolors[t] = mactimg;
+					break;
+				}
+			}
+		}
+	}
 	get_unseen_images(dside, mac_interp_imf, mac_load_imf, NULL);
 }
 
@@ -2511,6 +2604,7 @@ add_default_terrain_image(ImageFamily *imf, int t, char *name)
 	Image *img;
 	MacImage *macimg;
 
+	imf->ersatz = TRUE;
 	img = get_img(imf, 8, 8);
 	if (img == NULL)
 	  return;
@@ -2527,10 +2621,6 @@ add_default_terrain_image(ImageFamily *imf, int t, char *name)
 	SET_IMG_PAT (macimg, 6, 0x01);
 	SET_IMG_PAT (macimg, 7, 0xff);
 	macimg->patdefined = 1;
-	/* Count this type as having only a default pattern, but only if we can't
-	   possibly render the terrain with a solid color. */
-	if (!(hasColorQD && t_color_defined(t)))
-	  imf->ersatz = TRUE;
 }
 
 /* Given a color name, get actual RGB values for it.  As a special case for b/w
@@ -2539,7 +2629,7 @@ add_default_terrain_image(ImageFamily *imf, int t, char *name)
 void
 interp_named_color(char *name, enum grays *grayvar, RGBColor *colorvar)
 {
-	ImageColor tmpcolor;
+	Image *tmpimg;
 
 	if (strcmp(name, "black") == 0) {
 		*grayvar = blackgray;
@@ -2556,12 +2646,14 @@ interp_named_color(char *name, enum grays *grayvar, RGBColor *colorvar)
 	} else if (strcmp(name, "white") == 0) {
 		*grayvar = whitegray;
 		colorvar->red = colorvar->green = colorvar->blue = 65535;
-	} else if (hasColorQD) {
+	} else if (0 /* hasColorQD */) {
+#if 0
 		tmpcolor.name = name;
 		mac_load_image_color(&tmpcolor);
 		colorvar->red = tmpcolor.r;
 		colorvar->green = tmpcolor.g;
 		colorvar->blue = tmpcolor.b;
+#endif
 	}
 }
 
@@ -2579,6 +2671,20 @@ init_all_emblem_images()
 	}
 }
 
+void
+reinit_all_emblem_images()
+{
+	Side *side2;
+	int i;
+
+	dispose_emblem_cicns();
+	/* This isn't great, because all the collected data goes away, and we have to
+	   reread and reallocate everything.  It would be smarter just to reinit the
+	   Mac-specific parts alone. */
+	init_all_emblem_images();
+	init_emblem_cicns();
+}
+
 /* Set up the emblems for a particular side.  This is used both to do all sides
    initially and whenever a side changes emblems. */
 
@@ -2650,3 +2756,4 @@ add_default_emblem_image(ImageFamily *imf, int e, char *name)
 		macimg->monopict = pichandle;
 	}
 }
+
