@@ -1,6 +1,5 @@
 /* Commands for the X11 interface to Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
-   Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -8,6 +7,7 @@ the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
 #include "conq.h"
+#include "kpublic.h"
 #include "xconq.h"
 extern void create_orders_window PARAMS ((Side *side, Map *map));
 
@@ -30,12 +30,13 @@ static void execute_named_command PARAMS ((Side *side, Map *map,
 static int execute_named_command_from_table PARAMS ((Side *side, Map *map,
 						    char *cmdstr,
 						    CmdTab *cmdtab));
-static void describe_commands PARAMS ((int arg, char *key, char *buf));
-static void describe_command_table PARAMS ((int arg, char *key, char *buf,
-					   CmdTab *cmdtab));
+static void describe_commands PARAMS ((int arg, char *key, TextBuffer *buf));
+static void describe_command_table PARAMS ((int arg, char *key,
+					    TextBuffer *buf, CmdTab *cmdtab));
 static void aux_add_terrain PARAMS ((Side *side, Map *map, int cancel));
 static void aux_add_terrain_2 PARAMS ((Side *side, Map *map, int cancel));
 static void do_add_terrain_2 PARAMS ((Side *side, Map *map, int t));
+static void aux_attack PARAMS ((Side *side, Map *map, int cancel));
 static void aux_build PARAMS ((Side *side, Map *map, int cancel));
 static void impl_build PARAMS ((Side *side, Unit *unit, int u2, int n));
 static void aux_distance PARAMS ((Side *side, Map *map, int cancel));
@@ -46,6 +47,9 @@ static void aux_message PARAMS ((Side *side, Map *map, int cancel));
 static void aux_move_to PARAMS ((Side *side, Map *map, int cancel));
 static void aux_name PARAMS ((Side *side, Map *map, int cancel));
 static void aux_others PARAMS ((Side *side, Map *map, int cancel));
+static void aux_remove_terrain PARAMS ((Side *side, Map *map, int cancel));
+static void aux_remove_terrain_2 PARAMS ((Side *side, Map *map, int cancel));
+static void do_remove_terrain_2 PARAMS ((Side *side, Map *map, int t));
 static void aux_resign PARAMS ((Side *side, Map *map, int cancel));
 static void aux_resign_2 PARAMS ((Side *side, Map *map, int cancel));
 static void aux_leave_game PARAMS ((Side *side, Map *map, int cancel));
@@ -60,8 +64,6 @@ static void cmd_error PARAMS ((Side *side, char *fmt, ...));
 
 char tmpkey;
 
-Map *tmpmap;
-
 #define C(c) ((c)-0x40)
 
 #undef DEF_CMD
@@ -145,10 +147,10 @@ Side *side;
 Map *map;
 char *cmdstr;
 {
-    int found;
+    int prefix, found;
     char *cmdname;
 
-    parse_long_name_command(cmdstr, &cmdname, &cmdargstr, copy_string(cmdstr)); 
+    parse_long_name_command(cmdstr, &prefix, &cmdname, &cmdargstr, copy_string(cmdstr)); 
     if (empty_string(cmdname)) {
 	notify(side, "No command.");
     }
@@ -194,10 +196,11 @@ CmdTab *cmdtab;
 static void
 describe_commands(arg, key, buf)
 int arg;
-char *key, *buf;
+char *key;
+TextBuffer *buf;
 {
     describe_command_table(arg, key, buf, commands);
-    strcat(buf, "\n");
+    tbcat(buf, "\n");
     describe_command_table(arg, key, buf, xcommands);
 }
 
@@ -206,7 +209,8 @@ char *key, *buf;
 static void
 describe_command_table(arg, key, buf, cmdtab)
 int arg;
-char *key, *buf;
+char *key;
+TextBuffer *buf;
 CmdTab *cmdtab;
 {
     CmdTab *cmd;
@@ -214,23 +218,23 @@ CmdTab *cmdtab;
     for (cmd = cmdtab; cmd->name != NULL; ++cmd) {
 	if (cmd->fchar != '\0') {
 	    if (cmd->fchar < ' ' || cmd->fchar > '~') { 
-		tprintf(buf, "^%c", (cmd->fchar ^ 0x40));
+		tbprintf(buf, "^%c", (cmd->fchar ^ 0x40));
 	    } else {
-		tprintf(buf, " %c", cmd->fchar);
+		tbprintf(buf, " %c", cmd->fchar);
 	    }
-	    strcat(buf, " ");
-	    strcat(buf, cmd->help);
-	    strcat(buf, "\n");
+	    tbcat(buf, " ");
+	    tbcat(buf, cmd->help);
+	    tbcat(buf, "\n");
 	}
     }
     for (cmd = cmdtab; cmd->name != NULL; ++cmd) {
 	if (cmd->fchar == '\0') {
-	    strcat(buf, "\"");
-	    strcat(buf, cmd->name);
-	    strcat(buf, "\"");
-	    strcat(buf, " ");
-	    strcat(buf, cmd->help);
-	    strcat(buf, "\n");
+	    tbcat(buf, "\"");
+	    tbcat(buf, cmd->name);
+	    tbcat(buf, "\"");
+	    tbcat(buf, " ");
+	    tbcat(buf, cmd->help);
+	    tbcat(buf, "\n");
 	}
     }
 }
@@ -251,7 +255,7 @@ do_add_player(side, map)
 Side *side;
 Map *map;
 {
-    request_additional_side(cmdargstr);
+    net_request_additional_side(cmdargstr);
 }
 
 void
@@ -259,7 +263,7 @@ do_add_terrain(side, map)
 Side *side;
 Map *map;
 {
-    int u, t, numtypes, tfirst, possibles[MAXTTYPES];
+    int u, t, numtypes, tfirst = NONTTYPE, possibles[MAXTTYPES];
     Unit *unit = map->curunit;
 
     REQUIRE_UNIT(side, map);
@@ -275,7 +279,7 @@ Map *map;
 	}
     }
     if (numtypes == 0) {
-	cmd_error(side, "Cannot add terrain using this unit");
+	cmd_error(side, "%s cannot add or alter terrain!", unit_handle(side, unit));
     } else if (numtypes == 1) {
 	map->argunitid = unit->id;
 	do_add_terrain_2(side, map, tfirst);
@@ -297,15 +301,16 @@ int cancel;
       return;
     if (map->inpch == '?') {
 	notify(side, "Type a key to select terrain type.");
-	map->modalhandler = aux_add_terrain;
-	return;
     }
     if (grok_terrain_type(side, map, &t)) {
 	if (t != NONTTYPE) {
 	    do_add_terrain_2(side, map, t);
 	}
     } else {
-	/* beep? */
+        if (map->inpch != '?')
+	  beep(side);
+	/* Stay in this mode until we get it right. */
+	map->modalhandler = aux_add_terrain;
     }
 }
 
@@ -335,33 +340,33 @@ int cancel;
     if (cancel)
       return;
     if (map->inpch == '?') {
-	notify(side, "Just pick a location!");
+	notify(side, "Pick a location to add %s.", t_type_name(map->tmpt));
 	map->modalhandler = aux_add_terrain_2;
 	return;
     }
-    if (grok_position(side, map, &x, &y)) {
+    if (grok_position(side, map, &x, &y, &unit)) {
 	if (in_area(x, y)) {
 	    unit = find_unit(map->argunitid);
 	    if (in_play(unit) && side_controls_unit(side, unit)) {
 		switch (t_subtype(map->tmpt)) {
 		  case cellsubtype:
-		    prep_alter_cell_action(unit, unit, x, y, map->tmpt);
+		    net_prep_alter_cell_action(unit, unit, x, y, map->tmpt);
 		    break;
 		  case bordersubtype:
 		  case connectionsubtype:
 		    dir = closest_dir(x - unit->x, y - unit->y);
-		    prep_add_terrain_action(unit, unit, unit->x, unit->y, dir, map->tmpt);
+		    net_prep_add_terrain_action(unit, unit, unit->x, unit->y, dir, map->tmpt);
 		    break;
 		  case coatingsubtype:
-		    cmd_error(side, "can't add coatings yet");
+		    net_prep_add_terrain_action(unit, unit, unit->x, unit->y, 1, map->tmpt);
 		    break;
 		}
 	    }
 	}
 	restore_cur(side, map);
     } else {
+	beep(side);     
 	map->modalhandler = aux_add_terrain_2;
-	/* beep? */
     }
 }
 
@@ -371,9 +376,9 @@ Side *side;
 Map *map;
 {
     if (side_has_ai(side)) {
-	set_side_ai(side, NULL);
+	net_set_side_ai(side, NULL);
     } else {
-	set_side_ai(side, "mplayer");
+	net_set_side_ai(side, "mplayer");
     }
 }
 
@@ -382,7 +387,44 @@ do_attack(side, map)
 Side *side;
 Map *map;
 {
-    notify(side, "can't do generic attacks yet");
+    Unit *unit = map->curunit;
+    
+    REQUIRE_UNIT(side, map);
+    save_cur(side, map);
+    map->argunitid = unit->id;
+    ask_position(side, map, "Attack where? [click to set]", aux_attack);
+}
+
+static void
+aux_attack(side, map, cancel)
+Side *side;
+Map *map;
+int cancel;
+{
+    int x, y;
+    Unit *unit, *unit2;
+
+    if (cancel)
+      return;
+    if (map->inpch == '?') {
+	notify(side, "Click on a location");
+    }
+    if (grok_position(side, map, &x, &y, &unit2)) {
+	unit = find_unit(map->argunitid);
+	if (in_play(unit) && side_controls_unit(side, unit)) {
+	    if (unit2 != NULL) {
+		net_prep_attack_action(unit, unit, unit2, 100);
+	    } else {
+	      /* (should attack anything in cell) */
+	    }
+	} else {
+	    cmd_error(side, "unit disappeared!?");
+	}
+    } else {
+        if (map->inpch != '?')
+	  beep(side);
+	map->modalhandler = aux_attack;
+    }
 }
 
 void
@@ -394,7 +436,7 @@ Map *map;
     
     REQUIRE_UNIT(side, map);
     if (unit->plan) {
-	set_unit_ai_control(side, unit, !unit->plan->aicontrol, FALSE);
+	net_set_unit_ai_control(side, unit, !unit->plan->aicontrol, FALSE);
 	/* a hack */
 	unit->plan->waitingfortasks = !unit->plan->aicontrol;
     }
@@ -405,7 +447,7 @@ do_build(side, map)
 Side *side;
 Map *map;
 {
-    int u, u2, possibles[MAXUTYPES], numtypes, ufirst;
+    int u, u2, possibles[MAXUTYPES], numtypes, ufirst = NONUTYPE;
     Unit *unit = map->curunit;
 
     REQUIRE_UNIT(side, map);
@@ -414,12 +456,6 @@ Map *map;
 	cmd_error(side, "%s can't build anything!", unit_handle(side, unit));
 	return;
     }
-    if (unit->transport != NULL
-	&& !uu_occ_can_build(unit->transport->type, u)) {
-	cmd_error(side, "%s can't build anything while inside another unit!",
-		  unit_handle(side, unit));
-	return;
-    }
     numtypes = 0;
     for_all_unit_types(u2) {
 	if (uu_acp_to_create(u, u2) > 0 && type_allowed_on_side(u, side)) {
@@ -430,6 +466,15 @@ Map *map;
 	    possibles[u2] = FALSE;
 	}
     }
+    if (unit->transport != NULL
+	&& !uu_occ_can_build(unit->transport->type, u)
+	&& !(numtypes == 1
+	     && !completed(unit->transport)
+	     && uu_acp_to_build(u, unit->transport->type) > 0)) {
+	cmd_error(side, "%s can't build anything while inside another unit!",
+		  unit_handle(side, unit));
+	return;
+    }
     switch (numtypes) {
       case 0:
 	cmd_error(side, "Nothing to build!");
@@ -459,18 +504,20 @@ int cancel;
       return;
     if (map->inpch == '?') {
 	notify(side, "Type a key or click on a unit type to select build.");
-	map->modalhandler = aux_build;
-	return;
     }
     if (grok_unit_type(side, map, &u2)) {
 	if (u2 != NONUTYPE) {
 	    unit = find_unit(map->argunitid);
 	    if (in_play(unit) && side_controls_unit(side, unit)) {
 		impl_build(side, unit, u2, map->prefixarg);
+	    } else {
+		cmd_error(side, "unit disappeared!?");
 	    }
 	}
     } else {
-	/* beep? */
+        if (map->inpch != '?')
+	  beep(side);
+	map->modalhandler = aux_build;
     }
 }
 
@@ -481,10 +528,12 @@ Unit *unit;
 int u2, n;
 {
     if (n < 0)
+      n = unit_doctrine(unit)->construction_run[u2];
+    if (n <= 0)
       n = 99;
-    notify(side, "%s will build %d %s",
+    notify(side, "%s will build %d %s.",
 	   unit_handle(side, unit), n, u_type_name(u2));
-    push_build_task(unit, u2, n);
+    net_push_build_task(unit, u2, n);
 }
 
 void
@@ -495,7 +544,7 @@ Map *map;
     Unit *unit = map->curunit;
 
     REQUIRE_UNIT(side, map);
-    set_unit_plan_type(side, unit, PLAN_NONE);
+    net_set_unit_plan_type(side, unit, PLAN_NONE);
 }
 
 void
@@ -503,7 +552,9 @@ do_copying(side, map)
 Side *side;
 Map *map;
 {
-    notify(side, "You may copy freely.  See the file COPYING.");
+    /* Go directly to the copying help node. */
+    side->ui->curhelpnode = copying_help_node;
+    popup_help(side);
 }
 
 void
@@ -515,7 +566,7 @@ Map *map;
 
     if (unit != NULL) {
 	if (unit->plan)
-	  delay_unit(unit, TRUE);
+	  net_delay_unit(unit, TRUE);
     } else {
 	unit = find_next_awake_mover(side, map->curunit);
 	if (unit != map->curunit) {
@@ -536,12 +587,13 @@ Map *map;
 
     REQUIRE_UNIT(side, map);
     if (!completed(unit)) {
-	cmd_error(side, "%s is incomplete; cannot detach", unit_handle(side, unit));
+	cmd_error(side, "%s is incomplete; cannot detach",
+		  unit_handle(side, unit));
 	return;
     }
     rslt = check_transfer_part_action(unit, unit, unit->hp / 2, NULL);
     if (valid(rslt)) {
-	prep_transfer_part_action(unit, unit, unit->hp / 2, NULL);
+	net_prep_transfer_part_action(unit, unit, unit->hp / 2, NULL);
     } else {
 	notify(side, "can't detach for some reason?");
     }
@@ -558,7 +610,7 @@ Map *map;
     REQUIRE_UNIT(side, map);
     rslt = check_detonate_action(unit, unit, unit->x, unit->y, unit->z);
     if (valid(rslt)) {
-	prep_detonate_action(unit, unit, unit->x, unit->y, unit->z);
+	net_prep_detonate_action(unit, unit, unit->x, unit->y, unit->z);
     } else {
 	notify(side, "can't detonate for some reason?");
     }
@@ -658,7 +710,7 @@ Unit *unit;
     if (n > 1) {
 	DGprintf("Ordering %s to move %d %s\n",
 		 unit_desig(unit), n, dirnames[dir]);
-	set_move_dir_task(unit, dir, n);
+	net_set_move_dir_task(unit, dir, n);
     } else {
 	if (!point_in_dir(unit->x, unit->y, dir, &x, &y)) {
 	    return;
@@ -686,29 +738,10 @@ do_disband(side, map)
 Side *side;
 Map *map;
 {
-    int u;
     Unit *unit = map->curunit;
 
     REQUIRE_UNIT(side, map);
-    u = unit->type;
-#ifdef DESIGNERS
-    /* A designer can take out any unit, no questions asked. */
-    if (side->designer) {
-	kill_unit(unit, -1);
-	return;
-    }
-#endif /* DESIGNERS */
-    if (!completed(unit)) {
-	kill_unit(unit, H_UNIT_DISBANDED);
-    } else if (u_hp_per_disband(u) > 0) {
-	if (prep_disband_action(unit, unit)) {
-	    notify(side, "%s will go home.", unit_handle(side, unit));
-	} else {
-	    cmd_error(side, "odd failure");
-	}
-    } else {
-	cmd_error(side, "You can't just get rid of the %s!", u_type_name(u));
-    }
+    net_disband_unit(side, unit);
 }
 
 void
@@ -731,14 +764,14 @@ Map *map;
     /* Try moving into the transport's transport, if there is one. */
     if (transport->transport != NULL
         && can_occupy(unit, transport->transport)) {
-	prep_enter_action(unit, unit, transport->transport);
+	net_prep_enter_action(unit, unit, transport->transport);
 	/* (should be able to set up task if can't do action immedly) */
 	return;
     }
     /* Try moving into the open in the cell. */
     if (can_occupy_cell(unit, unit->x, unit->y)
 	|| can_occupy_conn(unit, unit->x, unit->y, unit->z)) {
-	prep_move_action(unit, unit, unit->x, unit->y, unit->z);
+	net_prep_move_action(unit, unit, unit->x, unit->y, unit->z);
 	/* (should be able to set up task if can't do action immedly) */
 	return;
     }
@@ -765,19 +798,19 @@ int cancel;
     if (cancel)
       return;
     if (map->inpch == '?') {
-	notify(side, "Not being helpful yet.");
+	notify(side, "Pick a location to which you want the distance.");
 	map->modalhandler = aux_distance;
 	return;
     }
-    if (grok_position(side, map, &x, &y)) {
+    if (grok_position(side, map, &x, &y, NULL)) {
 	if (in_area(x, y)) {
 	    dist = distance(map->savedcurx, map->savedcury, x, y);
-	    notify(side, "The distance is %d cells.", dist);
+	    notify(side, "Distance from the current location is %d cells.", dist);
 	}
 	restore_cur(side, map);
     } else {
+        beep(side);
 	map->modalhandler = aux_distance;
-	/* beep? */
     }
 }
 
@@ -791,13 +824,38 @@ Map *map;
     if (cmdargstr) {
 	side2 = parse_side_spec(cmdargstr);
 	if (side2 != NULL && side2 != side) {
-	    set_trust(side, side2, 0);
+	    net_set_trust(side, side2, 0);
 	    /* publicize? */
 	    return;
 	} 
     }
 }
 
+void
+do_doctrine(side, map)
+Side *side;
+Map *map;
+{
+  /* (should make generic) */
+  if (cmdargstr) {
+    if (strcmp(cmdargstr, "set") == 0) {
+      notify(side, "Set what?");
+    } else if (strncmp(cmdargstr, "set ", 4) == 0) {
+      net_set_doctrine(side, cmdargstr + 4);
+    }
+    return;
+  }
+  notify_doctrine(side, cmdargstr);
+}
+
+void
+do_down(side, map)
+Side *side;
+Map *map;
+{
+    beep(side);
+}
+
 void
 do_draw_willingness(side, map)
 Side *side;
@@ -805,7 +863,7 @@ Map *map;
 {
     if (map->prefixarg < 0)
       map->prefixarg = 1;
-    set_willing_to_draw(side, (map->prefixarg ? 1 : 0));
+    net_set_willing_to_draw(side, (map->prefixarg ? 1 : 0));
 }
 
 void
@@ -822,7 +880,7 @@ Map *map;
 	/* make sure its not the transport we're in and we can enter it */
 	if (transport != unit->transport &&
 	    valid(check_enter_action(unit, unit, transport))) {
-	    prep_enter_action(unit, unit, transport);
+	    net_prep_enter_action(unit, unit, transport);
 	    return;
 	}
 
@@ -830,7 +888,7 @@ Map *map;
 	for_all_occupants(transport, occ) {
 	    if (occ != unit->transport &&
 		valid(check_enter_action(unit, unit, occ))) {
-		prep_enter_action(unit, unit, occ);
+		net_prep_enter_action(unit, unit, occ);
 		return;
 	    }
 	}
@@ -843,7 +901,7 @@ do_end_turn(side, map)
 Side *side;
 Map *map;
 {
-    finish_turn(side);
+    net_finish_turn(side);
 }
 
 void
@@ -878,10 +936,10 @@ Map *map;
 		    if (other->side == unit->side) {
 			cmd_error(side, "You can't fire at one of your own units!");
 		    } else if (valid(check_fire_at_action(unit, unit, other, -1))) {
-			prep_fire_at_action(unit, unit, other, -1);
+			net_prep_fire_at_action(unit, unit, other, -1);
 		    }
 		} else {
-		    cmd_error(side, "Nothing there to fire at.");
+		    cmd_error(side, "Nothing there to fire at");
 		}
 	    }
 	}
@@ -899,7 +957,7 @@ int cancel;
 
     if (cancel)
       return;
-    if (grok_position(side, map, &x, &y)) {
+    if (grok_position(side, map, &x, &y, &unit)) {
 	unit = find_unit(map->argunitid);
 	if (x != unit->x || y != unit->y) {
 	    if (unit->act && unit->plan) { /* (should be more sophisticated?) */
@@ -909,7 +967,7 @@ int cancel;
 		    if (other->side == unit->side) {
 			cmd_error(side, "You can't fire at one of your own units!");
 		    } else if (valid(check_fire_at_action(unit, unit, other, -1))) {
-			prep_fire_at_action(unit, unit, other, -1);
+			net_prep_fire_at_action(unit, unit, other, -1);
 		    }
 		} else {
 		    cmd_error(side, "Nothing there to fire at");
@@ -941,7 +999,7 @@ Map *map;
 	if (x != unit->x || y != unit->y) {
 	    rslt = check_fire_into_action(unit, unit, x, y, 0, -1);
 	    if (valid(rslt)) {
-		prep_fire_into_action(unit, unit, x, y, 0, -1);
+		net_prep_fire_into_action(unit, unit, x, y, 0, -1);
 	    } else {
 		cmd_error(side, "%s fire into %d,%d not valid: %s",
 			  unit_handle(side, unit), x, y,
@@ -1053,6 +1111,21 @@ Map *map;
     popup_help(side);
 }
 
+/* Tell the unit to sit around for a given number of turns. */
+
+void
+do_idle(side, map)
+Side *side;
+Map *map;
+{
+    Unit *unit = map->curunit;
+
+    REQUIRE_UNIT(side, map);
+    if (map->prefixarg < 0)
+      map->prefixarg = 9999;
+    net_set_sentry_task(unit, map->prefixarg);
+}
+
 /* Send a short (1 line) message to another player.  Some messages are
    recognized specially, causing various actions. */
 
@@ -1101,7 +1174,7 @@ int cancel;
 	    sidemask = add_side_to_set(map->argside, NOSIDES);
 	}
 	if (!empty_string(msg) && sidemask != NOSIDES)
-	  send_message(side, sidemask, msg);
+	  net_send_message(side, sidemask, msg);
     } else {
 	map->modalhandler = aux_message;
     }
@@ -1136,15 +1209,15 @@ int cancel;
 
     if (cancel)
       return;
-    if (grok_position(side, map, &x, &y)) {
+    if (grok_position(side, map, &x, &y, &unit)) {
 	unit = find_unit(map->argunitid);
 #ifdef DESIGNERS
 	if (side->designer) {
-	    designer_teleport(unit, x, y, NULL);
+	    net_designer_teleport(unit, x, y, NULL);
 	} else
 #endif /* DESIGNERS */
 	if (in_play(unit)) {
-	    set_move_to_task(unit, x, y);
+	    net_set_move_to_task(unit, x, y);
 	} else {
 	    beep(side);
 	}
@@ -1184,7 +1257,7 @@ int cancel;
     if (grok_string(side, map, &name)) {
 	unit = find_unit(map->argunitid);
 	if (in_play(unit) && side_controls_unit(side, unit)) {
-	    set_unit_name(side, unit, name);
+	    net_set_unit_name(side, unit, name);
 	} else {
 	    cmd_error(side, "Nothing here that could be named!");
 	}
@@ -1336,7 +1409,7 @@ int cancel;
 	    ask_side(side, map, "Who do you want to surrender to?", NULL,
 		     aux_resign_2);
 	} else {
-	    resign_game(side, NULL);
+	    net_resign_game(side, NULL);
 	}
     }
 }
@@ -1352,7 +1425,7 @@ int cancel;
     if (cancel)
       return;
     grok_side(side, map, &side2);
-    resign_game(side, side2);
+    net_resign_game(side, side2);
 }
 
 /* Do the act of leaving the game. */
@@ -1412,13 +1485,156 @@ Map *map;
     draw_view_in_panner(side, map);
 }
 
+void
+do_release(side, map)
+Side *side;
+Map *map;
+{
+    Side *side2;
+
+    if (cmdargstr) {
+	side2 = parse_side_spec(cmdargstr);
+	if (side2 != NULL && side2 != side) {
+	    net_set_controlled_by(side2, side, FALSE);
+	    return;
+	}
+    }
+    beep(side);
+}
+
 void
 do_remove_terrain(side, map)
 Side *side;
 Map *map;
 {
+    int u, t, numtypes, tfirst = NONTTYPE, possibles[MAXTTYPES];
+    Unit *unit = map->curunit;
+
     REQUIRE_UNIT(side, map);
-    notify(side, "can't remove terrain yet");
+    u = unit->type;
+    numtypes = 0;
+    for_all_terrain_types(t) {
+	if (ut_acp_to_remove_terrain(u, t) > 0) {
+	    possibles[t] = TRUE;
+	    ++numtypes;
+	    tfirst = t;
+	} else {
+	    possibles[t] = FALSE;
+	}
+    }
+    if (numtypes == 0) {
+	cmd_error(side, "%s cannot remove terrain!", unit_handle(side, unit));
+    } else if (numtypes == 1) {
+	map->argunitid = unit->id;
+	do_remove_terrain_2(side, map, tfirst);
+    } else {
+	map->argunitid = unit->id;
+	ask_terrain_type(side, map, "Type to remove:",
+			 possibles, aux_remove_terrain);
+    }
+}
+
+static void
+aux_remove_terrain(side, map, cancel)
+Side *side;
+Map *map;
+int cancel;
+{
+    int t;
+
+    if (cancel)
+      return;
+    if (map->inpch == '?') {
+	notify(side, "Type a key to select terrain type to remove.");
+    }
+    if (grok_terrain_type(side, map, &t)) {
+	if (t != NONTTYPE) {
+	    do_remove_terrain_2(side, map, t);
+	}
+    } else {
+        if (map->inpch != '?')
+	  beep(side);
+	/* Stay in this mode until we get it right. */
+	map->modalhandler = aux_remove_terrain;
+    }
+}
+
+/* This is like do_remove_terrain, but with a terrain type given. */
+
+static void
+do_remove_terrain_2(side, map, t)
+Side *side;
+Map *map;
+int t;
+{
+    map->tmpt = t;
+    save_cur(side, map);
+    ask_position(side, map, "Remove where? [click to set]",
+		 aux_remove_terrain_2);
+}
+
+static void
+aux_remove_terrain_2(side, map, cancel)
+Side *side;
+Map *map;
+int cancel;
+{
+    int x, y, dir;
+    Unit *unit;
+
+    if (cancel)
+      return;
+    if (map->inpch == '?') {
+	notify(side, "Pick a location to remove %s.", t_type_name(map->tmpt));
+	map->modalhandler = aux_remove_terrain_2;
+	return;
+    }
+    if (grok_position(side, map, &x, &y, NULL)) {
+	if (in_area(x, y)) {
+	    unit = find_unit(map->argunitid);
+	    if (in_play(unit) && side_controls_unit(side, unit)) {
+		switch (t_subtype(map->tmpt)) {
+		  case cellsubtype:
+		    cmd_error(side, "can't remove cell terrain!");
+		    break;
+		  case bordersubtype:
+		  case connectionsubtype:
+		    dir = closest_dir(x - unit->x, y - unit->y);
+		    net_prep_remove_terrain_action(unit, unit, unit->x, unit->y, dir, map->tmpt);
+		    break;
+		  case coatingsubtype:
+		    net_prep_remove_terrain_action(unit, unit, unit->x, unit->y, 1, map->tmpt);
+		    break;
+		}
+	    }
+	}
+	restore_cur(side, map);
+    } else {
+	beep(side);     
+	map->modalhandler = aux_remove_terrain_2;
+    }
+}
+
+void
+do_repair(side, map)
+Side *side;
+Map *map;
+{
+    Unit *unit, *other;
+
+    REQUIRE_UNIT(side, map);
+    unit = map->curunit;
+    if (valid(check_repair_action(unit, unit, unit))) {
+	net_prep_repair_action(unit, unit, unit);
+	return;
+    }
+    other = unit->transport;
+    if (other != NULL
+	&& valid(check_repair_action(other, other, unit))) {
+	net_prep_repair_action(other, other, unit);
+	return;
+    }
+    cmd_error(side, "No repair possible right now");
 }
 
 void
@@ -1427,24 +1643,23 @@ Side *side;
 Map *map;
 {
     REQUIRE_UNIT(side, map);
-    set_unit_reserve(side, map->curunit, TRUE, TRUE /* is this right??? */);
+    net_set_unit_reserve(side, map->curunit, TRUE, TRUE /* is this right??? */);
 }
 
-/* (should reindent) */
 void
 do_resign(side, map)
 Side *side;
 Map *map;
 {
-	if (endofgame) {
-		notify(side, "Game is already over.");
-		beep(side);
-	} else if (!side->ingame) {
-		notify(side, "You are already out of the game.");
-		beep(side);
-	} else {
-		ask_bool(side, map, "You really want to resign?", FALSE, aux_resign_b);
-	}
+    if (endofgame) {
+	notify(side, "Game is already over.");
+	beep(side);
+    } else if (!side->ingame) {
+	notify(side, "You are already out of the game.");
+	beep(side);
+    } else {
+	ask_bool(side, map, "You really want to resign?", FALSE, aux_resign_b);
+    }
 }
 
 /* This is semi-redundant with aux_resign. */
@@ -1458,7 +1673,7 @@ int cancel;
     if (cancel)
       return;
     if (grok_bool(side, map)) {
-	resign_game(side, NULL);
+	net_resign_game(side, NULL);
     }
 }
 
@@ -1476,7 +1691,7 @@ Map *map;
     REQUIRE_UNIT(side, map);
     /* (should doublecheck range and error out if no chance) */
     if (in_play(unit)) {
-	set_resupply_task(unit, NONMTYPE);
+	net_set_resupply_task(unit, NONMTYPE);
     }
 }
 
@@ -1650,22 +1865,22 @@ int cancel;
 	map->modalhandler = aux_set_formation;
 	return;
     }
-    if (grok_position(side, map, &x, &y)) {
+    if (grok_position(side, map, &x, &y, &unit)) {
 	unit = find_unit(map->argunitid);
 	if (in_play(unit) && in_area(x, y)) {
 	    leader = unit_at(x, y);
 	    if (in_play(leader)
 		&& leader != unit
 		&& trusted_side(unit->side, leader->side)) {
-		set_formation(unit, leader, leader->x - unit->x, leader->y - unit->y, 1, 1);
+		net_set_formation(unit, leader, leader->x - unit->x, leader->y - unit->y, 1, 1);
 	    } else {
 		cmd_error(side, "Nobody here to form on!");
 	    }
 	}
 	restore_cur(side, map);
     } else {
+	beep(side);
 	map->modalhandler = aux_set_formation;
-	/* beep? */
     }
 }
 
@@ -1692,7 +1907,7 @@ Map *map;
     Unit *unit = map->curunit;
 
     REQUIRE_UNIT(side, map);
-    set_unit_asleep(side, unit, TRUE, TRUE /* is this right??? */);
+    net_set_unit_asleep(side, unit, TRUE, TRUE /* is this right??? */);
 }
 
 void
@@ -1720,11 +1935,19 @@ Map *map;
 }
 
 void
-do_surrender_to(side, map)
+do_submit(side, map)
 Side *side;
 Map *map;
 {
-    notify(side, "can't surrender to a side yet");
+    Side *side2;
+
+    if (cmdargstr) {
+	side2 = parse_side_spec(cmdargstr);
+	if (side2 != NULL && side2 != side) {
+	    net_set_controlled_by(side, side2, TRUE);
+	    return;
+	} 
+    }
 }
 
 void
@@ -1751,51 +1974,19 @@ do_take(side, map)
 Side *side;
 Map *map;
 {
-    int something = FALSE;
-    int u, u2, m, need, actual, n;
-    Unit *unit = map->curunit, *unit2;
-    char buf[BUFSIZE];
+    short m, amts[MAXMTYPES], rslts[MAXMTYPES], needed;
+    Unit *unit = map->curunit;
 
     if (nummtypes == 0) {
 	cmd_error(side, "No materials in this game!");
 	return;
     }
     REQUIRE_UNIT(side, map);
-    u = unit->type;
-    unit2 = unit->transport;
-    if (unit2 == NULL) {
-	cmd_error(side, "Nothing to take from here!");
-	return;
-    }
-    u2 = unit2->type;
-    n = (map->prefixarg < 0 ? 1 : map->prefixarg);
-
-    buf[0] = '\0';
     for_all_material_types(m) {
-	need = (n < 0 ? um_storage_x(u, m) - unit->supply[m] : n);
-	if (need > 0) {
-	    something = TRUE;
-	    /* Be stingy if we're low */
-	    if (2 * unit2->supply[m] < um_storage_x(u2, m))
-	      need = max(1, need/2);
-	    actual = transfer_supply(unit2, unit, m, need);
-	    tprintf(buf, " %d %s", actual, m_type_name(m));
-	}
+	amts[m] = map->prefixarg;
     }
-    if (something) {
-	notify(side, "%s got%s.", unit_handle(side, unit), buf);
-    } else {
-	notify(side, "%s needed nothing.", unit_handle(side, unit));
-    }
-}
-
-void
-do_take_unit(side, map)
-Side *side;
-Map *map;
-{
-    REQUIRE_UNIT(side, map);
-    notify(side, "can't take units yet");
+    needed = take_supplies(unit, amts, rslts);
+    report_take(side, unit, needed, rslts);
 }
 
 void
@@ -1808,12 +1999,20 @@ Map *map;
     if (cmdargstr) {
 	side2 = parse_side_spec(cmdargstr);
 	if (side2 != NULL && side2 != side) {
-	    set_trust(side, side2, 1);
+	    net_set_trust(side, side2, 1);
 	    return;
 	} 
     }
 }
 
+void
+do_up(side, map)
+Side *side;
+Map *map;
+{
+    beep(side);
+}
+
 /* Command to display the program version. */
 
 void
@@ -1832,13 +2031,13 @@ Map *map;
 {
     /* Do the curunit explicitly, might only be an occupant. */
     if (in_play(map->curunit)) {
-	wake_unit(map->curunit, FALSE, 0, NULL);
+	net_wake_unit(side, map->curunit, FALSE);
 	draw_map_info(side, map);
     }
     /* If an argument was given, apply to all "top-level" units
        within the radius specified by the argument. */
     if (map->prefixarg >= 0)
-      wake_area(side, map->curx, map->cury, map->prefixarg, FALSE);
+      net_wake_area(side, map->curx, map->cury, map->prefixarg, FALSE);
 }
 
 void
@@ -1849,7 +2048,7 @@ Map *map;
     int n;
 
     n = (map->prefixarg < 0 ? 0 : map->prefixarg);
-    wake_area(side, map->curx, map->cury, n, TRUE);
+    net_wake_area(side, map->curx, map->cury, n, TRUE);
 }
 
 void
@@ -1857,7 +2056,9 @@ do_warranty(side, map)
 Side *side;
 Map *map;
 {
-    notify(side, "There is no warranty.");
+	/* Go directly to the warranty help node. */
+	side->ui->curhelpnode = warranty_help_node;
+    popup_help(side);
 }
 
 /* X-interface-specific commands. */
@@ -1915,6 +2116,30 @@ Map *map;
     notify(side, "can't close a map yet");
 }
 
+void
+do_x_meridians(side, map)
+Side *side;
+Map *map;
+{
+    if (map->prefixarg > 0) {
+	side->ui->default_latlong_interval = map->prefixarg;
+	map->vp->latlong_interval = side->ui->default_latlong_interval;
+	map->vp->draw_latlong = TRUE;
+    } else {
+	map->vp->draw_latlong = FALSE;
+    }
+    do_refresh(side, map);
+}
+
+void
+do_x_prefer_pattern(side, map)
+Side *side;
+Map *map;
+{
+    side->ui->pref_solid_color_terrain = FALSE;
+    do_refresh(side, map);
+}
+
 void
 do_x_reverse_video(side, map)
 Side *side;
@@ -1930,6 +2155,15 @@ Map *map;
     }
 }
 
+void
+do_x_prefer_solid_color(side, map)
+Side *side;
+Map *map;
+{
+    side->ui->pref_solid_color_terrain = TRUE;
+    do_refresh(side, map);
+}
+
 void
 do_x_unit_closeup(side, map)
 Side *side;
@@ -1937,12 +2171,12 @@ Map *map;
 {
     Unit *unit = map->curunit;
     UnitCloseup *unitcloseup;
+
     REQUIRE_UNIT(side, map);
-    
-    unitcloseup = find_unit_closeup(unit, side);
+    unitcloseup = find_unit_closeup(side, unit);
     if (!unitcloseup)
-      unitcloseup = create_unit_closeup(unit, side, map);
-    draw_unit_closeup(unitcloseup, side);
+      unitcloseup = create_unit_closeup(side, map, unit);
+    draw_unit_closeup(side, unitcloseup);
 }
 
 void
@@ -1960,12 +2194,12 @@ Map *map;
 {
     Unit *unit = map->curunit;
     SideCloseup *sidecloseup;
+
     REQUIRE_UNIT(side, map);
-    
-    sidecloseup = find_side_closeup(unit->side, side);
+    sidecloseup = find_side_closeup(side, unit->side);
     if (!sidecloseup)
-      sidecloseup = create_side_closeup(unit->side, side, map);
-    draw_side_closeup(sidecloseup, side);
+      sidecloseup = create_side_closeup(side, map, unit->side);
+    draw_side_closeup(side, sidecloseup);
 }
 
 /* Create a new world map (a regular map zoomed to display the whole
@@ -2026,7 +2260,7 @@ Map *map;
 	    really_do_design(side);
 	}
     } else {
-	become_nondesigner(side);
+	net_become_nondesigner(side);
 	for_all_maps(side, map2) {
 	    map2->seeall = (all_see_all || side->may_set_see_all);
 	    update_controls(side, map2);
@@ -2060,7 +2294,7 @@ Side *side;
     int u;
     Map *map2;
 
-    become_designer(side);
+    net_become_designer(side);
     for_all_maps(side, map2) {
 	map2->seeall = (all_see_all || side->may_set_see_all);
 	update_controls(side, map2);
@@ -2077,7 +2311,7 @@ Side *side;
 Map *map;
 {
     if (cmdargstr)
-      interp_form(NULL, read_form_from_string(cmdargstr, NULL, NULL));
+      interp_form(NULL, read_form_from_string(cmdargstr, NULL, NULL, NULL));
     /* (should broadcast to all?) */
 }
 
@@ -2132,9 +2366,7 @@ int cancel;
     if (cancel)
       return;
     if (map->inpch == '?') {
-	notify(side, "Not being helpful yet.");
-	map->modalhandler = aux_set_terrain_type;
-	return;
+	notify(side, "Type a key to select terrain type to paint.");
     }
     if (grok_terrain_type(side, map, &t2)) {
 	if (t2 != NONUTYPE) {
@@ -2145,7 +2377,9 @@ int cancel;
 		   side->ui->curbrushradius, t_type_name(side->ui->curttype));
 	}
     } else {
-	/* beep? */
+        if (map->inpch != '?')
+	  beep(side);
+	map->modalhandler = aux_set_terrain_type;
     }
 }
 
@@ -2158,8 +2392,8 @@ Map *map;
 {
     if (!check_designer_status(side, "paint terrain"))
       return;
-    paint_cell(side, map->curx, map->cury,
-	       side->ui->curbrushradius, side->ui->curttype);
+    net_paint_cell(side, map->curx, map->cury,
+		   side->ui->curbrushradius, side->ui->curttype);
 }
 
 /* Command to set the current unit type to create. */
@@ -2203,9 +2437,7 @@ int cancel;
     if (cancel)
       return;
     if (map->inpch == '?') {
-	notify(side, "Not being helpful yet.");
-	map->modalhandler = aux_set_unit_type;
-	return;
+	notify(side, "Type a key or click on a unit type to select creation type.");
     }
     if (grok_unit_type(side, map, &u2)) {
 	if (u2 != NONUTYPE) {
@@ -2216,7 +2448,9 @@ int cancel;
 		   side->ui->cursidenumber, u_type_name(side->ui->curutype));
 	}
     } else {
-	/* beep? */
+        if (map->inpch != '?')
+	  beep(side);
+	map->modalhandler = aux_set_unit_type;
     }
 }
 
@@ -2231,9 +2465,10 @@ Map *map;
 
     if (!check_designer_status(side, "create units"))
       return;
-    unit = designer_create_unit(side,
-				side->ui->curutype, side->ui->cursidenumber,
-				map->curx, map->cury);
+    unit = net_designer_create_unit(side,
+				    side->ui->curutype,
+				    side->ui->cursidenumber,
+				    map->curx, map->cury);
     if (unit != NULL) {
 	set_current_unit(side, map, unit);
     } else {
@@ -2267,6 +2502,9 @@ Map *map;
 #ifndef DebugG
     DebugG = !DebugG;
 #endif
+    /* Prevent batching-up of server actions, so that if you stop in the
+       middle of drawing operations, the screen reflects what has been
+       sent to the server so far. */
     XSynchronize(side->ui->dpy, DebugG);
 }
 
