@@ -1,29 +1,23 @@
 /* Definitions for the X11 interface to Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
-   Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.  See the file COPYING.  */
 
-/* Default color of text and icons - 0 is for white on black, 1
-   is for black on white.  Should be set appropriately for the most
-   common monochrome display (color displays always do white on black).
-   This is also settable by the player, so the default is just for
+/* Default color of text and icons - 0 is for white on black, 1 is for
+   black on white.  Should be set appropriately for the most common
+   monochrome display (color displays always do white on black).  This
+   is also settable by the player, so the default is just for
    convenience of the majority. */
 
 #define BLACKONWHITE 1
 
-#define BARGRAPHS
-
 /* The default fonts can be altered by users, so these are just hints. */
 
 #define TEXTFONT "fixed"
 
-#define ICONFONT "xconq"
-#define STANDARD "standard"
-
 /* Some X11 servers die if too much is written between output flushes. */
 
 /* #define STUPIDFLUSH */
@@ -74,6 +68,7 @@ any later version.  See the file COPYING.  */
 #endif /* IMFLIB */
 
 enum grayshade {
+  black,
   gray,
   darkgray,
   numgrays
@@ -85,7 +80,6 @@ enum whattouse {
   dontdraw,
   useblocks,
   usepictures,
-  usefontchars,
   usepolygons
 };
 
@@ -105,9 +99,14 @@ enum controltypeenum {
     SHOW_FEATURE_BOUNDARIES,
     SHOW_NUMBERS,
     SHOW_PEOPLE,
+    SHOW_CONTROL,
     SHOW_ELEV,
     SHOW_TEMP,
+    SHOW_CLOUDS,
+    SHOW_WINDS,
+    SHOW_COVER,
     SHOW_ALL,
+    SHOW_LATLONG,
     SHOW_MORE,
     COLR_UNITS,
     COLR_TERR,
@@ -138,8 +137,19 @@ enum tooltypes {
   numtools
 };
 
+enum movie_type {
+  movie_null,
+  movie_miss,
+  movie_hit,
+  movie_death,
+  movie_nuke,
+  movie_sound,
+  movie_flash
+};
+
 struct a_movie {
-  enum movie_type type;
+  char *type;
+  enum movie_type itype;
   int args[5];
 };
 
@@ -197,6 +207,8 @@ typedef struct a_ui {
     short dflt_color_unit_images;
     short dflt_color_terr_images;
     short dflt_color_embl_images;
+    short pref_solid_color_terrain;
+    short default_latlong_interval;
     GC gc;			/* a tmp graphics context for this display */
     GC textgc;			/* foreground on background text */
     GC ltextgc;			/* foreground on background text */
@@ -298,6 +310,7 @@ typedef struct a_ui {
     Widget orders_shell;
     Widget orders_label;
     Widget orders_form;
+    Widget orders_radio;
     Widget orders_help_shell;
     StandingOrder *sorder_edit;
     char *sorder_types_edit;
@@ -313,13 +326,15 @@ typedef struct a_ui {
     Pixmap controlpics[numcontrols];
     Pixmap toolcursors[numtools];
     Pixmap unitcursors[MAXUTYPES];
-    Pixmap boxcurs, boxmask;
     Pixmap grays[numgrays];
+    Pixmap dots;
     int numscheduled;
     struct a_movie movies[10];
+    short told_outcome;
 } UI;
 
 /* A closer look at a side. */
+
 typedef struct a_side_closeup {
     struct a_map *map;
     struct a_side *side;
@@ -328,6 +343,7 @@ typedef struct a_side_closeup {
 } SideCloseup;
 
 /* A list of units. */
+
 typedef struct a_unit_list {
     struct a_map *map;
     int number;
@@ -338,6 +354,7 @@ typedef struct a_unit_list {
 } UnitList;
 
 /* A closer look at a unit. */
+
 typedef struct a_unit_closeup {
     struct a_map *map;
     struct a_unit *unit;
@@ -346,6 +363,7 @@ typedef struct a_unit_closeup {
 } UnitCloseup;
 
 /* List of unit closeups */
+
 typedef struct a_closeup_summary {
     int number;
     UnitCloseup **unitcloseups;
@@ -353,7 +371,8 @@ typedef struct a_closeup_summary {
     Widget shell, label, list;
 } CloseupSummary;
  
-/* Orders interface */
+/* Orders interface. */
+
 typedef struct a_order_interface {
     StandingOrder *sorder;
     Widget form, toggle, types, etype, eparms, task, tparms;
@@ -362,14 +381,17 @@ typedef struct a_order_interface {
 } OrderInterface;
 
 /* Iteration over all of a side's unit closeups. */
+
 #define for_all_unit_closeups(c,s)  \
   for ((c) = (s)->ui->unitcloseuplist; (c) != NULL; (c) = (c)->next)
 
 /* Iteration over all of a side's unit lists. */
+
 #define for_all_unit_lists(l,s)  \
   for ((l) = (s)->ui->unitlistlist; (l) != NULL; (l) = (l)->next)
 
 /* Iteration over all of a side's side closeups. */
+
 #define for_all_side_closeups(c,s)  \
   for ((c) = (s)->ui->sidecloseuplist; (c) != NULL; (c) = (c)->next)
 
@@ -415,24 +437,8 @@ typedef struct a_map {
     Pixmap panner_pix;
     /* How to draw the map. */
     short seeall;		/* True if viewing world data directly */
-    short drawterrain;		/* Display solid color terrain on the map? */
-    short drawgrid;		/* Draw outlines around cells? */
-    short drawcellpats;		/* Draw terrain patterns? */
-    short drawunits;		/* Display units on the map? */
-    short drawnames;		/* Display unit names/numbers on the map? */
     short drawfeatureboundaries;
     short drawfeaturenames;	/* true if displaying names of features */
-    short oldesttoshow;		/* the relative time of the oldest data */
-    short agetofade;		/* point at which to gray out the display */
-    short newesttoshow;		/* the relative time of the newest data */
-    short fadeterrain;		/* fade world features as well as units? */
-    short lowestlevel;		/* the lowest level of units to display */
-    short highestlevel;		/* the highest level of units to display */
-    short drawpeople;		/* true if displaying people sides */
-    short drawelevations;		/* true if display elevations */
-    short drawresources[MAXMTYPES];
-    short drawtemp;
-    short drawweather;
     short use_color_unit_images;
     short use_color_terr_images;
     short use_color_embl_images;
@@ -450,9 +456,6 @@ typedef struct a_map {
     short list1w, list1h;
     short list2w, list2h;
     short panw, panh;
-    short vx, vy;		/* lower left of viewport in cell coords */
-    short vw, vh;		/* Boundaries of viewport in cell coords */
-    short vw2, vh2;		/* 1/2 (rounded down) of above values */
     short sidespacing;		/* Vertical space for each side in side list */
     short last_num_in_play[MAXUTYPES];
     short last_num_incomplete[MAXUTYPES];
@@ -477,6 +480,7 @@ typedef struct a_map {
     char answer[BUFSIZE];	/* string being typed in */
     int tmpint;
     short frombutton;		/* true if command was issued from button */
+    short anim_state;
     /* Link to the next map. */
     struct a_map *next;
 } Map;
@@ -492,6 +496,8 @@ typedef struct a_map {
 #define unit_color(s,s2,u,n)  \
   (((s)->ui->numunitcolors)[3 * (numutypes * (s2) + (u)) + (n)])
 
+#define side_has_local_display(side) ((side)->ui != NULL)
+
 /* Values shared by all displays and all sides. */
 
 extern Widget thistoplevel;
@@ -526,17 +532,18 @@ extern void ask_string PARAMS ((Side *side, Map *map, char *prompt, char *dflt,
 extern int grok_unit_type PARAMS ((Side *side, Map *map, int *typep));
 extern int grok_terrain_type PARAMS ((Side *side, Map *map, int *typep));
 extern int grok_side PARAMS ((Side *side, Map *map, Side **side2p));
-extern int grok_position PARAMS ((Side *side, Map *map, int *xp, int *yp));
+extern int grok_position PARAMS ((Side *side, Map *map, int *xp, int *yp,
+				  Unit **unitp));
 extern int grok_bool PARAMS ((Side *side, Map *map));
 extern int grok_string PARAMS ((Side *side, Map *map, char **strp));
 
-extern int xform PARAMS ((Side *side, Map *map, int x, int y,
-			 int *sxp, int *syp));
-extern int x_xform_unit PARAMS ((Side *side, Map *map, Unit *unit,
-				int *sxp, int *syp, int *swp, int *shp));
-extern int x_xform_unit_self PARAMS ((Side *side, Map *map, Unit *unit,
+extern void xform PARAMS ((Side *side, Map *map, int x, int y,
+			   int *sxp, int *syp));
+extern void x_xform_unit PARAMS ((Side *side, Map *map, Unit *unit,
+				   int *sxp, int *syp, int *swp, int *shp));
+extern void x_xform_unit_self PARAMS ((Side *side, Map *map, Unit *unit,
 				     int *sxp, int *syp, int *swp, int *shp));
-extern int x_xform_occupant PARAMS ((Side *side, Map *map,
+extern void x_xform_occupant PARAMS ((Side *side, Map *map,
 				    Unit *transport, Unit *unit,
 				    int sx, int sy, int sw, int sh,
 				    int *sxp, int *syp, int *swp, int *shp));
@@ -616,7 +623,7 @@ extern void draw_row PARAMS ((Side *side, Map *map,
 			     int x0, int y0, int len, int clearit));
 extern void draw_unit_image PARAMS ((Side *side, Window win,
 				    int sx, int sy, int sw, int sh,
-				    int u, int s2, int fg, int bg));
+				    int u, int s2, int fg, int bg, int mod));
 extern void draw_side_emblem PARAMS ((Side *side, Window win,
 				     int ex, int ey, int ew, int eh,
 				     int s2, int style));
@@ -688,7 +695,6 @@ void place_legends PARAMS ((Side *side));
 
 void enable_in_unit_type_list PARAMS ((Side *side, Map *map, int u, int flag));
 
-extern int smallest_image PARAMS ((ImageFamily *imf, int *wp, int *hp));
 extern void zoom_in_out PARAMS ((Side *side, Map *map, int which));
 
 extern int find_side_and_map_via_ctrlpanel_form PARAMS ((Widget w,
@@ -741,15 +747,23 @@ extern void handle_designer_map_click PARAMS ((Side *side, Map *map,
 extern void popdown_design PARAMS ((Side *side));
 #endif /* DESIGNERS */
 
-extern UnitCloseup *find_unit_closeup PARAMS ((Unit *unit, Side *side));
-extern UnitCloseup *find_unit_closeup_via_button PARAMS ((Widget w, Side *side));
-extern UnitCloseup *create_unit_closeup PARAMS ((Unit *unit, Side *side, Map *map));
-extern void draw_unit_closeup PARAMS ((UnitCloseup *unitcloseup, Side *side));
-extern void destroy_unit_closeup PARAMS ((UnitCloseup *unitcloseup, Side *side));
-extern SideCloseup *find_side_closeup PARAMS ((Side *side1, Side *side));
-extern SideCloseup *find_side_closeup_via_button PARAMS ((Widget w, Side *side));
-extern SideCloseup *create_side_closeup PARAMS ((Side *side1, Side *side, Map *map));
-extern void draw_side_closeup PARAMS ((SideCloseup *sidecloseup, Side *side));
-extern void destroy_side_closeup PARAMS ((SideCloseup *sidecloseup, Side *side));
+extern UnitCloseup *find_unit_closeup PARAMS ((Side *side, Unit *unit));
+extern UnitCloseup *find_unit_closeup_via_button PARAMS ((Widget w,
+							  Side *side));
+extern UnitCloseup *create_unit_closeup PARAMS ((Side *side, Map *map,
+						 Unit *unit));
+extern void draw_unit_closeup PARAMS ((Side *side, UnitCloseup *unitcloseup));
+extern void destroy_unit_closeup PARAMS ((Side *side,
+					  UnitCloseup *unitcloseup));
+extern SideCloseup *find_side_closeup PARAMS ((Side *side, Side *side2));
+extern SideCloseup *find_side_closeup_via_button PARAMS ((Widget w,
+							  Side *side));
+extern SideCloseup *create_side_closeup PARAMS ((Side *side, Map *map,
+						 Side *side2));
+extern void draw_side_closeup PARAMS ((Side *side, SideCloseup *sidecloseup));
+extern void destroy_side_closeup PARAMS ((Side *side,
+					  SideCloseup *sidecloseup));
 extern Pixmap get_unit_picture PARAMS ((int utype, Side *side));
 extern Pixmap get_side_picture PARAMS ((Side *side, Side *side1));
+
+void raise_widget PARAMS ((Widget w));
