@@ -1,6 +1,5 @@
 /* Initialization for the X11 interface to Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
-   Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1998 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -9,7 +8,6 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 #include "xconq.h"
-extern int smallest_image PARAMS ((ImageFamily *imf, int *wp, int *hp));
 
 extern XrmOptionDescRec xoptions[];
 extern int xoptions_count;
@@ -18,6 +16,7 @@ extern int xoptions_count;
 void abort ();
 #endif
 
+static void set_optional_colors PARAMS ((Side *side));
 static void init_gcs PARAMS ((Side *side));
 static void init_fonts PARAMS ((Side *side));
 static void init_bitmaps PARAMS ((Side *side));
@@ -31,6 +30,7 @@ static void get_default_terrain_image PARAMS ((Side *side, ImageFamily *imf,
 					      int t));
 static void get_default_emblem PARAMS ((Side *side, ImageFamily *imf,
 				       Side *side2));
+static void report_missing_images PARAMS ((Side *side));
 static void describe_imf PARAMS ((Side *side, char *classname, char *typename,
 				 ImageFamily *imf));
 
@@ -144,9 +144,6 @@ int min_h_for_unit_image = 16;
 #include "bitmaps/buildcurs.b"
 #include "bitmaps/buildmask.b"
 
-#include "bitmaps/boxcurs.b"
-#include "bitmaps/boxmask.b"
-
 #include "bitmaps/bomb1.b"
 #include "bitmaps/bomb2.b"
 #include "bitmaps/bomb3.b"
@@ -156,8 +153,6 @@ int min_h_for_unit_image = 16;
 #include "bitmaps/hit.b"
 #include "bitmaps/kill.b"
 
-#include "bitmaps/hex8.b"
-#include "bitmaps/hex8b.b"
 #include "bitmaps/hex16.b"
 #include "bitmaps/hex16b.b"
 #include "bitmaps/hex32.b"
@@ -169,6 +164,8 @@ int min_h_for_unit_image = 16;
 #include "bitmaps/gray.b"
 #include "bitmaps/darkgray.b"
 
+#include "bitmaps/dots.b"
+
 #ifdef DESIGNERS
 #include "bitmaps/hexcurs.b"
 #include "bitmaps/hexcursmask.b"
@@ -255,17 +252,17 @@ Side *side;
 
 /* Set up all sides' displays all at once. */
 
-int numdisplays;
-
 void
 init_all_displays()
 {
+    int numdisplays;
     Side *side;
 
     numdisplays = 0;
     for_all_sides(side) {
 	if (side_has_display(side)) {
-	    init_display(side);
+	    if (side_has_local_display(side))
+	      init_display(side);
 	    ++numdisplays;
 	}
     }
@@ -291,8 +288,9 @@ init_redraws()
     }
 }
 
-/* Open display, create all the windows we'll need, do misc setup things,
-   and initialize some globals to out-of-range values for recognition later. */
+/* Open display, create all the windows we'll need, do misc setup
+   things, and initialize some globals to out-of-range values for
+   recognition later. */
 
 int toplevel_display_used = FALSE;
 
@@ -349,6 +347,7 @@ Side *side;
     side->ui->dflt_color_terr_images = TRUE;
     side->ui->dflt_color_unit_images = TRUE;
     side->ui->dflt_color_embl_images = TRUE;
+    side->ui->default_latlong_interval = 600;
     side->ui->bonw = BLACKONWHITE;
     depth = XDisplayCells(dpy, side->ui->screen);
     DGprintf("%d different colors available ...\n", depth);
@@ -380,6 +379,8 @@ Side *side;
     init_unit_images(side);
     init_terrain_images(side);
     init_emblem_images(side);
+    report_missing_images(side);
+    set_optional_colors(side);
     init_bitmaps(side);
     init_gcs(side);
     init_cursors(side);
@@ -402,9 +403,6 @@ Side *side;
       popup_design(side);
 #endif /* DESIGNERS */
 
-    /* Preallocate space needed to save display state. */
-    ui_save_state(side);
-
     Dprintf("Successfully initialized `%s'!\n", dpyname);
 }
 
@@ -417,7 +415,6 @@ set_colors(side)
 Side *side;
 {
     int	t;
-    char *colorname, substname[BUFSIZE];
     char *unseencolorname = "black";
     char *gridcolorname = "black";
 
@@ -438,23 +435,20 @@ Side *side;
       unseencolorname = g_unseen_color();
     side->ui->unseencolor =
       (strcmp(unseencolorname, "white") == 0 ? side->ui->whitecolor : side->ui->blackcolor);
+    /* Set colors to distinctive default; will set usefully later. */
     for_all_terrain_types(t) {
-	if (side->ui->monochrome) {
-	    side->ui->cellcolor[t] = side->ui->blackcolor;
-	} else {
-	    colorname = t_color(t);
-	    /* Warn if we have to substitute a valid color name. */
-	    if (empty_string(colorname)) {
-		/* Substitute grays 33 to 99 only. */
-		sprintf(substname, "gray%d", ((t * 66) / numttypes) + 33);
-		init_warning("no color for terrain type \"%s\", using \"%s\"",
-			     t_type_name(t), substname);
-		colorname = substname;
-	    }
-	    side->ui->cellcolor[t] = request_color(side, colorname);
-	}
+	side->ui->cellcolor[t] = -1;
     }
-    /* Get special-purpose colors. */
+}
+
+/* This will set up colors that are not crucial to the game.  By doing
+   after basic image color allocation, these won't suck up colormap
+   space that should go to images. */
+
+static void
+set_optional_colors(side)
+Side *side;
+{
     if (!side->ui->monochrome) {
 	side->ui->diffcolor = request_color(side, "maroon");
 	side->ui->graycolor = request_color(side, "light gray");
@@ -462,11 +456,40 @@ Side *side;
 	side->ui->neutcolor = request_color(side, "gray");
 	side->ui->goodcolor = request_color(side, "green");
 	side->ui->badcolor = request_color(side, "red");
-	side->ui->gridcolor = request_color(side, gridcolorname);
-	side->ui->unseencolor = request_color(side, unseencolorname);
     }
 }
 
+/* Collect all the names of types etc for which no images could be found,
+   report them all at once. */
+
+static void
+report_missing_images(side)
+Side *side;
+{
+  int u, t, e;
+  Side *side2;
+
+  for_all_terrain_types(t) {
+    if (side->ui->timages[t] == NULL
+	|| side->ui->timages[t]->ersatz)
+      record_missing_image(TTYP, t_type_name(t));
+  }
+  for_all_unit_types(u) {
+    if (side->ui->uimages[u] == NULL
+	|| side->ui->uimages[u]->ersatz)
+      record_missing_image(UTYP, u_type_name(u));
+  }
+  for_all_sides(side2) {
+    e = side_number(side2);
+    if (side->ui->eimages[e] == NULL
+	|| side->ui->eimages[e]->ersatz)
+      record_missing_image(3, side_desig(side));
+  }
+  if (missing_images(tmpbuf)) {
+    init_warning("Could not find %s", tmpbuf);
+  }
+}
+
 /* A predicate that tests whether our display can safely be written to. */
 /* (should have a macro version for internal-to-interface use?) */
 
@@ -565,7 +588,7 @@ Side *side;
 /* Get a color set up and warn if not getting what was asked for.  We can */
 /* tolerate being off somewhat.  (Note X rgb value range is 0-65535.) */
 
-#define CLOSE_ENOUGH(X,Y) (ABS(((int) X) - ((int) Y)) < 2000)
+#define CLOSE_ENOUGH(X,Y) (ABS(((int) X) - ((int) Y)) < 4000)
 
 long
 request_color(side, name)
@@ -619,10 +642,6 @@ Side *side;
     Window win = side->ui->rootwin;
 
     /* Get the solid hex outlines. */
-    side->ui->hexpics[3] =
-      XCreateBitmapFromData(dpy, win, hex8_bits, hex8_width, hex8_height);
-    side->ui->bhexpics[3] =
-      XCreateBitmapFromData(dpy, win, hex8b_bits, hex8b_width, hex8b_height);
     side->ui->hexpics[4] =
       XCreateBitmapFromData(dpy, win, hex16_bits, hex16_width, hex16_height);
     side->ui->bhexpics[4] =
@@ -631,6 +650,8 @@ Side *side;
       XCreateBitmapFromData(dpy, win, hex32_bits, hex32_width, hex32_height);
     side->ui->bhexpics[5] =
       XCreateBitmapFromData(dpy, win, hex32b_bits, hex32b_width, hex32b_height);
+    /* (should use for larger images? polygons have poor borders) */
+
     /* Get pictures of mushroom clouds. */
     side->ui->bombpics[0] =
       XCreateBitmapFromData(dpy, win, bomb1_bits, bomb1_width, bomb1_height);
@@ -646,14 +667,14 @@ Side *side;
       XCreateBitmapFromData(dpy, win, hit_bits, hit_width, hit_height);
     side->ui->hitpics[2] =
       XCreateBitmapFromData(dpy, win, kill_bits, kill_width, kill_height);
-    side->ui->boxcurs =
-      XCreateBitmapFromData(dpy, win, boxcurs_bits, boxcurs_width, boxcurs_height);
-    side->ui->boxmask =
-      XCreateBitmapFromData(dpy, win, boxmask_bits, boxmask_width, boxmask_height);
+
     side->ui->grays[gray] =
       XCreateBitmapFromData(dpy, win, gray_bits, gray_width, gray_height);
     side->ui->grays[darkgray] =
       XCreateBitmapFromData(dpy, win, darkgray_bits, darkgray_width, darkgray_height);
+    side->ui->dots =
+      XCreateBitmapFromData(dpy, win, dots_bits, dots_width, dots_height);
+
     /* Collect pictures that will be used with map controls. */
     for (i = 0; i < numcontrols; ++i)
       side->ui->controlpics[i] = None;
@@ -821,24 +842,27 @@ static void
 init_unit_images(side)
 Side *side;
 {
-    int u, w, h;
+    int u;
     ImageFamily *imf;
+    Image *smallest;
 
     side->ui->uimages =
       (ImageFamily **) xmalloc(numutypes * sizeof(ImageFamily *));
     tmp_display = side->ui->dpy;
     tmp_root_window = side->ui->rootwin;
     for_all_unit_types(u) {
-	imf = get_unit_type_images(side, u, x11_interp_fn, x11_load_fn, NULL);
+	imf = get_unit_type_images(side, u, "s", x11_interp_fn, x11_load_fn,
+				   NULL);
 	/* (should fabricate something as a substitute) */
 	if (DebugG)
 	  describe_imf(side, "unit type", u_type_name(u), imf);
 	side->ui->uimages[u] = imf;
-	if (smallest_image(imf, &w, &h)) {
-	    if (w > min_w_for_unit_image)
-	      min_w_for_unit_image = w;
-	    if (h > min_h_for_unit_image)
-	      min_h_for_unit_image = h;
+	smallest = smallest_image(imf);
+	if (smallest) {
+	    if (smallest->w > min_w_for_unit_image)
+	      min_w_for_unit_image = smallest->w;
+	    if (smallest->h > min_h_for_unit_image)
+	      min_h_for_unit_image = smallest->h;
 	}
     }
 }
@@ -859,7 +883,8 @@ Side *side;
     tmp_display = side->ui->dpy;
     tmp_root_window = side->ui->rootwin;
     for_all_terrain_types(t) {
-	imf = get_terrain_type_images(side, t, x11_interp_fn, x11_load_fn, NULL);
+	imf = get_terrain_type_images(side, t, x11_interp_fn, x11_load_fn,
+				      NULL);
 	if (imf != NULL && imf->numsizes == 0)
 	  get_default_terrain_image(side, imf, t);
 	if (DebugG)
@@ -867,41 +892,63 @@ Side *side;
 	side->ui->timages[t] = imf;
 	/* Precalculate which images to use at which magnifications. */
 	for (p = 0; p < NUMPOWERS; ++p) {
-	    if (1) {
-		timg = best_image(side->ui->timages[t], hws[p], hhs[p]);
-		if (timg != NULL) {
-		    ximg = (X11Image *) timg->hook;
-		    if (ximg != NULL) {
-			if (!side->ui->monochrome
-/* if somebody changes their mind about using the color images, this will lose */
- 			    && side->ui->dflt_color_terr_images
- 			    && ximg->colr != None)
-			  side->ui->terrpics[p][t] = ximg->colr;
-			else
-			  side->ui->terrpics[p][t] = ximg->mono;
-		    }
+	    timg = best_image(side->ui->timages[t], hws[p], hhs[p]);
+	    if (timg != NULL) {
+		ximg = (X11Image *) timg->hook;
+		if (ximg != NULL) {
+		    if (!side->ui->monochrome
+			/* if somebody changes their mind about using
+                           the color images, this will lose */
+			&& side->ui->dflt_color_terr_images
+			&& ximg->colr != None)
+		      side->ui->terrpics[p][t] = ximg->colr;
+		    else
+		      side->ui->terrpics[p][t] = ximg->mono;
 		}
 	    }
 	}
+	/* Look for a solid color. */
+	timg = best_image(side->ui->timages[t], 1, 1);
+	if (timg->w == 1 && timg->h == 1) {
+	    if (timg->hook) {
+		ximg = (X11Image *) timg->hook;
+		/* Save the color if found. */
+		if (ximg->colpix) 
+		  side->ui->cellcolor[t] = ximg->colpix[0];
+	    }
+	}
     }
 }
 
+/* The default terrain image is ugly but functional; basically a binary
+   encoding of the terrain type number, enclosed in a box. */
+
 static void
 get_default_terrain_image(side, imf, t)
 Side *side;
 ImageFamily *imf;
 int t;
 {
+    char bits[8];
     Image *img;
     X11Image *ximg;
 
-    img = get_img(imf, 20, 22);
+    img = get_img(imf, 8, 8);
     img->minw = 1;  img->minh = 1;
     img->maxw = 999;  img->maxh = 999;
+    img->istile = TRUE;
     ximg = get_x11_image(img);
+    bits[0] = 0x01;
+    bits[1] = (t << 2) + 0x01;
+    bits[2] = (t << 2) + 0x01;
+    bits[3] = 0x7d;
+    bits[4] = (t << 2) + 0x01;
+    bits[5] = (t << 2) + 0x01;
+    bits[6] = 0x01;
+    bits[7] = 0xff;
     ximg->mono = 
-      XCreateBitmapFromData(side->ui->dpy, side->ui->rootwin,
-			    hex16b_bits, hex16b_width, hex16b_height);
+      XCreateBitmapFromData(side->ui->dpy, side->ui->rootwin, bits, 8, 8);
+    imf->ersatz = TRUE;
 }
 
 /* Set up a side's view of everybody else's colors and emblems. */
@@ -939,6 +986,7 @@ Side *side, *side2;
        them for our own display. */
     if (!side->ui->monochrome && !empty_string(side2->colorscheme)) {
 	/* take spec apart and iterate for multiple colors */
+        /* (should be generic code) */
 	for (s = side2->colorscheme, c = cbuf, i = 0; i < 3; ++s) {
 	    if (*s == ',' || *s == '\0') {
 		*c = '\0';
@@ -1007,11 +1055,15 @@ ImageFamily *imf;
 	DGprintf("    %dx%d", img->w, img->h);
 	ximg = (X11Image *) img->hook;
 	if (ximg)
-	  DGprintf(" (x11 mono %d color %d mask %d)", ximg->mono, ximg->colr, ximg->mask);
+	  DGprintf(" (x11 mono %d color %d mask %d)",
+		   ximg->mono, ximg->colr, ximg->mask);
 	DGprintf("\n");
     }
 }
 
+/* Pop up a dialog listing all the standard games in the library, plus
+   a bit of description of each one. */
+
 void 
 popup_game_dialog ()
 {
@@ -1019,7 +1071,8 @@ popup_game_dialog ()
     Widget outform, label, quit, viewp, inform;
     Module *module;
     
-    games = (Gamew *) xmalloc(numgames*sizeof(Gamew));
+    collect_possible_games();
+    games = (Gamew *) xmalloc(numgames * sizeof(Gamew));
     
     gameShell =
       XtVaCreatePopupShell("gameDialog", topLevelShellWidgetClass, thistoplevel,
@@ -1076,7 +1129,7 @@ popup_game_dialog ()
 				  XtNright,  XawChainLeft,
 				  NULL);
 	XtAddCallback (games[i].name, XtNcallback, game_callback, NULL);
-	if (module->title && module->title[0]) {
+	if (!empty_string(module->title)) {
 	    games[i].title =
 	      XtVaCreateManagedWidget("title", labelWidgetClass, games[i].form,
 				      XtNlabel, module->title,
@@ -1089,7 +1142,7 @@ popup_game_dialog ()
 	} else {
 	    games[i].title = NULL;
 	}
-	if (module->blurb && module->blurb[0]) {
+	if (!empty_string(module->blurb)) {
 	    games[i].blurb =
 	      XtVaCreateManagedWidget("blurb", labelWidgetClass, games[i].form,
 				      XtNlabel, module->blurb,
@@ -1428,7 +1481,7 @@ int g, flag;
 	}
     }
     XtPopup (pop->shell, XtGrabNone);
-    XMapRaised(XtDisplay(pop->shell), XtWindow(pop->shell));
+    raise_widget(pop->shell);
 }
 
 static Widget
@@ -1584,7 +1637,7 @@ XtPointer client_data;
 XtPointer call_data;
 {
     GamePop *pop = games[chosen_game].pop;
-    char *value;
+    char *value, *name;
     int i, flag;
 
     if (!valid_gamepop()) {
@@ -1604,24 +1657,26 @@ XtPointer call_data;
     /* get side values */
     for (i = 0; i < numsides; ++i) {
 	if (pop->assign[i].pname) {
+	    name = NULL;
 	    value = XawDialogGetValueString(pop->assign[i].pname);
-	    if (value && value[0]) {
-		assignments[i].player->name = copy_string(value);
-	    } else {
-		assignments[i].player->name = NULL;
-	    }
+	    if (!empty_string(value))
+	      name = copy_string(value);
+	    if (assignments[i].player)
+	      assignments[i].player->name = name;
 	    if (pop->assign[i].messname)
 	      free(pop->assign[i].messname);
 	}
 	if (pop->assign[i].pid) {
+	    name = NULL;
 	    value = XawDialogGetValueString(pop->assign[i].pid);
 	    flag = type_player(value);
-	    if (ABS(flag) == 1) {
-		assignments[i].player->displayname = copy_string(value);
-		assignments[i].player->aitypename  = NULL;
-	    } else {
-		assignments[i].player->displayname = NULL;
-		assignments[i].player->aitypename  = copy_string(value);
+	    if (!empty_string(value))
+	      name = copy_string(value);
+	    if (assignments[i].player) {
+		assignments[i].player->displayname =
+		  (ABS(flag) == 1 ? name : NULL);
+		assignments[i].player->aitypename =
+		  (ABS(flag) != 1 ? name : NULL);
 	    }
 	    if (pop->assign[i].messid)
 	      free(pop->assign[i].messid);
@@ -1675,7 +1730,7 @@ XtPointer call_data;
 {
     GamePop *pop = games[chosen_game].pop;
     char *value;
-    int i, flag;
+    int i, flag = 0; /* init to keep GCC quiet */
 
     /* save variant values */
     for (i = 0; i < pop->numvariants; ++i) {
@@ -1730,7 +1785,7 @@ Variant *var;
 	if (var->dflt == lispnil)  return "true";
 	return c_number(eval(var->dflt)) ? " true" : "false";
       case K_WORLD_SIZE:
-	sprintlisp(buf3, var->dflt);
+	sprintlisp(buf3, var->dflt, BUFSIZE);
 	for (p = buf3; *p; ++p) {
 	    if (!isdigit(*p))
 	      *p = ' ';
@@ -1748,7 +1803,7 @@ Variant *var;
 	}
 	return buf3;
       default:
-	sprintlisp(buf3, var->dflt);
+	sprintlisp(buf3, var->dflt, BUFSIZE);
 	return buf3;
     }
 }
@@ -1956,11 +2011,11 @@ init_assignw_player(assignw, player)
 Assignw *assignw;
 Player *player;
 {
-    assignw->vname = (player->name ? player->name : "");
-    if (player->displayname) {
+    assignw->vname = (player && player->name ? player->name : "");
+    if (player && player->displayname) {
 	assignw->vid = player->displayname;
 	assignw->vflag = 1;
-    } else if (player->aitypename) {
+    } else if (player && player->aitypename) {
 	assignw->vid = player->aitypename;
 	assignw->vflag = 2;
     } else {
