@@ -1,6 +1,5 @@
 /* Map handling for the X11 interface to Xconq.
-   Copyright (C) 1987, 1988, 1989, 1991, 1992, 1993, 1994, 1995, 1996
-   Stanley T. Shebs.
+   Copyright (C) 1987-1989, 1991-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -12,19 +11,15 @@ any later version.  See the file COPYING.  */
 
 #include "conq.h"
 extern int num_features PARAMS ((void));
+#include "kpublic.h"
 #include "xconq.h"
 void update_unit_controls PARAMS ((Side *side, Map *map));
 
 static void create_map_controls PARAMS ((Side *side, Map *map));
-
 static void interp_key_command PARAMS ((Side *side, Map *map));
-
 static void handle_map_click PARAMS ((Side *side, Map *map, int sx, int sy));
-
 static void help_unit_type PARAMS ((Side *side, Map *map));
-
 static void help_terrain_type PARAMS ((Side *side, Map *map));
-
 static void mode_callback PARAMS ((Widget w, XtPointer client_data,
 				  XtPointer call_data));
 static void command_callback PARAMS ((Widget w, XtPointer client_data,
@@ -41,10 +36,8 @@ static void panner_callback PARAMS ((Widget w, XtPointer client_data,
 				    XtPointer call_data));
 static void porthole_callback PARAMS ((Widget w, XtPointer client_data,
 				      XtPointer call_data));
-
 static void panner_resize_handler PARAMS ((Widget w, XtPointer client_data,
 					  XEvent *event, Boolean *cont));
-
 static void MA_keypress PARAMS ((Widget w, XEvent *event, String *params,
 				Cardinal *num_params));
 static void MA_mapexpose PARAMS ((Widget w, XEvent *event, String *params,
@@ -77,14 +70,7 @@ static void MA_movelook_dr PARAMS ((Widget wdgt, XEvent *event, String *params,
 				   Cardinal *num_params));
 static void MA_center PARAMS ((Widget wdgt, XEvent *event, String *params,
 			      Cardinal *num_params));
-#if 0
-static void MA_message PARAMS ((Widget wdgt, XEvent *event, String *params,
-			       Cardinal *num_params));
-static void SetMessageArea PARAMS ((Widget msgarea, char *msg));
-#endif
-
 static void movelook_one_cell PARAMS ((Widget w, int xdelta, int ydelta));
-
 static void popup_ctrlpanel PARAMS ((Side *side, Map *map));
 static void popdown_ctrlpanel PARAMS ((Side *side, Map *map));
 static void create_ctrlpanel PARAMS ((Side *side, Map *map));
@@ -96,6 +82,7 @@ static void ctrlpanel_apply_callback PARAMS ((Widget w, XtPointer client_data,
 					     XtPointer call_data));
 static void ctrlpanel_done_callback PARAMS ((Widget w, XtPointer client_data,
 					    XtPointer call_data));
+static int set_from_controls PARAMS ((Side *side, Map *map));
 
 XtActionsRec map_actions_table[] = {
     { "keypress", MA_keypress },
@@ -168,11 +155,6 @@ Map *map;
 
 /* Create a generic map object. */
 
-
-#if 0
-XtTranslations saved_translations;
-#endif
-
 Map *
 create_map(side, power, geospec)
 Side *side;
@@ -243,13 +225,14 @@ char *geospec;
     pick_a_focus(side, &x, &y);
     set_view_focus(map->vp, x, y);
     /* Set default values for the display controls. */
-    map->drawterrain = TRUE;
-    map->drawgrid = TRUE;  /* should get from a resource */
-    map->drawcellpats = side->ui->monochrome;
-    map->drawunits = TRUE;
-    map->drawnames = FALSE /* (map->hh >= 8) */;
-    map->drawpeople = FALSE;
-    map->drawelevations = FALSE;
+    /* (default values should all come from resources) */
+    map->vp->draw_terrain = TRUE;
+    map->vp->draw_grid = TRUE;
+    map->vp->draw_units = TRUE;
+    map->vp->draw_names = FALSE /* (map->hh >= 8) */;
+    map->vp->draw_people = FALSE;
+    map->vp->draw_control = FALSE;
+    map->vp->draw_elevations = FALSE;
     map->drawfeatureboundaries = FALSE;
     map->drawfeaturenames = FALSE;
     /* cache the highest feature number; don't forget to update this 
@@ -257,10 +240,13 @@ char *geospec;
     side->ui->numfeatures = num_features();
     place_legends(side);
     for_all_material_types(m)
-      map->drawresources[m] = FALSE;
-    map->drawtemp = FALSE;
-    map->drawweather = FALSE;
+      map->vp->draw_materials[m] = FALSE;
+    map->vp->draw_temperature = FALSE;
+    map->vp->draw_clouds = FALSE;
+    map->vp->draw_winds = FALSE;
     map->seeall = all_see_all;
+    map->vp->draw_cover = FALSE;
+    map->vp->latlong_interval = side->ui->default_latlong_interval;
     /* Start the map off in generic move mode normally. */
     map->curtool = movetool;
     map->inpch = '\0';
@@ -292,25 +278,26 @@ char *geospec;
 			      XtNheight, map->totalh,
 			      XtNorientation, XtorientHorizontal,
 			      NULL);
-    
     map->leftpane =
       XtVaCreateManagedWidget("leftPane", panedWidgetClass, map->mainwidget,
 			      XtNwidth, map->leftw,
 			      XtNmin, 50,
+			      XtNorientation, XtorientVertical,
 			      NULL);
-
     map->infoform =
       XtVaCreateManagedWidget("infoForm", panedWidgetClass, map->leftpane,
-			      XtNmin, 150,
+			      XtNmax, map->totalh - 50,
+			      XtNmin, 50,
+			      XtNorientation, XtorientVertical,
+			      XtNpreferredPaneSize, map->toph,
 			      NULL);
-
     map->history =
       XtVaCreateManagedWidget("history", asciiTextWidgetClass, map->infoform,
+			      XtNpreferredPaneSize, 100,
 			      XtNscrollHorizontal, XawtextScrollWhenNeeded,
 			      XtNscrollVertical, XawtextScrollAlways,
 			      XtNshowGrip, False,
 			      NULL);
-
     /* It would be helpful sometimes to have a two-line input prompt window,
        but it chews valuable space.  Perhaps have two lines only if map
        is "large". */
@@ -320,7 +307,6 @@ char *geospec;
 			      XtNshowGrip, False,
 			      XtNskipAdjust, True,
 			      NULL);
-
     map->gamedate =
       XtVaCreateManagedWidget("gameDate", labelWidgetClass, map->infoform,
 			      XtNlabel, " ",
@@ -337,40 +323,26 @@ char *geospec;
 			      XtNshowGrip, False,
 			      XtNskipAdjust, True,
 			      NULL);
-#if 0 /* this needs a home, but don't let it chew up screen space all the time */
-    map->msgarea =
-      XtVaCreateManagedWidget("message", labelWidgetClass, map->infoform,
-			      /* Make it a two line label. */
-			      XtNlabel, "one\ntwo",
-			      XtNright, XtChainRight,
-			      XtNleft, XtChainLeft,
-			      XtNtop, XtChainBottom,
-			      XtNbottom, XtChainBottom,
-			      XtNfromVert, map->gameclock,
-			      NULL);
-#endif
-
     map->leftform =
       XtVaCreateManagedWidget("leftForm", panedWidgetClass, map->leftpane,
 			      XtNmin, 50,
 			      XtNorientation, XtorientHorizontal,
 			      NULL);
-
     map->controlform =
       XtVaCreateManagedWidget("controlForm", formWidgetClass, map->leftform,
+			      XtNdefaultDistance, 1,
+			      /* (should tie these to font size) */
 			      XtNmax, 150,
 			      XtNmin, 50,
 			      NULL);
     XawFormDoLayout(map->controlform, False);
     create_map_controls(side, map);
     XawFormDoLayout(map->controlform, True);
-
     map->mapform =
       XtVaCreateManagedWidget("mapForm", panedWidgetClass, map->leftform,
 			      XtNmin, 50,
 			      XtNorientation, XtorientVertical,
 			      NULL);
-
     map->info =
       XtVaCreateManagedWidget("info", widgetClass, map->mapform,
 			      XtNwidth, map->pxw,
@@ -384,7 +356,6 @@ char *geospec;
     		      KeyPressMask|ButtonPressMask|ButtonReleaseMask
 		      |ExposureMask|StructureNotifyMask,
     		      False, handle_map_info_events, NULL);
-
     map->porthole =
       XtVaCreateManagedWidget("porthole", portholeWidgetClass, map->mapform,
 			      XtNwidth, map->pxw,
@@ -401,16 +372,13 @@ char *geospec;
 			      XtNwidth, map->pxw,
 			      XtNheight, map->pxh,
 			      NULL);
-
     /* The right side of the window is dedicated to lists and to the
        map panner. */
-
     map->rightpane =
       XtVaCreateManagedWidget("rightPane", panedWidgetClass, map->mainwidget,
 			      /* Let this side be small but not too small. */
 			      XtNmin, 50,
 			      NULL);
-
     map->sides =
       XtVaCreateManagedWidget("sides", widgetClass, map->rightpane,
 			      XtNwidth, map->list1w,
@@ -424,7 +392,6 @@ char *geospec;
     		      KeyPressMask|ButtonPressMask|ButtonReleaseMask
 		      |ExposureMask|StructureNotifyMask,
     		      False, handle_map_sides_events, NULL);
-
     map->listview =
       XtVaCreateManagedWidget("listView", viewportWidgetClass, map->rightpane,
 			      XtNallowVert, True,
@@ -452,11 +419,6 @@ char *geospec;
 				      XtNleftBitmap, pic,
 				      XtNresize, False,
 				      NULL);
-#if 0
-	    XtVaGetValues(map->list_buttons[i],
-			  XtNtranslations, &saved_translations,
-			  NULL);
-#endif
 	    /* We never do anything with this "button". */
 	    XtUninstallTranslations(map->list_buttons[i]);
 	} else {
@@ -468,7 +430,7 @@ char *geospec;
 	       fgcolor & bgcolor. */
 	    draw_unit_image(side, pic, 0, 0,
 			    min_w_for_unit_image, min_h_for_unit_image,
-			    i - 1, -1, 1, 0);
+			    i - 1, -1, 1, 0, 0);
 
 	    /* Name the widget. */
 	    build_name(wbuff, "utype_", u_type_name(i - 1));
@@ -538,7 +500,7 @@ char *geospec;
     map->viewwin = XtWindow(map->portlabel);
     map->sideswin = XtWindow(map->sides);
 
-    /* initialize panner pixmap */
+    /* Initialize panner pixmap. */
     map->panner_pix = None;
 
     draw_map(side, map);
@@ -627,40 +589,22 @@ Map *map;
     XtAddCallback(map->controls[UNIT_BUILD], XtNcallback,
 		  command_callback, (XtPointer) UNIT_BUILD);
 
-    map->controls[SHOW_TERRAIN] =
-      XtVaCreateManagedWidget("terrain", toggleWidgetClass, map->controlform,
-			      XtNlabel, "Terrain",
-			      XtNresize, False,
-			      XtNwidth, wid,
-			      XtNfromVert, map->controls[UNIT_BUILD],
-			      XtNvertDistance, 10,
-			      NULL);
-    XtAddCallback(map->controls[SHOW_TERRAIN], XtNcallback,
-		  view_flag_callback, (XtPointer) SHOW_TERRAIN);
     map->controls[SHOW_GRID] =
       XtVaCreateManagedWidget("grid", toggleWidgetClass, map->controlform,
 			      XtNlabel, "Grid",
 			      XtNresize, False,
 			      XtNwidth, wid,
-			      XtNfromVert, map->controls[SHOW_TERRAIN],
+			      XtNfromVert, map->controls[UNIT_BUILD],
+			      XtNvertDistance, 10,
 			      NULL);
     XtAddCallback(map->controls[SHOW_GRID], XtNcallback,
 		  view_flag_callback, (XtPointer) SHOW_GRID);
-    map->controls[SHOW_UNITS] =
-      XtVaCreateManagedWidget("units", toggleWidgetClass, map->controlform,
-			      XtNlabel, "Units",
-			      XtNresize, False,
-			      XtNwidth, wid,
-			      XtNfromVert, map->controls[SHOW_GRID],
-			      NULL);
-    XtAddCallback(map->controls[SHOW_UNITS], XtNcallback,
-		  view_flag_callback, (XtPointer) SHOW_UNITS);
     map->controls[SHOW_NAMES] =
       XtVaCreateManagedWidget("names", toggleWidgetClass, map->controlform,
 			      XtNlabel, "Names",
 			      XtNresize, False,
 			      XtNwidth, wid,
-			      XtNfromVert, map->controls[SHOW_UNITS],
+			      XtNfromVert, map->controls[SHOW_GRID],
 			      NULL);
     XtAddCallback(map->controls[SHOW_NAMES], XtNcallback,
 		  view_flag_callback, (XtPointer) SHOW_NAMES);
@@ -721,6 +665,7 @@ Map *map;
 			      XtNresize, False,
 			      XtNwidth, wid,
 			      XtNfromVert, map->controls[SHOW_MORE],
+			      XtNvertDistance, 10,
 			      NULL);
     XtAddCallback(map->controls[POPUP_HELP], XtNcallback,
 		  map_help_callback, NULL);
@@ -1253,10 +1198,7 @@ XEvent *evt;
 	/* Collect only the first char of a possible sequence. */
 	ch = buf[0];
 	DGprintf("%s typed char '%c'\n", side_desig(side), ch);
-	/* so we can do ^L even while typing a string */
-	if (ch == REDRAW_CHAR) {
-	    redraw(side);
-	} else if (map != NULL) {
+	if (map != NULL) {
 	    /* Save the char, we'll need it. */
 	    map->inpch = ch;
 	    /* Call the modal handler if defined, giving it side
@@ -1272,6 +1214,10 @@ XEvent *evt;
 		    clear_prompt(side, map);
 		    notify(side, "Cancelled.");
 		}
+		/* Reset the prefix arg unless there is still more modal
+		 * input to be read. */
+		if (map->modalhandler == NULL)
+		    map->prefixarg = -1;
 	    } else {
 		interp_key_command(side, map);
 	    }
@@ -1310,8 +1256,10 @@ Map *map;
 	clear_prompt(side, map);
 	map->frombutton = FALSE;
 	execute_command(side, map);
-	/* Reset the argument for next time. */
-	map->prefixarg = -1;
+	/* Reset the argument for next command, unless the current one is still
+	 * waiting for further input. */
+	if (map->modalhandler == NULL)
+	    map->prefixarg = -1;
     }
 }
 
@@ -1467,17 +1415,20 @@ Map *map;
     char helpbuf[BUFSIZE];
 
     for (i = 0; map->ustr[i] != '\0'; ++i) {
+	/* Put out several types on each line. */
 	if (i % 4 == 0) {
 	    if (i > 0) {
-		tprintf(helpbuf, "? for this help info"); 
 		notify(side, "%s", helpbuf);
 	    }
-	    helpbuf[0] = '\0';
+	    /* Indent each line a bit (also avoids confusion due to
+	       notify's capitalization). */
+	    strcpy(helpbuf, "  ");
 	}
-	tprintf(helpbuf, "%c %s", map->ustr[i], u_type_name(map->uvec[i]));
-	if (i > 0)
-	  tprintf(helpbuf, ", ");
+	tprintf(helpbuf, "%c %s, ", map->ustr[i], u_type_name(map->uvec[i]));
     }
+    /* Add an extra helpful comment, then dump any leftovers. */
+    tprintf(helpbuf, "? for this help info"); 
+    notify(side, "%s", helpbuf);
 }
 
 void
@@ -1490,9 +1441,6 @@ int u, flag;
       case -1:
 	XtVaSetValues(map->list_buttons[u + 1],
 		      XtNsensitive, False,
-#if 0
-		      XtNtranslations, saved_translations,
-#endif
 		      NULL);
 	break;
       case 0:
@@ -1500,18 +1448,11 @@ int u, flag;
 	XtVaSetValues(map->list_buttons[u + 1],
 		      XtNsensitive, True,
 		      NULL);
-#if 0
-	/* (need to force button state out somehow?) */
-	XtUninstallTranslations(map->list_buttons[u + 1]);
-#endif
 	break;
       case 1:
 	XtVaSetValues(map->list_buttons[u + 1],
 		      XtNhighlightThickness, 2,
 		      XtNsensitive, True,
-#if 0
-		      XtNtranslations, saved_translations,
-#endif
 		      NULL);
 	break;
     }
@@ -1580,21 +1521,24 @@ Map *map;
     char helpbuf[BUFSIZE];
 
     for (i = 0; map->tstr[i] != '\0'; ++i) {
+	/* Put out several types on each line. */
 	if (i % 4 == 0) {
 	    if (i > 0) {
-		tprintf(helpbuf, "? for this help info"); 
 		notify(side, "%s", helpbuf);
 	    }
-	    helpbuf[0] = '\0';
+	    /* Indent each line a bit (also avoids confusion due to
+	       notify's capitalization). */
+	    strcpy(helpbuf, "  ");
 	}
-	tprintf(helpbuf, "%c %s", map->tstr[i], t_type_name(map->tvec[i]));
-	if (i > 0)
-	  tprintf(helpbuf, ", ");
+	tprintf(helpbuf, "%c %s, ", map->tstr[i], t_type_name(map->tvec[i]));
     }
+    /* Add an extra helpful comment, then dump any leftovers. */
+    tprintf(helpbuf, "? for this help info"); 
+    notify(side, "%s", helpbuf);
 }
 
-/* User is asked to pick a position on map.  This will iterate until the */
-/* space bar designates the final position. */
+/* User is asked to pick a position on map.  This will iterate until the
+   space bar designates the final position. */
 
 /* (should change the cursor temporarily) */
 
@@ -1612,19 +1556,24 @@ void (*handler) PARAMS ((Side *side, Map *map, int cancelled));
 }
 
 int
-grok_position(side, map, xp, yp)
+grok_position(side, map, xp, yp, unitp)
 Side *side;
 Map *map;
 int *xp, *yp;
+Unit **unitp;
 {
     
     if (in_area(map->curx, map->cury)) {
 	*xp = map->curx;  *yp = map->cury;
+	if (unitp != NULL)
+	  *unitp = map->curunit;
 	clear_prompt(side, map);
 	return TRUE;
     } else {
 	/* Make any possible usage attempts fail. */
 	*xp = *yp = -1;
+	if (unitp != NULL)
+	  *unitp = NULL;
 	return FALSE;
     }
 }
@@ -1958,19 +1907,19 @@ XtPointer call_data;
 
     switch (which) {
       case SHOW_TERRAIN:
-	map->drawterrain = !map->drawterrain;
+	map->vp->draw_terrain = !map->vp->draw_terrain;
 	break;
       case SHOW_GRID:
-	map->drawgrid = !map->drawgrid;
+	map->vp->draw_grid = !map->vp->draw_grid;
 	break;
       case SHOW_UNITS:
-	map->drawunits = !map->drawunits;
+	map->vp->draw_units = !map->vp->draw_units;
 	break;
       case SHOW_NAMES:
-	map->drawnames = !map->drawnames;
+	map->vp->draw_names = !map->vp->draw_names;
 	break;
       case SHOW_PEOPLE:
-	map->drawpeople = !map->drawpeople;
+	map->vp->draw_people = !map->vp->draw_people;
 	break;
       case SHOW_ALL:
 	if (!all_see_all)
@@ -1979,7 +1928,7 @@ XtPointer call_data;
 	  beep(side);
 	break;
       case SHOW_ELEV:
-	map->drawelevations = !map->drawelevations;
+	map->vp->draw_elevations = !map->vp->draw_elevations;
 	break;
       case SHOW_MORE:
 	if (!map->fullpanel)
@@ -2173,6 +2122,7 @@ int sx, sy;
 {
     int ax, ay;
     Unit *unit2;
+    void (*fn) PARAMS ((Side *sidex, Map *mapx, int cancelledx));
 
     if (map == NULL) {
 	beep(side);
@@ -2193,8 +2143,6 @@ int sx, sy;
     }
 #endif /* DESIGNERS */
     if (map->modalhandler) {
-       void (*fn) PARAMS ((Side *sidex, Map *mapx, int cancelledx));
-
        fn = map->modalhandler;
        map->modalhandler = NULL;
        move_look(side, map, sx, sy);
@@ -2219,14 +2167,14 @@ int sx, sy;
 	    if (inside_area(ax, ay)) {
 		if ((unit2 = unit_at(ax, ay)) != NULL) {
 		    if (map->curunit != unit2) {
-			prep_fire_at_action(map->curunit, map->curunit,
+			net_prep_fire_at_action(map->curunit, map->curunit,
 					    unit2, -1);
 		    } else {
 			/* don't attack ourselves */
 		    }
 		} else {
 		    /* We're just shooting for the hell of it. */
-		    prep_fire_into_action(map->curunit, map->curunit,
+		    net_prep_fire_into_action(map->curunit, map->curunit,
 					  ax, ay, 0, 0);
 		}
 	    } else {
@@ -2263,7 +2211,7 @@ int sx, sy;
     /* Designers use this function to push units around, bound only by the
        limits on occupancy. */
     if (side->designer) {
-	designer_teleport(unit, x, y, other);
+	net_designer_teleport(unit, x, y, other);
 	return;
     }
 #endif
@@ -2276,7 +2224,7 @@ int sx, sy;
 	    if (distance(unit->x, unit->y, x, y) > 1) {
 		DGprintf("Ordering %s to move to %d,%d\n",
 			 unit_desig(unit), x, y);
-		set_move_to_task(unit, x, y);
+		net_set_move_to_task(unit, x, y);
 		return;
 	    }
 
@@ -2284,7 +2232,7 @@ int sx, sy;
 	    if (unit_at(x, y) == NULL) {
 		if (can_occupy_cell(unit, x, y)
 		    && valid(check_move_action(unit, unit, x, y, 0))) {
-		    prep_move_action(unit, unit, x, y, 0);
+		    net_prep_move_action(unit, unit, x, y, 0);
 		} else {
 		    beep(side);
 		}
@@ -2294,31 +2242,31 @@ int sx, sy;
 	    /* There are units at our desired destination. */
 	    if (other == NULL) {
 		if (can_occupy_cell(unit, x, y)) {
-		    prep_move_action(unit, unit, x, y, 0);
+		    net_prep_move_action(unit, unit, x, y, 0);
 		} else {
 		    beep(side);
 		}
 	    } else if (other->side == unit->side) {
 		/* One of ours, maybe get on it. */
 		if (can_occupy(unit, other)) {
-		    prep_enter_action(unit, unit, other);
+		    net_prep_enter_action(unit, unit, other);
 		} else if (can_occupy(other, unit)) {
 		    /* Have other unit do an enter action, then move. */
 		    /* (not quite right, move should happen after other unit
 		       is actually inside, in case it fills dest) */
-		    prep_enter_action(other, other, unit);
-		    set_move_to_task(unit, x, y);
+		    net_prep_enter_action(other, other, unit);
+		    net_set_move_to_task(unit, x, y);
 		} else if (can_occupy_cell(unit, x, y)) {
-		    prep_move_action(unit, unit, x, y, 0);
+		    net_prep_move_action(unit, unit, x, y, 0);
 		} else {
 		    beep(side);
 		}
 	    } else {
 		/* Somebody else's unit, try to victimize it. */
 		if (valid(check_capture_action(unit, unit, other))) {
-		    prep_capture_action(unit, unit, other);
+		    net_prep_capture_action(unit, unit, other);
 		} else if (valid(check_attack_action(unit, unit, other, 100))) {
-		    prep_attack_action(unit, unit, other, 100);
+		    net_prep_attack_action(unit, unit, other, 100);
 		} else {
 		    beep(side);
 		}
@@ -2329,7 +2277,7 @@ int sx, sy;
 	if (other != NULL && other != unit) {
 	    /* ok we're trying to hop onto another transport */
 	    if (can_occupy(unit, other)) {
-		prep_enter_action(unit, unit, other);
+		net_prep_enter_action(unit, unit, other);
 	    } else {
 		/* maybe we should restack? */
 		beep(side);
@@ -2378,7 +2326,9 @@ Boolean *cont;
     }
 }
  
-/* draw the world view in the background of the panner */
+/* Draw the world view in the background of the panner. */
+
+#define USEIMAGE 1
 
 void
 draw_view_in_panner(side, map)
@@ -2387,23 +2337,21 @@ Map *map;
 {
     int x, y, l, t, mx, my, depth, w, h, b;
     Display *dpy = side->ui->dpy;
-    Pixel pixel, bg=0, unknown;
+    Pixel pixel, bg = 0, unknown, known;
     XImage *img;
     char *dp, *data;
     GC gc;
-    extern int numdisplays;
 
     if (map->panner == NULL)
       return;
 
     unknown = XBlackPixel(dpy, side->ui->screen);
+    known = XWhitePixel(dpy, side->ui->screen);
     depth = DefaultDepth(dpy, side->ui->screen);
-    if (depth % 8)
-      return;
 
-   /* we need w, h, b as int, otherwise some compiler will miscompile
-   mx = ((2 * (x - b) * area.width) / (w - 2 * b) - my) / 2;
-   while XtVaGetValues needs them as Dimension */
+    /* we need w, h, b as int, otherwise some compiler will miscompile
+       mx = ((2 * (x - b) * area.width) / (w - 2 * b) - my) / 2;
+       while XtVaGetValues needs them as Dimension */
     {
 	Dimension wd = 0, hd = 0, bd = 0;
 
@@ -2422,6 +2370,10 @@ Map *map;
     if (b <= 0 || h <= 2 * b || w <= 2 * b)
       return;
 
+#if USEIMAGE
+    if (depth < 8)
+      return;
+
     /* Not using xmalloc because data will be freed within this function */
     data = (char *) malloc(w * h * depth / 8 * sizeof(char));
 
@@ -2436,7 +2388,7 @@ Map *map;
 	    if (x < b || x >= w - b) {
 		mx = -1;
 	    } else {
-	        /* get the right rounding for the division by 2 */
+		/* get the right rounding for the division by 2 */
 		mx = ((2 * (x - b) * area.width) / (w - 2 * b) - my) / 2;
 		if (area.xwrap) {
 		    mx = wrapx(mx);
@@ -2450,6 +2402,8 @@ Map *map;
 		    pixel = unknown;
 		} else {
 		    pixel = side->ui->cellcolor[t];
+		    if (pixel < 0)
+		      pixel = side->ui->fgcolor;
 		}
 	    } else {
 		pixel = bg;
@@ -2469,6 +2423,7 @@ Map *map;
     }
     img->byte_order = MSBFirst;
     img->bitmap_bit_order = MSBFirst;
+#endif
 
     /* clear the panner background before freeing the pixmap */    
 #ifdef NO_PANNER_BGPIXMAP
@@ -2481,16 +2436,55 @@ Map *map;
       XFreePixmap(dpy, map->panner_pix);
     map->panner_pix = XCreatePixmap(dpy, side->ui->rootwin, w, h, depth);
     if (!map->panner_pix || map->panner_pix == None) {
+#if USEIMAGE
 	XDestroyImage(img);
 	/* XDestroyImage also frees data */
+#endif
 	return;
     }
 
     gc = XCreateGC(dpy, map->panner_pix, 0, NULL);
+#if USEIMAGE
     XPutImage(dpy, map->panner_pix, gc, img, 0, 0, 0, 0, w, h);
-    XFreeGC(dpy, gc);
     XDestroyImage(img);
     /* XDestroyImage also frees data */
+#else
+    for (y = 0; y < h; y++) {
+	if (y < b || y >= h - b) {
+	    my = -1;
+	} else {
+	    my = ((h - 1 - b - y) * area.height) / (h - 2 * b);
+	}
+	for (x = 0; x < w; x++) {
+	    if (x < b || x >= w - b) {
+		mx = -1;
+	    } else {
+		/* get the right rounding for the division by 2 */
+		mx = ((2 * (x - b) * area.width) / (w - 2 * b) - my) / 2;
+		if (area.xwrap) {
+		    mx = wrapx(mx);
+		} else {
+		    mx += area.height / 4;
+		}
+	    }
+	    if (mx >= 0 && my >= 0 && in_area(mx, my)) {
+		t = terrain_seen_at(side, mx, my);
+		if (t == NONTTYPE) {
+		    pixel = unknown;
+		} else {
+		    pixel = side->ui->cellcolor[t];
+		    if (pixel < 0)
+		      pixel = side->ui->fgcolor;
+		}
+	    } else {
+		pixel = bg;
+	    }
+	    XSetForeground(dpy, gc, pixel);
+	    XDrawPoint(dpy, map->panner_pix, gc, x, y);
+	}
+    }
+#endif
+    XFreeGC(dpy, gc);
 
 #ifdef NO_PANNER_BGPIXMAP
     XSetWindowBackgroundPixmap(dpy, XtWindow(map->panner), map->panner_pix);
@@ -2549,19 +2543,15 @@ Map *map;
     map->ctrlpanel_shell =
       XtVaCreatePopupShell("controlPanel", topLevelShellWidgetClass, side->ui->shell,
 			   NULL);
-
     map->ctrlpanel_form =
       XtVaCreateManagedWidget("form", boxWidgetClass, map->ctrlpanel_shell,
 			      NULL);
-
     buttons = (Widget *) xmalloc(numcontrols * sizeof (Widget));
     map->ctrlpanel_buttons = buttons;
-
     label =
       XtVaCreateManagedWidget("---what---", labelWidgetClass, map->ctrlpanel_form,
 			      XtNborderWidth, 0,
 			      NULL);
-
     map->ctrlpanel_buttons[SHOW_TERRAIN] =
       XtVaCreateManagedWidget("Terrain", toggleWidgetClass, map->ctrlpanel_form,
 			      NULL);
@@ -2579,26 +2569,40 @@ Map *map;
 			       XtNlabel, "Feature Names", NULL);
     map->ctrlpanel_buttons[SHOW_FEATURE_BOUNDARIES] =
       XtVaCreateManagedWidget("featureBoundaries",  toggleWidgetClass, map->ctrlpanel_form,
-			       XtNlabel, "Feature Bound.", NULL);
+			       XtNlabel, "Feature Edges", NULL);
     map->ctrlpanel_buttons[SHOW_PEOPLE] =
       XtVaCreateManagedWidget("People", toggleWidgetClass, map->ctrlpanel_form,
 			      NULL);
+    map->ctrlpanel_buttons[SHOW_CONTROL] =
+      XtVaCreateManagedWidget("Control", toggleWidgetClass, map->ctrlpanel_form,
+			      NULL);
     map->ctrlpanel_buttons[SHOW_ELEV] =
       XtVaCreateManagedWidget("Elevations", toggleWidgetClass, map->ctrlpanel_form,
 			      NULL);
     map->ctrlpanel_buttons[SHOW_TEMP] =
       XtVaCreateManagedWidget("Temperatures", toggleWidgetClass, map->ctrlpanel_form,
 			      NULL);
+    map->ctrlpanel_buttons[SHOW_CLOUDS] =
+      XtVaCreateManagedWidget("Clouds", toggleWidgetClass, map->ctrlpanel_form,
+			      NULL);
+    map->ctrlpanel_buttons[SHOW_WINDS] =
+      XtVaCreateManagedWidget("Winds", toggleWidgetClass, map->ctrlpanel_form,
+			      NULL);
+    map->ctrlpanel_buttons[SHOW_COVER] =
+      XtVaCreateManagedWidget("Coverage", toggleWidgetClass, map->ctrlpanel_form,
+			      NULL);
     map->ctrlpanel_buttons[SHOW_ALL] =
-      XtVaCreateManagedWidget("See All", toggleWidgetClass, map->ctrlpanel_form,
+      XtVaCreateManagedWidget("seeAll", toggleWidgetClass, map->ctrlpanel_form,
 			      XtNsensitive, !all_see_all,
+			      XtNlabel, "See All",
+			      NULL);
+    map->ctrlpanel_buttons[SHOW_LATLONG] =
+      XtVaCreateManagedWidget("Meridians", toggleWidgetClass, map->ctrlpanel_form,
 			      NULL);
-
     label =
       XtVaCreateManagedWidget("--color---", labelWidgetClass, map->ctrlpanel_form,
 			      XtNborderWidth, 0,
 			      NULL);
-
     map->ctrlpanel_buttons[COLR_TERR] =
      XtVaCreateManagedWidget("Terrain", toggleWidgetClass, map->ctrlpanel_form,
 			      XtNsensitive, !side->ui->monochrome,
@@ -2619,7 +2623,6 @@ Map *map;
       XtVaCreateManagedWidget("Reverse", toggleWidgetClass, map->ctrlpanel_form,
 			      XtNsensitive, side->ui->monochrome,
 			      NULL);
-
     label =
       XtVaCreateManagedWidget("----------", labelWidgetClass, map->ctrlpanel_form,
 			      XtNborderWidth, 0,
@@ -2684,8 +2687,6 @@ Widget w;
     return rslt; 
 }
 
-static int set_from_controls PARAMS ((Side *side, Map *map));
-
 static int
 set_from_controls(side, map)
 Side *side;
@@ -2694,21 +2695,21 @@ Map *map;
     int newval, anychanged = FALSE;
 
     newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_TERRAIN]);
-    if (newval != map->drawterrain)
+    if (newval != map->vp->draw_terrain)
       anychanged = TRUE;
-    map->drawterrain = newval;
+    map->vp->draw_terrain = newval;
     newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_GRID]);
-    if (newval != map->drawgrid)
+    if (newval != map->vp->draw_grid)
       anychanged = TRUE;
-    map->drawgrid = newval;
+    map->vp->draw_grid = newval;
     newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_UNITS]);
-    if (newval != map->drawunits)
+    if (newval != map->vp->draw_units)
       anychanged = TRUE;
-    map->drawunits = newval;
+    map->vp->draw_units = newval;
     newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_NAMES]);
-    if (newval != map->drawnames)
+    if (newval != map->vp->draw_names)
       anychanged = TRUE;
-    map->drawnames = newval;
+    map->vp->draw_names = newval;
     newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_FEATURE_NAMES]);
     if (newval != map->drawfeaturenames)
       anychanged = TRUE;
@@ -2718,21 +2719,41 @@ Map *map;
       anychanged = TRUE;
     map->drawfeatureboundaries = newval;
     newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_PEOPLE]);
-    if (newval != map->drawpeople)
+    if (newval != map->vp->draw_people)
       anychanged = TRUE;
-    map->drawpeople = newval;
+    map->vp->draw_people = newval;
+    newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_CONTROL]);
+    if (newval != map->vp->draw_control)
+      anychanged = TRUE;
+    map->vp->draw_control = newval;
     newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_ELEV]);
-    if (newval != map->drawelevations)
+    if (newval != map->vp->draw_elevations)
       anychanged = TRUE;
-    map->drawelevations = newval;
+    map->vp->draw_elevations = newval;
     newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_TEMP]);
-    if (newval != map->drawtemp)
+    if (newval != map->vp->draw_temperature)
       anychanged = TRUE;
-    map->drawtemp = newval;
+    map->vp->draw_temperature = newval;
+    newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_CLOUDS]);
+    if (newval != map->vp->draw_clouds)
+      anychanged = TRUE;
+    map->vp->draw_clouds = newval;
+    newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_WINDS]);
+    if (newval != map->vp->draw_winds)
+      anychanged = TRUE;
+    map->vp->draw_winds = newval;
+    newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_COVER]);
+    if (newval != map->vp->draw_cover)
+      anychanged = TRUE;
+    map->vp->draw_cover = newval;
     newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_ALL]);
     if (newval != map->seeall)
       anychanged = TRUE;
     map->seeall = newval;
+    newval = get_toggle_state(map->ctrlpanel_buttons[SHOW_LATLONG]);
+    if (newval != map->vp->draw_latlong)
+      anychanged = TRUE;
+    map->vp->draw_latlong = newval;
     /* (shouldn't be affecting all maps without redrawing all...) */
     newval = get_toggle_state(map->ctrlpanel_buttons[COLR_TERR]);
     if (newval != side->ui->dflt_color_terr_images)
@@ -2806,19 +2827,42 @@ Map *map;
     set_control_state(side, map, LOOK, TRUE, (map->curtool == looktool));
     set_control_state(side, map, MOVE, TRUE, (map->curtool == movetool));
     update_unit_controls(side, map);
-    set_control_state(side, map, SHOW_TERRAIN, TRUE, map->drawterrain);
-    set_control_state(side, map, SHOW_GRID, TRUE, map->drawgrid);
-    set_control_state(side, map, SHOW_UNITS, TRUE, map->drawunits);
-    set_control_state(side, map, SHOW_NAMES, TRUE, map->drawnames);
-    set_control_state(side, map, SHOW_FEATURE_NAMES, TRUE, map->drawfeaturenames);
-    set_control_state(side, map, SHOW_FEATURE_BOUNDARIES, TRUE, map->drawfeatureboundaries);
-    set_control_state(side, map, SHOW_PEOPLE, people_sides_defined(), map->drawpeople);
-    set_control_state(side, map, SHOW_ELEV, elevations_defined(), map->drawelevations);
-    set_control_state(side, map, SHOW_TEMP, temperatures_defined(), map->drawtemp);
-    set_control_state(side, map, SHOW_ALL, side->may_set_see_all, map->seeall);
-    set_control_state(side, map, SHOW_MORE, TRUE, map->fullpanel);
-    set_control_state(side, map, ZOOM_OUT, (map->vp->power > 0), FALSE);
-    set_control_state(side, map, ZOOM_IN,  (map->vp->power < NUMPOWERS - 1), FALSE);
+    set_control_state(side, map, SHOW_TERRAIN,
+		      TRUE, map->vp->draw_terrain);
+    set_control_state(side, map, SHOW_GRID,
+		      TRUE, map->vp->draw_grid);
+    set_control_state(side, map, SHOW_UNITS,
+		      TRUE, map->vp->draw_units);
+    set_control_state(side, map, SHOW_NAMES,
+		      TRUE, map->vp->draw_names);
+    set_control_state(side, map, SHOW_FEATURE_NAMES,
+		      TRUE, map->drawfeaturenames);
+    set_control_state(side, map, SHOW_FEATURE_BOUNDARIES,
+		      TRUE, map->drawfeatureboundaries);
+    set_control_state(side, map, SHOW_PEOPLE,
+		      people_sides_defined(), map->vp->draw_people);
+    set_control_state(side, map, SHOW_CONTROL,
+		      control_sides_defined(), map->vp->draw_control);
+    set_control_state(side, map, SHOW_ELEV,
+		      elevations_defined(), map->vp->draw_elevations);
+    set_control_state(side, map, SHOW_TEMP,
+		      temperatures_defined(), map->vp->draw_temperature);
+    set_control_state(side, map, SHOW_CLOUDS,
+		      clouds_defined(), map->vp->draw_clouds);
+    set_control_state(side, map, SHOW_WINDS,
+		      winds_defined(), map->vp->draw_winds);
+    set_control_state(side, map, SHOW_COVER,
+		      !map->seeall, map->vp->draw_cover);
+    set_control_state(side, map, SHOW_ALL,
+		      side->may_set_see_all, map->seeall);
+    set_control_state(side, map, SHOW_LATLONG,
+		      world.circumference > 0, map->vp->draw_latlong);
+    set_control_state(side, map, SHOW_MORE,
+		      TRUE, map->fullpanel);
+    set_control_state(side, map, ZOOM_OUT,
+		      (map->vp->power > 0), FALSE);
+    set_control_state(side, map, ZOOM_IN,
+		      (map->vp->power < NUMPOWERS - 1), FALSE);
     set_control_state(side, map, COLR_TERR, !side->ui->monochrome,
 		      side->ui->dflt_color_terr_images);
     set_control_state(side, map, COLR_UNITS, !side->ui->monochrome,
@@ -2909,20 +2953,12 @@ draw_game_state(side, map)
 Side *side;
 Map *map;
 {
-    char tmpbuf[BUFSIZE];
-    extern char *curseasonname;
-    extern int paused;
-
     if (map == NULL || map->gamedate == None)
       return;
-    sprintf(tmpbuf, "%s", absolute_date_string(g_turn()));
-    if (curseasonname != NULL)
-      tprintf(tmpbuf, " (%s)", curseasonname);
-    if (paused)
-      tprintf(tmpbuf, " PAUSED");
-    XtVaSetValues(map->gamedate, XtNlabel, tmpbuf, NULL);
+    XtVaSetValues(map->gamedate, XtNlabel, curdatestr, NULL);
     draw_game_clocks(side, map);
-    /* update the Info window; Massimo: */ draw_map_info(side, map);
+    /* update the Info window; Massimo: */
+    draw_map_info(side, map);
 }
 
 /* This displays the gamewide clock(s) if defined. */
