@@ -74,7 +74,10 @@ typedef struct a_family_stuff {
 
 FamilyStuff family[MAXFAMS];
 
-int basew=32, baseh=32;
+int numremotes;
+int my_rid;
+
+int basew = 32, baseh = 32;
 
 int cols, rows, color_comb;
 
@@ -188,22 +191,22 @@ static char Translations[] =
    <LeaveWindow>:  ShowFamily() \n\
    <BtnDown>:      ImageAction() ";
 
-void do_help();
-void done_help();
-void do_select();
-void do_deselect();
-void do_toggle();
-void do_read();
-void do_save();
-void do_delete();
-void do_quit();
-void do_open_family();
-void do_prep_delete();
-void do_prep_update();
-void do_prep_export();
-void do_close_family();
-void do_increase_magn();
-void do_decrease_magn();
+void do_help PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void done_help PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_select PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_deselect PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_toggle PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_read PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_save PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_delete PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_quit PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_open_family PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_prep_delete PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_prep_update PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_prep_export PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_close_family PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_increase_magn PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
+void do_decrease_magn PARAMS ((Widget w, XtPointer cldata, XtPointer cadata));
 
 void show_image_families PARAMS ((int first_time));
 void hide_image_families PARAMS ((void));
@@ -213,6 +216,7 @@ void build_name PARAMS ((char *name, char *first, char *second));
 void display_family PARAMS ((int i, int imagn));
 void undisplay_family PARAMS ((int i));
 ImageStuff *init_ims PARAMS ((Image *img, Widget parent));
+ImageStuff *find_ims PARAMS ((FamilyStuff *fms, Widget w));
 int shell_index PARAMS ((Widget w));
 Pixmap magnify_bitmap PARAMS ((char *data, int h, int w, int s));
 Pixmap magnify_colrpix PARAMS ((Image *img, int s, Pixel bg_pix, Pixmap mask));
@@ -513,6 +517,7 @@ int first_time;
 	XtSetArg(tmpargs[a], XtNlabel, "");  a++;
 	x11_interp_imf(dpy, rootwin, images[i], TRUE);
 	img = best_image(images[i], basew, baseh);
+	if (!img)  continue;
 	if (img->hook != NULL) {
 	    ximg = (X11Image *) img->hook;
 	    if (ximg->colr != None) {
@@ -585,7 +590,7 @@ int imagn;
     Widget last, up;
     Image *img;
     FamilyStuff *fms = &family[i];
-    ImageStuff *ims;
+    ImageStuff *ims = NULL;
 
     fms->imagn = imagn;
 
@@ -638,6 +643,7 @@ int imagn;
 			      NULL);
 
     last = fms->label;
+    up = None;
     img0 = 1;
     for (img = images[i]->images; img != NULL; img = img->next) {
 	if (last)
@@ -905,8 +911,10 @@ char *name, *first, *second;
 }
 
 void
-do_help(w)
+do_help(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     Widget w_form, w_done, w_text;
     
@@ -932,15 +940,19 @@ Widget w;
 }
 
 void
-done_help(w)
+done_help(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     XtPopdown(help_shell);
 }
 
 void
-do_select(w)
+do_select(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     int i;
 
@@ -950,8 +962,10 @@ Widget w;
 }
 
 void
-do_deselect(w)
+do_deselect(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     int i;
 
@@ -961,8 +975,10 @@ Widget w;
 }
 
 void
-do_toggle(w)
+do_toggle(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     int i;
     Boolean state;
@@ -974,8 +990,10 @@ Widget w;
 }
 
 void
-do_read(w)
+do_read(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     FILE *stream = NULL;
     char *filename;
@@ -1027,27 +1045,31 @@ int dummy;
 }
 
 void
-do_save(w)
+do_save(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     int i, n;
     Boolean state;
-    FILE *stream=NULL;
-    char *filename;
+    FILE *fp = NULL;
+    char *filename = NULL; /* init so GCC doesn't complain */
     
     n = 0;
     for (i = 0; i < numfamilies; i++) {
 	XtVaGetValues(family[i].name, XtNstate, &state, NULL);
 	if (state) {
-	    if (!stream) {
+	    if (fp == NULL) {
 #ifdef HAVE_SELFILE
-		stream = XsraSelFile(toplevel,
+		fp = XsraSelFile(toplevel,
 				     "Save in file:              ",
 				     "Okay", "Cancel",
 				     "Error: can't open file   ",
 				     write_suggest, "w", NULL, &filename);
+#else
+		filename = NULL;
 #endif
-		if (stream) {
+		if (fp) {
 		    write_suggest = filename;
 		} else {
 		    return;
@@ -1056,31 +1078,35 @@ Widget w;
 	    /* write_imf expects rawdata in "natural" order, not in the 
 	       X11 "byte-reversed" order the rest of this program uses */
 	    reverse_rawdata(images[i]);
-	    write_imf(stream, images[i]);
+	    write_imf(fp, images[i]);
 	    reverse_rawdata(images[i]);
 	    n++;
 	}
     }
-    if (stream) {
-	fclose(stream);
+    if (fp != NULL) {
+	fclose(fp);
 	if (n) {
 	    sprintf(buffer, "%d image families saved in \"%s\"",
-		    numfamilies, filename);
+		    numfamilies, (filename ? filename : "???"));
 	    XtVaSetValues(message, XtNlabel, buffer, NULL);
 	}
     }
 }
 
 void
-do_delete(w)
+do_delete(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     delete_family(-1);
 }
 
 void
-do_quit(w)
+do_quit(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     exit(0);
 }
@@ -1088,8 +1114,10 @@ Widget w;
 /* callback: popup the family corresponding to the widget 
 	     do_open_family was called from */
 void
-do_open_family(w)
+do_open_family(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     int i;
 
@@ -1110,8 +1138,10 @@ Widget w;
 }
 
 void
-do_prep_delete(w)
+do_prep_delete(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     int i = shell_index(XtParent(XtParent(w)));
     FamilyStuff *fms = &family[i];
@@ -1125,8 +1155,10 @@ Widget w;
 }
 
 void
-do_prep_update(w)
+do_prep_update(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     int i = shell_index(XtParent(XtParent(w)));
     FamilyStuff *fms = &family[i];
@@ -1139,8 +1171,10 @@ Widget w;
 }
 
 void
-do_prep_export(w)
+do_prep_export(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     int i = shell_index(XtParent(XtParent(w)));
     FamilyStuff *fms = &family[i];
@@ -1189,8 +1223,10 @@ Cursor cursor;
 }
 
 void
-do_close_family(w)
+do_close_family(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     int i = shell_index(XtParent(XtParent(w)));
 
@@ -1200,8 +1236,10 @@ Widget w;
 }
 
 void 
-do_increase_magn(w)
+do_increase_magn(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     int i = shell_index(XtParent(XtParent(w)));
     int imag;
@@ -1217,8 +1255,10 @@ Widget w;
 }
 
 void 
-do_decrease_magn(w)
+do_decrease_magn(w, client_data, call_data)
 Widget w;
+XtPointer client_data;
+XtPointer call_data;
 {
     int i = shell_index(XtParent(XtParent(w)));
     int imag;
@@ -1251,19 +1291,19 @@ int h, w, s;
 {
     Pixmap pix;
     int lo, ln, i, j, is, js;
-    char *new, *n, *o, *so, mo, mn;
+    char *new, *n, *o, *so, mo = 0, mn = 0;
     
-    lo = (w + 7) / 8;
-    ln = (w*s + 7) / 8;
-    new = (char *) malloc(s * h* ln * sizeof(char));
-    if (!new)
+    lo = computed_rowbytes(w, 1);
+    ln = computed_rowbytes(w, s);
+    new = (char *) malloc(s * h * ln * sizeof(char));
+    if (new == NULL)
       return None;
     
     for (i = 0; i < s * h * ln; i++)
       new[i] = '\0';
     
-    n = new-1;
-    o = data-1;
+    n = new - 1;
+    o = data - 1;
     
     for (i = 0; i < h; i++) {
 	so = o;
@@ -1277,7 +1317,7 @@ int h, w, s;
 		    mo <<= 1;
 		}
 		for (js = 0; js < s; js++) {
-		    if (!((s*j+js) & 7)) {
+		    if (!((s * j + js) & 7)) {
 			n++;
 			mn = '\001';
 		    } else {
@@ -1289,13 +1329,11 @@ int h, w, s;
 	    }
 	}
     }
-    
-    pix = XCreateBitmapFromData(dpy, rootwin, new, s*w, s*h);
+    pix = XCreateBitmapFromData(dpy, rootwin, new, s * w, s * h);
     free(new); 
     return pix;      
 }
 
-
 Pixmap 
 magnify_colrpix (img, s, bg_pix, mask)
 Image *img;
@@ -1303,94 +1341,66 @@ int s;
 Pixel bg_pix;
 Pixmap mask;
 {
-    int i, j, r, ri, rc, c, l,
+    int r, ri, rc, c,
     rsize, rowbytesize, bytesize, rmask;
-    char *dp, *rp, *rpsave, *data;
+    char *rp;
     Pixmap pixmap;
-    Pixel index[256], pixel;
-    XImage *ximage;
-    GC gc;
+    GC bgc, cgc[256];
     X11Image *ximg = (X11Image *) img->hook;
     
     if (!ximg || !img->rawcolrdata)
       return None;
     
-    /* find reverse index->pixel mapping */
-    for (c=0; c<256; c++) {
-	index[c] = XBlackPixel(dpy,screen);
+    pixmap = XCreatePixmap(dpy, rootwin, img->w * s, img->h * s, depth);
+    if (pixmap == None)
+      return None;
+
+    /* create the GC's */
+    /* background */
+    bgc = XCreateGC(dpy, pixmap, 0, NULL);
+    XSetClipOrigin(dpy, bgc, 0, 0);
+    XSetForeground(dpy, bgc, bg_pix);
+    XFillRectangle(dpy, pixmap, bgc, 0, 0, img->w*s, img->h*s);
+    for (c = 0; c < 256; c++) {
+	cgc[c] = bgc;
     }
-    for (c=0; c<img->numcolors; c++) {
-	index[img->rawpalette[4*c]] = ximg->colpix[c];
+    /* foreground colors */
+    for (c = 0; c < img->numcolors; c++) {
+	cgc[img->rawpalette[4*c]] = XCreateGC(dpy, pixmap, 0, NULL);
+	XSetClipOrigin(dpy, cgc[img->rawpalette[4*c]], 0, 0);
+	XSetClipMask(dpy, cgc[img->rawpalette[4*c]], mask);
+	XSetForeground(dpy, cgc[img->rawpalette[4*c]], ximg->colpix[c]);
     }
-    
-    /* make color data */
+
+    /* draw the image */
     rsize = img->pixelsize;
     rmask = (1<<img->pixelsize) - 1;
     rowbytesize = img->w*s * depth/8;
     bytesize = rowbytesize * img->h*s;
-    data = xmalloc(bytesize*sizeof(char));
-    dp = data;
     rp = img->rawcolrdata;
     for (r=0; r<img->h; r++) {
-	rpsave = rp;
-	for (j=0; j<s; j++) {
-	    rp = rpsave;
-	    ri = 8 - img->pixelsize;
-	    for (c=0; c<img->w; c++) {
-		rc = ((int) (*rp>>ri)) & rmask;
-		if (ri) {
-		    ri -= img->pixelsize;
-		} else {
-		    ri = 8 - img->pixelsize;
-		    rp++;
-		}
-		pixel = index[rc];
-		for (i = 0; i<s; i++) {
-		    for (l=depth-8; l>=0; l-=8) {
-			*dp = (pixel>>l) & 0xff;
-			dp++;
-		    }
-		}
+	ri = 8 - img->pixelsize;
+	for (c=0; c<img->w; c++) {
+	    rc = ((int) (*rp>>ri)) & rmask;
+	    if (ri) {
+	      ri -= img->pixelsize;
+	    } else {
+	      ri = 8 - img->pixelsize;
+	      rp++;
 	    }
+
+	    XFillRectangle(dpy, pixmap, cgc[rc], s*c, s*r, s, s);
 	    if ((img->pixelsize*img->w)%8) {
 		rp++;
 	    }
 	}
     }
-    
-    /* convert to XImage */
-    ximage = XCreateImage(dpy, DefaultVisual(dpy,screen), depth,
-			  ZPixmap, 0, data, img->w*s, img->h*s,
-			  8, rowbytesize);
-    if (!ximage) {
-	free(data);
-	return None;
+    /* clean up */
+    XFreeGC(dpy, bgc);
+    for (c = 0; c < img->numcolors; c++) {
+	XFreeGC(dpy, cgc[img->rawpalette[4*c]]);
     }
-    ximage->byte_order = MSBFirst;
-    ximage->bitmap_bit_order = MSBFirst;
-    
-    /* and finally to Pixmap */
-    pixmap = XCreatePixmap(dpy, rootwin, ximage->width, ximage->height, depth);
-    if (!pixmap)  pixmap = None;
-    if (pixmap == None) {
-	XDestroyImage(ximage);
-	/* XDestroyImage also frees data */
-	return None;
-    }
-    gc = XCreateGC(dpy, pixmap, 0, NULL);
-    
-    /* background */
-    XSetClipOrigin(dpy, gc, 0, 0);
-    XSetForeground(dpy, gc, bg_pix);
-    XFillRectangle(dpy, pixmap, gc, 0, 0, ximage->width, ximage->height);
-    
-    XSetClipMask(dpy, gc, mask);
-    XPutImage(dpy, pixmap, gc, ximage, 0, 0, 0, 0,
-	      ximage->width, ximage->height);
-    XFreeGC(dpy, gc);
-    XDestroyImage(ximage);
-    /* XDestroyImage also frees data */
-    
+
     return pixmap;
 }
 
@@ -1516,7 +1526,7 @@ Widget w;
     if (!w)  return 0;
     i = shell_index(XtParent(XtParent(w)));
     fms = &family[i];
-    ims = find_ims(fms,w);
+    ims = find_ims(fms, w);
     if (!ims)  return 0;
     img = find_image(images[i], ims->w, ims->h);
     if (!img)  return 0;
