@@ -1,5 +1,5 @@
 /* Utilities needed for X11 utility programs.
-   Copyright (C) 1993, 1994, 1995, 1996 Stanley T. Shebs.
+   Copyright (C) 1993-1997 Stanley T. Shebs.
 
 Xconq is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,22 +23,28 @@ any later version.  See the file COPYING.  */
 #include <X11/xpm.h>
 #endif
 
+extern void syntax_error PARAMS ((Obj *x, char *msg));
+
 extern int numimages;
+
 char *outdirname = NULL;
 char *imflib = "";
 
 LibraryPath *xconq_libs;
 
-/* this is needed to resolve
-   extern char spbuf[];
-   declared in kernel/unix.c */
 char spbuf[BUFSIZE];
 
+char readerrbuf[1000];
+
+/* This is needed because error handling refers to it. */
+
 void
 close_displays()
 {
 }
 
+/* This is needed because error handling refers to it. */
+
 int
 write_entire_game_state(fname)
 char *fname;
@@ -53,6 +59,15 @@ announce_read_progress()
 {
 }
 
+void  
+syntax_error(x, msg)
+Obj *x;
+char *msg;
+{
+    sprintlisp(readerrbuf, x, BUFSIZE);
+    init_warning("syntax error in `%s' - %s", readerrbuf, msg);
+}
+
 void
 low_init_error(str)
 char *str;
@@ -82,9 +97,6 @@ char *str;
     exit(1);
 }
 
-/* Runtime warnings are for when it's important to bug the players,
-   usually a problem with Xconq or a game design. */
-
 void
 low_run_warning(str)
 char *str;
@@ -142,7 +154,7 @@ prealloc_debug()
 {
 }
 
-/* read/write images in X format. */
+/* Read/write images in X format. */
 
 Image *
 read_xbm_file(filename, imf, hook)
@@ -157,11 +169,10 @@ readimf_hook hook;
     char *rawdata;
     Image *img;
 
-    if (!imf)
+    if (imf == NULL)
       return NULL;
-    if (!(ifp = fopen(filename, "r")))
+    if ((ifp = fopen(filename, "r")) == NULL)
       return NULL;
-
     if (XmuReadBitmapData(ifp, &ucols, &urows, &urawdata, NULL, NULL)
 	!= BitmapSuccess)
       return NULL;
@@ -170,13 +181,13 @@ readimf_hook hook;
     rawdata = (char *) urawdata;
     fclose(ifp);
     img = get_img(imf, cols, rows);
-    if (!img)
+    if (img == NULL)
       return NULL;
     if (hook) {
-	/* force image re-generation */
+	/* Force image recalculation. */
 	img->monodata = img->colrdata = img->maskdata = lispnil;
     }
-    rowbytes = (cols + 7) / 8;
+    rowbytes = computed_rowbytes(cols, 1);
     if (filename[strlen(filename)-1] == 'm') {
 	img->rawmaskdata = xmalloc(rows * rowbytes);
 	memcpy(img->rawmaskdata, rawdata, rows * rowbytes);
@@ -194,61 +205,63 @@ ImageFamily *imf;
 readimf_hook hook;
 {
 #ifdef HAVE_XPM
-    int c, numbytes, *dp, r, ri, cols, rows;
+    int c, numbytes, *dp, r, ri, cols, rows, rowbytes;
     char *rp, rmask;
     XpmImage xpmimage;
     XpmInfo info;
     Image *img;
 
-    if (!imf)
+    if (imf == NULL)
+      return NULL;
+    if (XpmReadFileToXpmImage(filename, &xpmimage, &info) != XpmSuccess)
       return NULL;
-    if (XpmReadFileToXpmImage(filename, &xpmimage, &info) != XpmSuccess) {
-	return NULL;
-    }
     cols = xpmimage.width;
     rows = xpmimage.height;
     img = get_img(imf, cols, rows);
-    if (!img)  return NULL;
+    if (img == NULL)
+      return NULL;
     if (hook) {
-	/* force image re-generation */
+	/* Force image recalculation. */
 	img->monodata = img->colrdata = img->maskdata = lispnil;
     }
     img->actualw = cols;
     img->actualh = rows;
     img->numcolors = xpmimage.ncolors;
-    if (xpmimage.ncolors>256) {
-	low_run_warning("Image with more then 256 colors not supported\n");
+    if (xpmimage.ncolors > 256) {
+	run_warning("Image with more then 256 colors not supported");
 	return NULL;
-    } else if (xpmimage.ncolors>16) {
+    } else if (xpmimage.ncolors > 16) {
 	img->pixelsize = 8;
-    } else if (xpmimage.ncolors>4) {
+    } else if (xpmimage.ncolors > 4) {
 	img->pixelsize = 4;
-    } else if (xpmimage.ncolors>2) {
+    } else if (xpmimage.ncolors > 2) {
 	img->pixelsize = 2;
     } else {
 	img->pixelsize = 1;
     }
-    img->rowbytes = (img->w*img->pixelsize + 7) / 8;
-    img->rawpalette = (int *) xmalloc(img->numcolors*4*sizeof(int));
-    for (c=0; c<img->numcolors; c++) {
-	img->rawpalette[4*c] = c;
+    rowbytes = computed_rowbytes(img->w, img->pixelsize);
+    img->rawpalette = (int *) xmalloc(img->numcolors * 4 * sizeof(int));
+    for (c = 0; c < img->numcolors; c++) {
+	img->rawpalette[4 * c] = c;
 	parse_xpm_colors(xpmimage.colorTable[c].c_color,
-			 &img->rawpalette[4*c+1],
-			 &img->rawpalette[4*c+2],
-			 &img->rawpalette[4*c+3]);
+			 &img->rawpalette[4 * c + 1],
+			 &img->rawpalette[4 * c + 2],
+			 &img->rawpalette[4 * c + 3]);
     }
-    /* this seems to be Stan's default ;-) */
-    img->rawpalette[4*(img->numcolors-1)] = (1<<img->pixelsize)-1;
-    numbytes = img->h * img->rowbytes;
+    /* This seems to be Stan's default ;-) */
+    /* (Actually, this is a Mac-ism, that black is given all 1-bits,
+        even when there are less than 2^n colors in an image -sts) */
+    img->rawpalette[4 * (img->numcolors - 1)] = (1 << img->pixelsize) - 1;
+    numbytes = img->h * rowbytes;
     img->rawcolrdata = xmalloc(numbytes);
     memset(img->rawcolrdata, '\0', numbytes);
     rp = img->rawcolrdata;
     dp = (int *) xpmimage.data;
-    rmask = (1<<img->pixelsize) - 1;
-    for (r=0; r<img->h; r++) {
+    rmask = (1 << img->pixelsize) - 1;
+    for (r = 0; r < img->h; r++) {
 	ri = 8 - img->pixelsize;
-	for (c=0; c<img->w; c++) {
-	    *rp |= (img->rawpalette[4*(*dp)] & rmask)<<ri;
+	for (c = 0; c < img->w; c++) {
+	    *rp |= (img->rawpalette[4*(*dp)] & rmask) << ri;
 	    dp++;
 	    if (ri) {
 		ri -= img->pixelsize;
@@ -257,19 +270,19 @@ readimf_hook hook;
 		rp++;
 	    }
 	}
-	if ((img->pixelsize*img->w)%8) {
+	if ((img->pixelsize * img->w) % 8) {
 	    rp++;
 	}
     }
-
     return img;
 #else
     return NULL;
 #endif
 }
 
-/* half-witted replacement for XParseColor, which we can't use
-   since we could have no display open */
+/* Half-witted replacement for XParseColor, which we can't use since
+   we could have no display open. */
+
 void
 parse_xpm_colors(name, r, g, b)
 char *name;
@@ -281,37 +294,35 @@ int *b;
     char xpmcbuf[32];
 
     /* sometimes "pixmap" comes up with these: */
-    if (!strcmp(name,"white") || !strcmp(name,"White")) {
+    if (strcmp(name, "white") == 0 || strcmp(name, "White") == 0) {
 	*r = *g = *b = 0xffff;
 	return;
     }
-    if (!strcmp(name,"black") || !strcmp(name,"Black")) {
+    if (strcmp(name, "black") == 0 || strcmp(name, "Black") == 0) {
 	*r = *g = *b = 0;
 	return;
     }
-
-
-    if (name[0]!='#') {
-	fprintf(stderr, "Error parsing color %s\n", name);
+    if (name[0] != '#') {
+	run_warning("Error parsing color %s", name);
 	return;
     }
     name++;
     n = strlen(name);
-    if (n%3) {
-	fprintf(stderr, "Error parsing color %s\n", name);
+    if (n % 3) {
+	run_warning("Error parsing color %s", name);
 	return;
     }
     n /= 3;
     sprintf(xpmcbuf, "%%%dx%%%dx%%%dx", n, n, n);
     q = sscanf(name, xpmcbuf, r, g, b);
-    if (q!=3) {
-	fprintf(stderr, "Error parsing color %s\n", name);
+    if (q != 3) {
+	run_warning("Error parsing color %s", name);
 	return;
     }
-    if (n<4) {
-	*r <<= 4*(4-n);
-	*g <<= 4*(4-n);
-	*b <<= 4*(4-n);
+    if (n < 4) {
+	*r <<= 4 * (4 - n);
+	*g <<= 4 * (4 - n);
+	*b <<= 4 * (4 - n);
     }
 }
 
@@ -325,13 +336,14 @@ int mkfiles;
     FILE *fp;
     Image *img;
 
-    if (imf == NULL || imf->name == NULL) return;
+    if (imf == NULL || imf->name == NULL)
+      return;
     for (img = imf->images; img != NULL; img = img->next) {
 	w = img->w;  h = img->h;
 	/* unified size marker in extension.  Massimo */
 	sprintf(ext, ".%dx%d", w, h);
 	if (img->monodata != lispnil && img->rawmonodata == NULL) {
-	    rowbytes = (w + 7) / 8;
+	    rowbytes = computed_rowbytes(w, 1);
 	    numbytes = h * rowbytes;
 	    img->rawmonodata = xmalloc(numbytes);
 	    interp_bytes(img->monodata, numbytes, img->rawmonodata, 0);
@@ -351,7 +363,7 @@ int mkfiles;
 	    }
 	}
 	if (img->maskdata != lispnil && img->rawmaskdata == NULL) {
-	    rowbytes = (w + 7) / 8;
+	    rowbytes = computed_rowbytes(w, 1);
 	    numbytes = h * rowbytes;
 	    img->rawmaskdata = xmalloc(numbytes);
 	    interp_bytes(img->maskdata, numbytes, img->rawmaskdata, 0);
@@ -371,7 +383,7 @@ int mkfiles;
 	    }
 	}
 	if (img->colrdata != lispnil && img->rawcolrdata == NULL) {
-	    rowbytes = (w * img->pixelsize + 7) / 8;
+	    rowbytes = computed_rowbytes(w, img->pixelsize);
 	    numbytes = h * rowbytes;
 	    img->rawcolrdata = xmalloc(numbytes);
 	    interp_bytes(img->colrdata, numbytes, img->rawcolrdata, 0);
@@ -401,16 +413,21 @@ char *name;
 int cols, rows;
 char *data;
 {
-    int row, i = 0, col;
-    int bytesperline = 8;
-    int byte, byte2, j, firstitem = 1;
-    int numbytes = ((cols + 7) / 8) * rows;
+    int row, i, col, bytesperline = 8, byte, byte2, j, firstitem = 1;
+    int numbytes = computed_rowbytes(cols, 1) * rows;
+    char fixedname[BUFSIZE];
 
+    for (i = 0; name[i] != '\0'; ++i) {
+      fixedname[i] = (name[i] == '-' ? '_' : name[i]);
+    }
+    fixedname[i] = '\0';
     reverse_bit_endianness(data, numbytes);
 
-    fprintf(fp, "#define %s_width %d\n", name, cols);
-    fprintf(fp, "#define %s_height %d\n", name, rows);
-    fprintf(fp, "static char %s_bits[] = {", name);
+    fprintf(fp, "#define %s_width %d\n", fixedname, cols);
+    fprintf(fp, "#define %s_height %d\n", fixedname, rows);
+    fprintf(fp, "static char %s_bits[] = {", fixedname);
+
+    i = 0;
     for (row = 0; row < rows; row++) {
 	for (col = 0; col < cols; col++) {
 	    if (col % 8 == 0) {
@@ -441,70 +458,70 @@ char *data;
     reverse_bit_endianness(data, numbytes);
 }
 
+/* Write a pixmap in XPM format.  Note that this does not need to
+   be conditionalized on HAVE_XPM, does not use any XPM functions. */
+
 void
 write_xpm_file(fp, name, img)
 FILE *fp;
 char *name;
 Image *img;
 {
-    int r, ri, rc, c, numcols, rmask;
+    int i, r, ri, rc, c, numcols, rmask, col, red, grn, blu;
     char pch[256], *rp;
     int idx[256];
-    Obj *palette, *color;
+    Obj *restpal;
+    char fixedname[BUFSIZE];
 
-    if (img->palette==lispnil && !(img->rawpalette && img->numcolors))
+    if (img->palette == lispnil && !(img->rawpalette && img->numcolors))
       return;
 
-    if (img->palette==lispnil) {
+    /* C-ify the name. */
+    for (i = 0; name[i] != '\0'; ++i) {
+	fixedname[i] = (name[i] == '-' ? '_' : name[i]);
+    }
+    fixedname[i] = '\0';
+    if (img->palette == lispnil) {
 	numcols = img->numcolors;
     } else {
-	c = 0;
-	for (palette=img->palette; palette != lispnil; palette = cdr(palette)) {
-	    c++;
-	}
-	numcols = c;
+	numcols = length(img->palette);
     }
-
     fprintf(fp, "/* XPM */\n");
-    fprintf(fp, "static char * %s [] = {\n", name);
+    fprintf(fp, "static char * %s [] = {\n", fixedname);
     fprintf(fp, "\"%d %d %d %d\",\n", img->w, img->h, numcols, 1);
-
-    for (c=0; c<numcols; c++) {
-	if (c<26) {
+    for (c = 0; c < numcols; c++) {
+	if (c < 26) {
 	    pch[c] = 'a' + c;
-	} else if (c<52) {
-	    pch[c] = 'A' + c-26;
-	} else if (c<62) {
-	    pch[c] = '0' + c-52;
+	} else if (c < 52) {
+	    pch[c] = 'A' + c - 26;
+	} else if (c < 62) {
+	    pch[c] = '0' + c - 52;
 	} else {
-	    pch[c] = '#' + c-62;
+	    pch[c] = '#' + c - 62;
 	}	
     }  
- 
-    if (img->palette==lispnil) {
-	for (c=0; c<numcols; c++) {
-	    fprintf(fp, "\"%c\tc #%4.4x%4.4x%4.4x\",\n", pch[c], 
-		    img->rawpalette[4*c+1], img->rawpalette[4*c+2],
-		    img->rawpalette[4*c+3]);
-	    idx[img->rawpalette[4*c]] = c;
+    if (img->palette == lispnil) {
+	for (c = 0; c < numcols; c++) {
+	    fprintf(fp, "\"%c\tc #%4.4x%4.4x%4.4x\",\n",
+		    pch[c], 
+		    img->rawpalette[4 * c + 1],
+		    img->rawpalette[4 * c + 2],
+		    img->rawpalette[4 * c + 3]);
+	    idx[img->rawpalette[4 * c]] = c;
 	}
     } else {
 	c = 0;
-	for (palette=img->palette; palette != lispnil; palette = cdr(palette)) {
-	    color = car(palette);
-	    fprintf(fp, "\"%c\tc #%4.4x%4.4x%4.4x\",\n", pch[c], 
-		    c_number(car(cdr(color))),
-		    c_number(car(cdr(cdr(color)))),
-		    c_number(car(cdr(cdr(cdr(color)))))
-		    );
-	    idx[c_number(car(color))] = c;
+	for_all_list (img->palette, restpal) {
+	    parse_lisp_palette_entry(restpal, &col, &red, &grn, &blu);
+	    fprintf(fp, "\"%c\tc #%4.4x%4.4x%4.4x\",\n",
+		    pch[c], red, grn, blu);
+	    idx[col] = c;
 	    c++;
 	}
     }
-  
     if (numcols > 74) {
-	fprintf(stderr, "write_xpm_file not implemented for %d colors\n",
-		numcols);
+	run_warning("write_xpm_file not implemented for %d colors",
+		    numcols);
 	return;
     }
 
@@ -545,7 +562,6 @@ char *rawname;
 
     b = copy_string(rawname);
     lastpos = strlen(b) - 1;
-
     /* Remove leading path. */
     for (a = b + lastpos - 1; a > b; --a) {
 	if (*a == '/') {
@@ -553,12 +569,10 @@ char *rawname;
 	    break;
 	}
     }
-
     /* Remove trailing extension. */
     if ((a = strchr(b, '.'))) {
 	*a = '\0';
     }
-
     return b;
 }
 
@@ -571,7 +585,7 @@ readimf_hook hook;
     char *ext;
     int rc;
 
-    /* find extension */
+    /* Find the extension. */
     for (ext = filename + strlen(filename) - 1; ext > filename; ext--) {
 	if (*ext == '.') {
 	    ext++;
@@ -580,20 +594,19 @@ readimf_hook hook;
     }
     if (ext == filename)
       ext = "";
-
 #ifdef HAVE_XPM
-    if (!strcmp(ext, "xpm")) {
+    if (strcmp(ext, "xpm") == 0) {
 	/* try a XPM file */
 	imf = get_imf(find_imf_name(filename));
-	if (!imf)
+	if (imf == NULL)
 	  return 0;
 	rc = !!read_xpm_file(filename, imf, hook);
 	if (hook && rc)
-	  hook(imf,0);
+	  hook(imf, 0);
 	return rc;
     }
 #endif /* HAVE_XPM */
-    if (!strcmp(ext,"b") || !strcmp(ext,"m")) {
+    if (strcmp(ext, "b") == 0 || strcmp(ext, "m") == 0) {
 	/* try a XBM file */
 	imf = get_imf(find_imf_name(filename));
 	if (!imf)
@@ -617,8 +630,8 @@ ImageFamily *imf;
 
     if (imf == NULL)
       return;
-    for (img = imf->images; img; img = img->next) {
-	numbytes = img->h * ((img->w + 7) / 8);
+    for (img = imf->images; img != NULL; img = img->next) {
+	numbytes = img->h * computed_rowbytes(img->w, 1);
 	if (img->rawmonodata)
 	  reverse_bit_endianness(img->rawmonodata, numbytes);
 	if (img->rawmaskdata)
